#{{{ copyright and setup 

#   FEAT - FMRI Expert Analysis Tool
#      (previously FMRIB's Easy Analysis Tool)
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2002 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   http://www.fmrib.ox.ac.uk/fsl
#   fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   
#   LICENCE
#   
#   FMRIB Software Library, Release 3.0 (c) 2002, The University of Oxford
#   (the "Software")
#   
#   The Software remains the property of the University of Oxford ("the
#   University").
#   
#   The Software is distributed "AS IS" under this Licence solely for
#   non-commercial use in the hope that it will be useful, but in order
#   that the University as a charitable foundation protects its assets for
#   the benefit of its educational and research purposes, the University
#   makes clear that no condition is made or to be implied, nor is any
#   warranty given or to be implied, as to the accuracy of the Software,
#   or that it will be suitable for any particular purpose or for use
#   under any specific conditions. Furthermore, the University disclaims
#   all responsibility for the use which is made of the Software. It
#   further disclaims any liability for the outcomes arising from using
#   the Software.
#   
#   The Licensee agrees to indemnify the University and hold the
#   University harmless from and against any and all claims, damages and
#   liabilities asserted by third parties (including claims for
#   negligence) which arise directly or indirectly from the use of the
#   Software or the sale of any products based on the Software.
#   
#   No part of the Software may be reproduced, modified, transmitted or
#   transferred in any form or by any means, electronic or mechanical,
#   without the express permission of the University. The permission of
#   the University is not required if the said reproduction, modification,
#   transmission or transference is done without financial return, the
#   conditions of this Licence are imposed upon the receiver of the
#   product, and all original and amended source code is included in any
#   transmitted product. You may be held legally responsible for any
#   copyright infringement that is caused or encouraged by your failure to
#   abide by these terms and conditions.
#   
#   You are not permitted under this Licence to use this Software
#   commercially. Use for which any financial return is received shall be
#   defined as commercial use, and includes (1) integration of all or part
#   of the source code or the Software into a product for sale or license
#   by or on behalf of Licensee to third parties or (2) use of the
#   Software or any derivative of it for research with the final aim of
#   developing software products for sale or license to a third party or
#   (3) use of the Software or any derivative of it for research with the
#   final aim of developing non-software products for sale or license to a
#   third party, or (4) use of the Software to provide any service to an
#   external organisation for which payment is received. If you are
#   interested in using the Software commercially, please contact Isis
#   Innovation Limited ("Isis"), the technology transfer company of the
#   University, to negotiate a licence. Contact details are:
#   innovation@isis.ox.ac.uk quoting reference DE/1112.

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

set VERSION 5.00

#}}}

##### FEAT GUI #####
#{{{ feat5 GUI

proc feat5 { w } {
    global fmri PXHOME FSLDIR USER VERSION INMEDX feat_files unwarp_files initial_highres_files highres_files VARS argc argv PWD gui_ext HOME
 
    #{{{ main window

toplevel $w

set fmri(version) $VERSION

set fmri(feat_filename) [ exec ${FSLDIR}/bin/tmpnam /tmp/feat ].fsf

wm title      $w "FEAT - FMRI Expert Analysis Tool - v$fmri(version)"
wm iconname   $w "FEAT [ expr int($fmri(version)) ]"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

tixBalloon $w.bhelp -initwait 5000 -options { background #b0ffb0 }

set fmri(font)      [ tix option get font ]
set fmri(bold_font) [ tix option get bold_font ]

#}}}

    #{{{ load ~/.fsl file after setting-up relevant variables

set fmri(help_yn) 1

if { [ file exists ${HOME}/.fsl ] } {
    source ${HOME}/.fsl
}

#}}}
    #{{{ mode

frame $w.mode

global fmri(level)
tixOptionMenu $w.mode.level -variable fmri(level)
$w.mode.level add command 1 -label "First-level analysis"
$w.mode.level add command 2 -label "Higher-level analysis"

$w.bhelp bind $w.mode.level -msg "Use \"First-level analysis\" for analysing each session's data -
i.e. the time-series analysis of the raw 4D FMRI data.

Use \"Higher-level analysis\" for combining first-level analyses. You
can use this hierarchically - for example at second-level to analyse
across several sessions and then at third-level to analyse across
several subjects."

global fmri(analysis)
tixOptionMenu $w.mode.analysis -variable fmri(analysis)
$w.mode.analysis add command 7 -label "Full analysis"
$w.mode.analysis add command 1 -label "Pre-stats"
$w.mode.analysis add command 3 -label "Pre-stats + Stats"
$w.mode.analysis add command 2 -label "                     Stats"
$w.mode.analysis add command 6 -label "                     Stats + Post-stats"
$w.mode.analysis add command 4 -label "                                  Post-stats"
$w.mode.analysis add command 0 -label "Registration only"

$w.bhelp bind $w.mode.analysis -msg "You can run a full analysis - Pre-Stats; Stats; Post-stats;
Registration - or a (sensible) subset of these options.

If you select \"Post-stats\" or \"Registration only\", you will need
to select a FEAT directory (or directories) instead of starting with
4D image data; the results already produced in those FEAT directories
will then be used as appropriate.

Note that if you want to run only \"Post-stats\", you must select the
FEAT directory/directories before editing the contrasts or
thresholding parameters, as these will get reset on selection of the
FEAT directory/directories."

pack $w.mode.level $w.mode.analysis -in $w.mode -side left -anchor w

#}}}
    #{{{ main notebook

tixNoteBook $w.nb
$w.nb add misc      -label "Misc"
$w.nb add data      -label "Data"
$w.nb add filtering -label "Pre-stats"
$w.nb add stats     -label "Stats"
$w.nb add poststats -label "Post-stats"
$w.nb add reg       -label "Registration"

#{{{ Misc

set fmri(miscf) [ $w.nb subwidget misc ]

#{{{ delay

set fmri(delay) 0

tixControl $w.delay -label "Delay before starting (hours) " \
	-variable fmri(delay) -step 1 -min 0 -selectmode immediate

$w.bhelp bind $w.delay -msg "If you are using FEAT to carry out multiple analyses, you might want
to do this overnight, to reduce the load on your computer. You can use
this delay feature to tell FEAT how long to wait before starting the
analyses." 

#}}}
#{{{ balloon help

checkbutton $w.help -variable fmri(help_yn) -text "Balloon help" -font $fmri(bold_font) -command "feat5:updatehelp $w" -justify right

$w.bhelp bind $w.help -msg "And don't expect this message to appear whilst you've turned this
option off!"

feat5:updatehelp $w

#}}}
#{{{ brain threshold

set fmri(brain_thresh) 10
tixControl $w.brain_thresh -label "Brain/background threshold, % " \
	-variable fmri(brain_thresh) -step 1 -min 0 -max 100 -selectmode immediate

$w.bhelp bind $w.brain_thresh -msg "This is automatically calculated, as a % of the maximum input image
intensity. It is used in intensity normalisation, brain mask
generation and various other places in the FEAT analysis."

#}}}
#{{{ relative filenames for custom timings and highres images

set fmri(relative_yn) 0

checkbutton $w.relative -variable fmri(relative_yn) -text "Use relative filenames for multiple analyses" -font $fmri(bold_font) -command "feat5:updatereg $w"

$w.bhelp bind $w.relative -msg "blah...."

#}}}
#{{{ new directory if re-thresholding

set fmri(newdir_yn) 0
tixOptionMenu $w.newdir_yn -variable fmri(newdir_yn)
$w.newdir_yn add command 0 -label "Overwrite original post-stats results"
$w.newdir_yn add command 1 -label "Copy original FEAT directory for new Post-stats / Registration"

$w.bhelp bind $w.newdir_yn -msg "If you are just re-running post-stats or registration, you can either
choose to overwrite the original post-stats and registration results
or create a complete copy of the original FEAT directory, with the new
results in it."

#}}}

#pack $w.delay $w.help $w.brain_thresh $w.relative -in $fmri(miscf) -anchor w -side top -padx 5 -pady 1
pack $w.delay $w.help $w.brain_thresh -in $fmri(miscf) -anchor w -side top -padx 5 -pady 1

#}}}
#{{{ Data

set fmri(dataf) [ $w.nb subwidget data ]

#{{{ input type for higher-level

global fmri(inputtype)

set fmri(inputtype) 1

tixOptionMenu $w.inputtype -variable fmri(inputtype)
$w.inputtype add command 1 -label "Inputs are lower-level FEAT directories"
$w.inputtype add command 2 -label "Inputs are cope images"

$w.inputtype configure  -command "feat5:updateselect $w"

$w.bhelp bind $w.inputtype -msg "You can choose whether your higher-level design matrix will be applied
to a set of lower-level COPE images or a set of lower-level FEAT
directories. In the latter, more normal, case, each contrast in the
lower-level FEAT directories will have the higher-level model applied,
each resulting its own FEAT directory within the new group FEAT directory."

#}}}

#{{{ multiple analyses

frame $w.multiple

set fmri(multiple) 1

tixControl $w.multiple.number -label "Number of analyses " \
	-variable fmri(multiple) -step 1 -min 1 -selectmode immediate \
	-options { entry.width 3 }

button $w.multiple.setup -text "Select 4D data" -command "feat5:multiple_select $w 0 \"Select input data\" "

pack $w.multiple.number $w.multiple.setup -in $w.multiple -side left -padx 5

set medxmessage ""
if { $INMEDX } {
    $w.multiple.number configure -min 0 -label "File-based first-level analyses "
    if { [ what_type 0 ] == 4 } {
	MxGetCurrentPage currentpage
	MxGetPageProperties $currentpage Props
	set fmri(groupname) [ keylget Props Name ]
	set fmri(multiple) 0
    }

    label $w.groupname

    set medxmessage "
If the \"Number of analyses\" is set to 0 then FEAT will use the
selected group page inside the current MEDx folder for analysis,
instead of a file on disk.
"
}

$w.bhelp bind $w.multiple -msg "Set the filename of the 4D input image (e.g. /home/sibelius/func.hdr).
You can setup FEAT to process many input images, one after another, as
long as they all require exactly the same analysis. Each one will
generate its own FEAT directory, the name of which is based on the
input data's filename.
$medxmessage
Alternatively, if you are running either just \"Post-stats\" or
\"Registration only\", or running \"Higher-level analysis\", the
selection of 4D data changes to the selection of FEAT directories.
Note that in this case you should select the FEAT directories before
setting up anything else in FEAT (such as changing the
thresholds). This is because quite a lot of FEAT settings are loaded
from the first selected FEAT directory, possibly over-writing any
settings which you wish to change!"

#}}}

#{{{ output directory

set fmri(outputdir) ""

FSLFileEntry $w.outputdir \
	-variable fmri(outputdir) \
	-directory $PWD \
	-label " Output directory" \
	-title "Name the output directory" \
	-width 35 \
	-filterhist VARS(history)

$w.bhelp bind $w.outputdir -msg "If this is left blank, the output FEAT directory name is derived from
the input data name. (For higher-level analysis, the output name is
derived from the first lower-level FEAT directory selected as input.)

If, however, you wish to explicitly choose the output FEAT directory
name, for example, so that you can include in the name a hint about
the particular analysis that was carried out, you can set this
here.

This output directory naming behaviour is modified if you are setting
up multiple first-level analyses, where you are selecting multiple
input data sets and will end up with multiple output FEAT
directories. In this case, whatever you enter here will be used and
appended to what would have been the default output directory name if
you had entered nothing. For example, if you are setting up 3 analyses
with input data names \"/home/neo/fmri1.hdr\", \"/home/neo/fmri2.hdr\"
and \"/home/neo/fmri3.hdr\", and set the output name to \"analysisA\",
the output directories will end up as
\"/home/neo/fmri1_analysisA.feat\" etc."

#}}}

frame $w.datamain
#{{{ npts & ndelete

frame $w.nptsndelete

#{{{ npts

set fmri(npts) 0

tixControl $w.npts -label "Total volumes " -variable fmri(npts) \
	-step 1 -min 0 -selectmode immediate -options { entry.width 4 }

$w.bhelp bind $w.npts -msg "The number of FMRI volumes in the time series, including any initial
volumes that you wish to delete. This will get set automatically once
valid input data has been selected.

The reason for allowing you to set this number by hand before
selecting data is so that you can setup and view a model without
having any data, for experimental planning purposes etc."

#}}}
#{{{ ndelete

set fmri(ndelete) 0

tixControl $w.ndelete -label "       Delete volumes " -variable fmri(ndelete) \
	-step 1 -min 0 -selectmode immediate -options { entry.width 3 }

$w.bhelp bind $w.ndelete -msg "The number of initial FMRI volumes to delete before any further
processing. You should have decided on this number when the scans were
acquired.  Typically your experiment would have begun after these
initial scans (sometimes called \"dummy scans\"). These should be the
volumes that are not wanted because steady-state imaging has not yet
been reached - typically two or three volumes. These volumes are
deleted as soon as FEAT is started, so all 4D data sets produced by
FEAT will not contain the deleted volumes.

Note that \"Delete volumes\" should not be used to correct for the
time lag between stimulation and the measured response - this is
corrected for in the design matrix by convolving the input stimulation
waveform with a blurring-and-delaying haemodynamic response function.

Most importantly, remember when setting up the design matrix, that the
timings in the design matrix start at t=0 seconds, and this
corresponds to the start of the first image taken after the deleted
scans. In other words, the design matrix starts AFTER the \"deleted
scans\" have been deleted."

#}}}

pack $w.npts $w.ndelete -in $w.nptsndelete -side left

#}}}
#{{{ TR & highpass

frame $w.trparadigm_hp

#{{{ TR

set fmri(tr) 3.0
tixControl $w.tr -label "TR (s) " -variable fmri(tr) \
	-step 0.25 -min 0.001 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.tr -msg "The time (in seconds) between scanning successive FMRI volumes."

#}}}
#{{{ High pass

set fmri(paradigm_hp) 100
tixControl $w.paradigm_hp -label "     High pass filter cutoff (s) " \
	-variable fmri(paradigm_hp) -step 5 -min 1 -selectmode immediate -options { entry.width 5 }

$w.bhelp bind $w.paradigm_hp -msg "The high pass frequency cutoff point (seconds), that is, the longest
temporal period that you will allow.

A sensible setting in the case of an ABAB or ABACABAC type block
design is the (A+B) or (A+B+A+C) total cycle time.

For event-related designs the rule is not so simple, but in general
the cutoff can typically be reduced at least to 50s.

This value is setup here rather than in Pre-stats because it also
affects the generation of the model; the same high pass filtering is
applied to the model as to the data, to get the best possible match
between the model and data."

#}}}

pack $w.tr $w.paradigm_hp -in $w.trparadigm_hp -side left

#}}}
pack $w.nptsndelete $w.trparadigm_hp -in $w.datamain -side top -padx 5 -pady 3 -anchor w

pack $w.multiple $w.datamain -in $fmri(dataf) -anchor w -side top

#{{{ FSL logo

set graphpic [ image create photo -file ${FSLDIR}/tcl/fsl-logo-tiny.ppm ]
button $w.logo -image $graphpic -command "FmribWebHelp file: ${FSLDIR}/doc/index.html" -borderwidth 0
pack $w.logo -in $fmri(dataf) -anchor e -side bottom -padx 5 -pady 5

#}}}

#}}}
#{{{ Pre-statistics processing

set fmri(filteringf) [ $w.nb subwidget filtering ]

set fmri(filtering_yn) 1

#{{{ slice timing correction

frame $w.st
set fmri(stf) $w.st

set fmri(st) 0

set fmri(st_file) ""
FSLFileEntry $w.st_file \
	-variable fmri(st_file) \
	-pattern "*" \
	-directory $PWD \
	-label "" \
	-title "Select a slice order/timings file" \
	-width 20 \
	-filterhist VARS(history)

tixOptionMenu $w.st.menu -label "Slice timing correction: " -variable fmri(st) -command "feat5:updatest $w"
$w.st.menu add command 0 -label "None"
$w.st.menu add command 1 -label "Regular up (0, 1, 2 ... n-1)"
$w.st.menu add command 2 -label "Regular down (n-1, n-2 ... 0)"
$w.st.menu add command 3 -label "Use slice order file"
$w.st.menu add command 4 -label "Use slice timings file"

pack $w.st.menu -in $fmri(stf) -side top -side left

$w.bhelp bind $w.st -msg "Slice timing correction corrects each voxel's time-series for the fact
that later processing assumes that all slices were acquired exactly
half-way through the relevant volume's acquisition time (TR), whereas
in fact each slice is taken at slightly different times.

Slice timing correction works by using (Hanning-windowed) sinc
interpolation to shift each time-series by an appropriate fraction of
a TR relative to the middle of the TR period. It is necessary to know
in what order the slices were acquired and set the appropriate option
here.

If slices were acquired from the bottom of the brain to the top select 
\"Regular up\".  If slices were acquired from the top of the brain
to the bottom select \"Regular down\".  If slices were not acquired
in regular order (e.g. interleaved slice acquisition) you will need to
use a slice order file or a slice timings file. If a slice order file
is to be used (e.g., to setup interleaved slice orderings), create a
text file with a single number on each line, where the first line
states which slice was acquired first, the second line states which
slice was acquired second, etc. The first slice is numbered 1 not 0."

#}}}
#{{{ motion correction

frame $w.mc
set fmri(mcf) $w.mc

global fmri(mc)
tixOptionMenu $w.mc.menu -label "Motion correction: " -variable fmri(mc) -command "feat5:updatemc $w"
$w.mc.menu add command 0 -label "None"
$w.mc.menu add command 1 -label "MCFLIRT"
set fmri(mc) 1

pack $w.mc.menu -in $fmri(mcf) -side top -side left

$w.bhelp bind $w.mc -msg "You will normally want to apply motion correction; this attempts to
remove the effect of subject head motion during the
experiment. MCFLIRT uses FLIRT (FMRIB's Linear Registration Tool)
tuned to the problem of FMRI motion correction, applying rigid-body
transformations.

Note that there is no \"spin history\" (aka \"correction for movement\")
option with MCFLIRT. This is because this is still a poorly understood
correction method which is under further investigation."

#}}}
#{{{ spin history

set fmri(sh_yn) 0

#frame $w.sh
#
#label $w.sh.label -text "Adjustment for movement"
#
#checkbutton $w.sh.yn -variable fmri(sh_yn)
#
#pack $w.sh.label $w.sh.yn -in $w.sh -padx 5 -side left
#
#$w.bhelp bind $w.sh -msg "blah"

#}}}
#{{{ BET brain extraction

frame $w.bet

label $w.bet.label -text "BET brain extraction"

set fmri(bet_yn) 1
checkbutton $w.bet.yn -variable fmri(bet_yn)

pack $w.bet.label $w.bet.yn -in $w.bet -side left

$w.bhelp bind $w.bet -msg "This uses BET brain extraction to create a brain mask from the first
volume in the FMRI data. This is normally better than simple
intensity-based thresholding for getting rid of unwanted voxels in
FMRI data. Note that here, BET is setup to run in a quite liberal way so that
there is very little danger of removing valid brain voxels.

If the field-of-view of the image (in any direction) is less than 30mm
then BET is turned off by default."

#}}}
#{{{ spatial filtering

set fmri(smooth) 5
tixControl $w.smooth -label "Spatial smoothing FWHM (mm) " \
	-variable fmri(smooth) -step 1 -min 0 -selectmode immediate

$w.bhelp bind $w.smooth -msg "This determines the extent of the spatial smoothing, carried out on
each volume of the FMRI data set separately. This is intended to
reduce noise without reducing valid activation; this is successful as
long as the underlying activation area is larger than the extent of
the smoothing. Thus if you are looking for very small activation areas
then you should maybe reduce smoothing from the default of 5mm, and if
you are looking for larger areas, you can increase it, maybe to 10 or
even 15mm.

To turn off spatial smoothing simply set FWHM to 0."

#}}}
#{{{ intensity normalization

frame $w.norm

label $w.norm.label -text "Intensity normalization"

set fmri(norm_yn) 0
checkbutton $w.norm.yn -variable fmri(norm_yn)

pack $w.norm.label $w.norm.yn -in $w.norm -side left

$w.bhelp bind $w.norm -msg "This forces every FMRI volume to have the same mean intensity. For
each volume it calculates the mean intensity and then scales the
intensity across the whole volume so that the global mean becomes a
preset constant. This step is normally discouraged - hence is turned
off by default. When this step is not carried out, the whole 4D data
set is still normalised by a single scaling factor (\"grand mean
scaling\") - each volume is scaled by the same amount. This is so that
higher-level analyses are valid."

#}}}
#{{{ temporal filtering

frame $w.temp

label $w.temp.label -text "Temporal filtering    "

set fmri(temphp_yn) 1
label $w.temp.hplabel -text "Highpass"
checkbutton $w.temp.hp_yn -variable fmri(temphp_yn)

set fmri(templp_yn) 0
#we've removed the lowpass temporal filtering option as it almost never helps the sensitivity
#label $w.temp.lplabel -text "Lowpass"
#checkbutton $w.temp.lp_yn -variable fmri(templp_yn)

#pack $w.temp.label $w.temp.hplabel $w.temp.hp_yn $w.temp.lplabel $w.temp.lp_yn -in $w.temp -side top -side left
pack $w.temp.label $w.temp.hplabel $w.temp.hp_yn -in $w.temp -side top -side left

$w.bhelp bind $w.temp -msg "\"Highpass temporal filtering\" uses a local fit of a straight line
(Gaussian-weighted within the line to give a smooth response) to
remove low frequency artefacts. This is preferable to sharp rolloff
FIR-based filtering as it does not introduce autocorrelations into the
data.

By default the same filtering will also be applied to the model."

#Lowpass filtering reduces high frequency noise by Gaussian smoothing,
#but also can reduce the strength of the signal of interest,
#particularly for single-event experiments. It is not considered to be
#helpful in general, so is turned off by default. With IRVA analysis,
#Gaussian smoothing lessens the validatity of the inference, so in this
#case it is strongly recommended that lowpass filtering not be carried
#out."

#}}}

pack $fmri(stf) $fmri(mcf) $w.bet $w.smooth $w.norm $w.temp -in $fmri(filteringf) -anchor w -pady 1 -padx 5

#}}}
#{{{ Stats

set fmri(statsf) [ $w.nb subwidget stats ]

set fmri(stats_yn) 1

set fmri(prewhiten_yn) 1
checkbutton $w.prewhiten -text "Use FILM prewhitening" -font $fmri(bold_font) -variable fmri(prewhiten_yn)
$w.bhelp bind $w.prewhiten -msg "For normal first-level time series analysis you should use
prewhitening to make the statistics valid and maximally efficient. For
other data - for example, very long TR (>30s) FMRI data, PET data or
data with very few time points (<50) - this should be turned off."

button $w.wizard -width 20 -text "Simple model setup" -command "feat5:film_wizard $w"
$w.bhelp bind $w.wizard -msg "This lets you easily setup either ABAB... or ABACABAC... designs
(block or single-event).

If you need to adjust the resulting setup, use \"Simple model setup\"
first, then press the \"Full model setup\" button to allow more
detailed setup of the model-based analysis."

button $w.model -width 20 -text "Full model setup" -command "feat5:setup_model $w"
$w.bhelp bind $w.model -msg "This allows complete control of the model-based analysis to be used."

set fmri(w_model) 0
set fmri(constcol) 0
set fmri(con_mode) orig
set fmri(con_mode_old) orig
set fmri(evs_orig) 1
set fmri(evs_real) 1
set fmri(ncon_real) 1
set fmri(ncon_orig) 1
set fmri(nftests_real) 0
set fmri(nftests_orig) 0
set fmri(ngroups) 1

set fmri(mixed_yn) 1	

frame $w.mixed
radiobutton $w.mixed.mixed -font $fmri(bold_font) -text "FLAME Mixed Effects " -value 1 -variable fmri(mixed_yn)
radiobutton $w.mixed.fixed -font $fmri(bold_font) -text "Simple Mixed Effects"  -value 0 -variable fmri(mixed_yn)

$w.bhelp bind $w.mixed -msg "By default you should use FLAME (FMRIB's Local Analysis of Mixed
Effects) modelling and estimation. This is better than simple mixed
effects in various ways, for example by allowing separate modelling of
the variance in different subject groups, and forcing random effects
variance to be non-negative.

However, FLAME is fairly computationally-intensive (typically 45-200
minutes), so if you want a quick (and less statistically accurate)
analysis then you can switch over to simple mixed effects."

set fmri(design_help) "This is a graphical representation of the design matrix and parameter
contrasts.

The bar on the left is a representation of time, which starts at the
top and points downwards. The white marks show the position of every
10th volume in time. The red bar shows the period of the longest
temporal cycle which was passed by the highpass filtering.

The main top part shows the design matrix; time is represented on the
vertical axis and each column is a different (real) explanatory
variable (e.g., stimulus type). Both the red lines and the black-white
images represent the same thing - the variation of the waveform in
time.

Below this is shown the requested contrasts; each row is a different
contrast vector and each column refers to the weighting of the
relevant explanatory variable. Thus each row will result in a Z
statistic image.

If F-tests have been specified, these appear to the right of the
contrasts; each column is a different F-test, with the inclusion of
particular contrasts depicted by filled squares instead of empty
ones."

pack $w.mixed.mixed $w.mixed.fixed -in $w.mixed -side left

#}}}
#{{{ Post-Stats

set fmri(poststatsf) [ $w.nb subwidget poststats ]

set fmri(poststats_yn) 1

#{{{ edit contrasts

button $w.modelcon -text "Edit contrasts" -command "feat5:setup_model $w"

$w.bhelp bind $w.modelcon -msg "This allows setup of contrasts and F-tests, to be run on a previous
analysis."

#}}}
#{{{ pre-thresholding masking

set fmri(threshmask) ""

FSLFileEntry $fmri(poststatsf).threshmask \
	-variable fmri(threshmask) \
	-pattern "*.hdr" \
	-directory "$PWD" \
	-label "Pre-threshold masking" \
	-title "Select mask" \
	-width 30 \
	-filterhist VARS(history)

$w.bhelp bind $fmri(poststatsf).threshmask -msg "If you choose a mask for \"Pre-threshold masking\" then all stats
images will be masked by the chosen mask before thresholding. There
are two reasons why you might want to do this. The first is that you
might want to constrain your search for activation to a particular
area. The second is that in doing so, you are reducing the number of
voxels tested and therefore will make any
multiple-comparison-correction in the thresholding less stringent.

The mask image chosen does not have to be a binary mask - for example,
it can be a thresholded stats image from a previous analysis (in the
same space as the data to be analysed here); only voxels containing
zero in the \"mask\" image will get zeroed in this masking process."

#}}}
#{{{ thresholding

tixLabelFrame $w.thresh -label "Thresholding"
set fmri(lfthresh) [ $w.thresh subwidget frame ]

global fmri(thresh)
tixOptionMenu $w.thresh.menu -variable fmri(thresh)
$w.thresh.menu add command 0 -label "None"
$w.thresh.menu add command 1 -label "Uncorrected"
$w.thresh.menu add command 2 -label "Voxel"
$w.thresh.menu add command 3 -label "Cluster"

set fmri(prob_thresh) 0.01
set fmri(z_thresh) 2.3

tixControl $w.uncorrected -label "Uncorrected voxel P threshold" \
	-variable fmri(prob_thresh) -step 0.005 -min 0 -max 1 -selectmode immediate
tixControl $w.voxel -label "Corrected voxel P threshold" \
	-variable fmri(prob_thresh) -step 0.005 -min 0 -max 1 -selectmode immediate
tixControl $w.prob_thresh -label " Cluster P threshold" \
	-variable fmri(prob_thresh) -step 0.005 -min 0 -max 1 -selectmode immediate
tixControl $w.z_thresh -label "Z threshold" \
	-variable fmri(z_thresh) -step 0.1 -min 0 -selectmode immediate

pack $w.thresh.menu -in $fmri(lfthresh) -side top -padx 5 -side left

$w.bhelp bind $w.thresh -msg "After carrying out the initial statistical test, the resulting Z
statistic image is then normally thresholded to show which voxels or
clusters of voxels are activated at a particular significance level.

If \"Cluster\" thresholding is selected, a Z statistic threshold is
used to define contiguous clusters.  Then each cluster's estimated
significance level (from GRF-theory) is compared with the cluster
probability threshold. Significant clusters are then used to mask the
original Z statistic image for later production of colour blobs. This
method of thresholding is an alternative to \"Voxel\"-based
correction, and is normally more sensitive to activation. You may well
want to increase the cluster creation \"Z threshold\" if you have high
levels of activation.

If \"Voxel\" thresholding is selected, GRF-theory-based maximum height
thresholding is carried out, with thresholding at the level set, using
one-tailed testing. This test is less overly-conservative than
Bonferroni correction.

You can also choose to simply threshold the uncorrected Z statistic
values, or apply no thresholding at all."

#}}}
#{{{ contrast masking

button $w.conmask -text "Contrast masking" -command "feat5:setup_conmask $w"

set fmri(conmask_help) "Setup the masking of contrasts by other contrasts; after thresholding
of all contrasts has taken place you can further \"threshold\" a given
Z statistic image by masking it with non-zeroed voxels from other
contrasts.

This means that of the voxels which passed thresholding in the
contrast (or F-test) of interest, only those which also survived
thresholding in the other contrasts (or F-tests) are kept.

As a further option, the generated masks can be derived from all
positive Z statistic voxels in the mask contrasts rather than all
voxels that survived thresholding."

$w.bhelp bind $w.conmask -msg $fmri(conmask_help) 

#}}}
#{{{ rendering

tixLabelFrame $w.render -label "Rendering"
set fmri(lfrendering) [ $w.render subwidget frame ]
set fmri(lfrenderingtop) [ frame $fmri(lfrendering).top ]

#{{{ Z display min and max

global fmri(zdisplay)
tixOptionMenu $w.zmaxmenu -variable fmri(zdisplay)
$w.zmaxmenu add command 0 -label "Use actual Z min/max"
$w.zmaxmenu add command 1 -label "Use preset Z min/max"

set fmri(zmin) 1
tixControl $w.zmin -label "Min" -variable fmri(zmin) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }
set fmri(zmax) 15
tixControl $w.zmax -label "Max" -variable fmri(zmax) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

$w.bhelp bind $w.zmaxmenu -msg "The Z statistic range selected for rendering is automatically
calculated by default, to run from red (minimum Z statistic after
thresholding) to yellow (maximum Z statistic). If more than one colour
rendered image is to be produced (i.e., when multiple constrasts are
created) then the overall range of Z values is automatically found
from all of the Z statistic images, for consistent Z statistic
colour-coding.

If multiple analyses are to be carried out, \"Use preset Z min/max\"
should be chosen, and the min/max values set by hand. Again, this
ensures consistency of Z statistic colour-coding - if several
experiments are to be reported side-by-side, colours will refer to the
same Z statistic values in each picture. When using this option, you
should choose a conservatively wide range for the min and max (e.g.,
min=1, max=15), to make sure that you do not carry out unintentional
thresholding via colour rendering."

#}}}
#{{{ render type

tixOptionMenu $w.rendertype -variable fmri(rendertype)

$w.rendertype add command 0 -label "Solid blobs"
$w.rendertype add command 1 -label "Transparent blobs"

set fmri(rendertype) 1

$w.bhelp bind $w.rendertype -msg "With \"Solid colours\" you don't see any sign of the background images
within the colour blobs; with \"Transparent colours\" you will see
through the colour blobs to the background intensity"

#}}}

pack $w.zmaxmenu $w.rendertype -in $fmri(lfrenderingtop) -side left -anchor n -padx 5 -pady 5
pack $fmri(lfrenderingtop) -in $fmri(lfrendering) -anchor w

#}}}

pack $fmri(poststatsf).threshmask -in $fmri(poststatsf) -side top -anchor w -padx 5 -pady 5
pack $w.thresh $w.render          -in $fmri(poststatsf) -side top -anchor w

set fmri(thresh) 3
set fmri(zdisplay) 0
set fmri(conmask_zerothresh_yn) 0
$w.thresh.menu configure -command "feat5:updatepoststats $w"
$w.zmaxmenu configure -command "feat5:updatepoststats $w"

#{{{ background image for group stats

set fmri(bgimage) 1

tixOptionMenu $w.bgimage -variable fmri(bgimage) -label " Background image"
$w.bgimage add command 1 -label "Mean highres"
$w.bgimage add command 2 -label "First highres"
$w.bgimage add command 3 -label "Mean functional"
$w.bgimage add command 4 -label "First functional"
$w.bgimage add command 5 -label "Standard space template"

$w.bhelp bind $w.bgimage -msg "With \"Higher-level analysis\" you can select what image will be used
as the background image for the activation colour overlays. The
default of \"Mean highres\" is probably the best for relating
activation to underlying structure. For a sharper underlying image,
(but one which is not so representative of the group of subjects), you
can instead choose to use the highres image from the first selected
subject.

You can alternatively choose to use the original lowres functional
data for the overlays, or the standard-space template image."

#}}}

#}}}
#{{{ Registration

set fmri(regf) [ $w.nb subwidget reg ]

set fmri(regreduce_dof) 0
set fmri(reg_yn) 1

#{{{ unwarp

frame $fmri(regf).unwarp

set fmri(regunwarp_yn) 0
checkbutton $fmri(regf).unwarp.yn -variable fmri(regunwarp_yn) -command "feat5:updatereg $w"

label $fmri(regf).unwarp.label -text "B0 unwarping"

tixLabelFrame $fmri(regf).unwarp.lf -label "B0 unwarping"
set fmri(unwarpf) [ $fmri(regf).unwarp.lf subwidget frame ]

set unwarp_files(1) "~"

FSLFileEntry $fmri(unwarpf).unwarpsingle \
	-variable unwarp_files(1) \
	-pattern "*.hdr" \
	-directory $PWD \
	-label "" \
	-title "Select the complex B0 fieldmap image" \
	-width 45 \
	-filterhist VARS(history) \
	-command "feat5:multiple_check $w 1 1 0"

button $fmri(unwarpf).unwarpmultiple -text "Select the complex B0 fieldmap images" \
	-command "feat5:multiple_select $w 1 \"Select the complex B0 fieldmap images\" "

frame $fmri(unwarpf).opts

set fmri(dwellasym) -1
tixControl $fmri(unwarpf).opts.dwellasym -label "  Dwell/asymmetry ratio" -variable fmri(dwellasym) \
	-step 0.1 -selectmode immediate -options { entry.width 4 }

pack $fmri(unwarpf).opts.dwellasym -in $fmri(unwarpf).opts -side left
pack $fmri(unwarpf).unwarpsingle $fmri(unwarpf).opts -in $fmri(unwarpf) -anchor w -side top -pady 2 -padx 3
pack $fmri(regf).unwarp.yn $fmri(regf).unwarp.label -in $fmri(regf).unwarp -side left

$w.bhelp bind $fmri(regf).unwarp -msg "B0 unwarping is carried out using the programs PRELUDE (for phase
unwrapping) and FUGUE (for the actual B0 field map-based unwarping)"

#}}}
#{{{ high res

frame $fmri(regf).initial_highres

set fmri(reginitial_highres_yn) 0
checkbutton $fmri(regf).initial_highres.yn -variable fmri(reginitial_highres_yn) -command "feat5:updatereg_hr_init $w"

label $fmri(regf).initial_highres.label -text "Initial structural image"

tixLabelFrame $fmri(regf).initial_highres.lf -label "Initial structural image"
set fmri(initial_highresf) [ $fmri(regf).initial_highres.lf subwidget frame ]

set initial_highres_files(1) "~"

FSLFileEntry $fmri(initial_highresf).initial_highressingle \
	-variable initial_highres_files(1) \
	-pattern "*.hdr" \
	-directory $PWD \
	-label "" \
	-title "Select initial structural image" \
	-width 45 \
	-filterhist VARS(history) \
	-command "feat5:multiple_check $w 2 1 0"

button $fmri(initial_highresf).initial_highresmultiple -text "Select initial structural images" \
	-command "feat5:multiple_select $w 2 \"Select initial structural images\" "

frame $fmri(initial_highresf).opts

global fmri(reginitial_highres_search)
tixOptionMenu $fmri(initial_highresf).opts.search -variable fmri(reginitial_highres_search) -label "  Linear "
$fmri(initial_highresf).opts.search add command 0 -label "No search"
$fmri(initial_highresf).opts.search add command 90 -label "Normal search"
$fmri(initial_highresf).opts.search add command 180 -label "Full search"
set fmri(reginitial_highres_search) 90

global fmri(reginitial_highres_dof)
tixOptionMenu $fmri(initial_highresf).opts.dof -variable fmri(reginitial_highres_dof)
$fmri(initial_highresf).opts.dof add command 3 -label "3 DOF (translation-only)"
$fmri(initial_highresf).opts.dof add command 6 -label "6 DOF"
$fmri(initial_highresf).opts.dof add command 7 -label "7 DOF"
$fmri(initial_highresf).opts.dof add command 12 -label "12 DOF"
set fmri(reginitial_highres_dof) 12

set fmri(reginitial_highres_nonlinear_yn) 0
label $fmri(initial_highresf).opts.nonlinear_label -text "     Nonlinear"
checkbutton $fmri(initial_highresf).opts.nonlinear_yn -variable fmri(reginitial_highres_nonlinear_yn)

#pack $fmri(initial_highresf).opts.search $fmri(initial_highresf).opts.dof $fmri(initial_highresf).opts.nonlinear_label \
#	$fmri(initial_highresf).opts.nonlinear_yn -in $fmri(initial_highresf).opts -side left
pack $fmri(initial_highresf).opts.search $fmri(initial_highresf).opts.dof -in $fmri(initial_highresf).opts -side left

pack $fmri(initial_highresf).initial_highressingle $fmri(initial_highresf).opts -in $fmri(initial_highresf) -anchor w -side top -pady 2 -padx 3
pack $fmri(regf).initial_highres.yn $fmri(regf).initial_highres.label -in $fmri(regf).initial_highres -side left

$w.bhelp bind $fmri(regf).initial_highres -msg "This is the initial high resolution structural image which the low
resolution functional data will be registered to, and this in turn
will be registered to the main highres image. It only makes sense to
have this initial highres image if a main highres image is also
specified and used in the registration. 

One example of an initial highres structural image might be a
medium-quality structural scan taken during a day's scanning, if a
higher-quality image has been previously taken for the subject. A
second example might be a full-brain image with the same MR sequence
as the functional data, useful if the actual functional data is only
partial-brain. It is strongly recommended that this image have
non-brain structures already removed, for example by using BET.

If the field-of-view of the functional data (in any direction) is less
than 120mm, then the registration of the functional data will by
default have a reduced degree-of-freedom, for registration stability.

If you are attempting to register partial field-of-view functional
data to a whole-brain image then \"3 DOF\" is recommended - in this
case only translations are allowed.

If the orientation of any image is different from any other image it
may be necessary to change the search to \"Full search\"."

#}}}
#{{{ high res2

frame $fmri(regf).highres

set fmri(reghighres_yn) 0
checkbutton $fmri(regf).highres.yn -variable fmri(reghighres_yn) -command "feat5:updatereg_hr $w"

label $fmri(regf).highres.label -text "Main structural image"

tixLabelFrame $fmri(regf).highres.lf -label "Main structural image"
set fmri(highresf) [ $fmri(regf).highres.lf subwidget frame ]

set highres_files(1) "~"

FSLFileEntry $fmri(highresf).highressingle \
	-variable highres_files(1) \
	-pattern "*.hdr" \
	-directory $PWD \
	-label "" \
	-title "Select main structural image" \
	-width 45 \
	-filterhist VARS(history) \
	-command "feat5:multiple_check $w 3 1 0"

button $fmri(highresf).highresmultiple -text "Select main structural images" \
	-command "feat5:multiple_select $w 3 \"Select main structural images\" "

frame $fmri(highresf).opts

global fmri(reghighres_search)
tixOptionMenu $fmri(highresf).opts.search -variable fmri(reghighres_search) -label "  Linear "
$fmri(highresf).opts.search add command 0 -label "No search"
$fmri(highresf).opts.search add command 90 -label "Normal search"
$fmri(highresf).opts.search add command 180 -label "Full search"
set fmri(reghighres_search) 90

global fmri(reghighres_dof)
tixOptionMenu $fmri(highresf).opts.dof -variable fmri(reghighres_dof)
$fmri(highresf).opts.dof add command 3 -label "3 DOF (translation-only)"
$fmri(highresf).opts.dof add command 6 -label "6 DOF"
$fmri(highresf).opts.dof add command 7 -label "7 DOF"
$fmri(highresf).opts.dof add command 12 -label "12 DOF"
set fmri(reghighres_dof) 12

set fmri(reghighres_nonlinear_yn) 0
label $fmri(highresf).opts.nonlinear_label -text "     Nonlinear"
checkbutton $fmri(highresf).opts.nonlinear_yn -variable fmri(reghighres_nonlinear_yn)

#pack $fmri(highresf).opts.search $fmri(highresf).opts.dof $fmri(highresf).opts.nonlinear_label \
#	$fmri(highresf).opts.nonlinear_yn -in $fmri(highresf).opts -side left
pack $fmri(highresf).opts.search $fmri(highresf).opts.dof -in $fmri(highresf).opts -side left

pack $fmri(highresf).highressingle $fmri(highresf).opts -in $fmri(highresf) -anchor w -side top -pady 2 -padx 3
pack $fmri(regf).highres.yn $fmri(regf).highres.label -in $fmri(regf).highres -side left

$w.bhelp bind $fmri(regf).highres -msg "This is the main high resolution structural image which the low resolution
functional data will be registered to (optionally via the \"initial
structural image\"), and this in turn will be registered to the
standard brain. It is strongly recommended that this image have
non-brain structures already removed, for example by using BET.

If the field-of-view of the functional data (in any direction) is less
than 120mm, then the registration of the functional data will by
default have a reduced degree-of-freedom, for registration stability.

If you are attempting to register partial field-of-view functional
data to a whole-brain image then \"3 DOF\" is recommended - in this
case only translations are allowed.

If the orientation of any image is different from any other image it
may be necessary to change the search to \"Full search\"."

#}}}
#{{{ standard

frame $fmri(regf).standard

set fmri(regstandard_yn) 1
checkbutton $fmri(regf).standard.yn -variable fmri(regstandard_yn) -command "feat5:updatereg $w"

label $fmri(regf).standard.label -text "Standard space"

tixLabelFrame $fmri(regf).standard.lf -label "Standard space"
set fmri(standardf) [ $fmri(regf).standard.lf subwidget frame ]

set fmri(regstandard) ${FSLDIR}/etc/standard/avg152T1_brain.hdr

FSLFileEntry  $fmri(standardf).standardentry \
	-variable fmri(regstandard) \
	-pattern "*.hdr" \
	-directory "${FSLDIR}/etc/standard/" \
	-label "" \
	-title "Select the standard image" \
	-width 45 \
	-filterhist VARS(history)

frame $fmri(standardf).opts

global fmri(regstandard_search)
tixOptionMenu $fmri(standardf).opts.search -variable fmri(regstandard_search) -label "  Linear "
$fmri(standardf).opts.search add command 0 -label "No search"
$fmri(standardf).opts.search add command 90 -label "Normal search"
$fmri(standardf).opts.search add command 180 -label "Full search"
set fmri(regstandard_search) 90

global fmri(regstandard_dof)
tixOptionMenu $fmri(standardf).opts.dof -variable fmri(regstandard_dof)
$fmri(standardf).opts.dof add command 3 -label "3 DOF (translation-only)"
$fmri(standardf).opts.dof add command 6 -label "6 DOF"
$fmri(standardf).opts.dof add command 7 -label "7 DOF"
$fmri(standardf).opts.dof add command 12 -label "12 DOF"
set fmri(regstandard_dof) 12

set fmri(regstandard_nonlinear_yn) 0
label $fmri(standardf).opts.nonlinear_label -text "     Nonlinear"
checkbutton $fmri(standardf).opts.nonlinear_yn -variable fmri(regstandard_nonlinear_yn)

#pack $fmri(standardf).opts.search $fmri(standardf).opts.dof $fmri(standardf).opts.nonlinear_label \
#	$fmri(standardf).opts.nonlinear_yn -in $fmri(standardf).opts -side left
pack $fmri(standardf).opts.search $fmri(standardf).opts.dof -in $fmri(standardf).opts -side left

pack $fmri(standardf).standardentry $fmri(standardf).opts -in $fmri(standardf) -anchor w -side top -pady 2 -padx 3
pack $fmri(regf).standard.yn $fmri(regf).standard.label -in $fmri(regf).standard -side left

$w.bhelp bind $fmri(regf).standard -msg "This is the standard (reference) image;
it should be an image already in Talairach space,
ideally with the non-brain structures already removed.

If the field-of-view of the functional data (in any direction) is less
than 120mm, then the registration of the functional data will by
default have a reduced degree-of-freedom, for registration stability.

If you are attempting to register partial field-of-view functional
data to a whole-brain image then \"3 DOF\" is recommended - in this
case only translations are allowed.

If the orientation of any image is different from any other image it
may be necessary to change the search to \"Full search\"."

#}}}

#pack $fmri(regf).unwarp $fmri(regf).initial_highres $fmri(regf).highres $fmri(regf).standard -in $fmri(regf) -side top -anchor w -pady 0
pack $fmri(regf).initial_highres $fmri(regf).highres $fmri(regf).standard -in $fmri(regf) -side top -anchor w -pady 0

#}}}

$w.multiple.number configure -command "feat5:updateselect $w"

set fmri(level) 1
set fmri(analysis) 7
$w.mode.level    configure -command "feat5:updatelevel $w"
$w.mode.analysis configure -command "feat5:updateanalysis $w"
feat5:updatelevel $w 0

$w.nb raise data

#}}}
    #{{{ button Frame

frame $w.btns
    
button $w.btns.apply -command "feat5:apply $w" -text "Go"

FSLFileDirSelectDialog $w.savedialog \
	-title  "Save FEAT setup" \
	-command "feat5:write $w 1 1"
$w.savedialog subwidget fsbox config -directory $PWD -pattern "*.fsf" -filterhist VARS(history)
button $w.btns.save -command "$w.savedialog popup" -text "Save"

FSLFileDirSelectDialog $w.loaddialog \
	-title  "Load FEAT setup" \
	-command "feat5:load $w 1"
$w.loaddialog subwidget fsbox config -directory $PWD -pattern "*.fsf" -filterhist VARS(history)
button $w.btns.load -command "$w.loaddialog popup" -text "Load"

button $w.btns.cancel -command "destroy $w" -text "Exit"

button $w.btns.help -command "FmribWebHelp file: ${FSLDIR}/doc/feat5/index.html" -text "Help"

#{{{ Utils

menubutton $w.btns.utils -text "Utils" -menu $w.btns.utils.menu -relief raised -font $fmri(font)

menu $w.btns.utils.menu

$w.btns.utils.menu add command -label "Featquery - get FEAT stats from ROI mask or co-ordinates" -command { exec ${FSLDIR}/bin/Featquery$gui_ext & }

if { $INMEDX } {
    $w.btns.utils.menu add command -label "Load FEAT session into MEDx" -command "feat5:medxload 0"
}

if { ! $INMEDX } {
    $w.btns.utils.menu add command -label "Simple Stats colour rendering " -command { exec ${FSLDIR}/bin/Renderstats$gui_ext & }
}

$w.btns.utils.menu add command -label "High-res FEAT stats colour rendering" -command { exec ${FSLDIR}/bin/Renderhighres$gui_ext & }

#}}}

pack $w.btns.apply $w.btns.save $w.btns.load $w.btns.cancel $w.btns.help $w.btns.utils -in $w.btns -side left -expand yes

#}}}

    pack $w.mode $w.nb -in $w -side top -anchor n -padx 10 -pady 10
    pack $w.btns -in $w -side bottom -fill x -padx 10 -pady 10

    #{{{ load fsf/hdr file

if { $argc > 0 && [ string length [ lindex $argv 0 ] ] > 0 } {

    set inputname [ lindex $argv 0 ]

    if { [ string first / $inputname ] != 0 && [ string first ~ $inputname ] != 0 } {
	set inputname ${PWD}/$inputname
    }

    if { [ string compare [ file extension $inputname ] .fsf ] == 0 } {

	puts "Loading FEAT setup file $inputname"
	feat5:load $w 1 $inputname

    } elseif { [ string compare [ file extension $inputname ] .hdr ] == 0 } {

	puts "Selected functional data $inputname"
	set feat_files(1) $inputname
	feat5:multiple_check $w 0 0 1 0
	
    }
}

#}}}
}

#}}}
#{{{ feat5:apply

proc feat5:apply { w } {
    global fmri feat_files unwarp_files initial_highres_files highres_files FSLDIR HOME INMEDX

    #{{{ check FILM setup

if { $fmri(analysis) > 1 && ! $fmri(filmsetup) } {
    MxPause "Please setup model/contrast details"
    return 1
}

#}}}
    #{{{ update variables

foreach v { tr ndelete paradigm_hp brain_thresh smooth uncorrected voxel prob_thresh z_thresh zmin zmax } {
    $w.$v update
}

#}}}
    #{{{ check unwarping

if { $fmri(regunwarp_yn) && $fmri(dwellasym) <= 0 } {
    MxPause "Please setup the Dwell/asymmetry ratio inside B0 unwarping"
    return 1
}

#}}}
    #{{{ if using a MEDx page, save the page

set fmri(medxload) 0

if { $fmri(multiple) < 1 } {

    set GroupFullNameTmp $fmri(groupname)

    regsub -all " " $GroupFullNameTmp "" GroupFullNameTmp2
    regsub -all "\"" $GroupFullNameTmp2 "" GroupFullName

    # find directory and filetailroot
    set GroupDirName [ file dirname  $GroupFullName ]
    regsub -all "\\." [ file rootname [ file tail $GroupFullName ] ] "_" GroupFileRoot

    set NotTmp [ string compare /tmp [ string range $GroupDirName 0 3 ] ]
    if { ! [ file writable $GroupDirName ] || $NotTmp == 0 } {
	set GroupName ${HOME}/${GroupFileRoot}
    } else {
	set GroupName ${GroupDirName}/${GroupFileRoot}
    }

    set tmpname [ new_filename ${GroupName}.hdr ]

    if { ! [ info exists feat_files(1) ] || [ FSLSaveAs $feat_files(1) AVW $tmpname true ] } {
	MxPause "Please select a valid group page"
	return 1
    }

    set feat_files(1) $tmpname
    
    set fmri(multiple) 1
    set fmri(medxload) 1
}

#}}}
    #{{{ write model

if { $fmri(level) > 1 || $fmri(stats_yn) || $fmri(poststats_yn) } {
    set createthemodel 1
} else {
    set createthemodel 0
}

if { [ feat5:write $w $createthemodel 1 $fmri(feat_filename) ] } {
    return 1
}

#}}}
    #{{{ if level>1 test for existing registration

if { $fmri(level) > 1 && $fmri(analysis) != 4 } {

    set problem 0

    for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {

	set featdirname $feat_files($i)
	if { $fmri(inputtype) == 2 } {
	    set featdirname [ file dirname [ file dirname $featdirname ] ]
	}

	if { ! [ file exists $featdirname/reg/example_func2standard.mat ] && 
	     ! [ file exists $featdirname/example_func2standard.mat ] && 
	     ! [ file exists $featdirname/design.lev ] } {
	    set problem 1
	}
    }

    if { $problem } {
	MxPause "Registration has not been run for all of the FEAT directories that you have selected for group analysis.

Please turn on and setup registration."
	return 1
    }

}

#}}}
    #{{{ wait if required

set thedelay ""
if { $fmri(delay) > 0.003 } {
    set thedelay "sleep [ expr int( $fmri(delay) * 60.0 * 60.0 ) ] ; "
}

#}}}

    set FSFROOT [ file rootname $fmri(feat_filename) ]

    if { ! $INMEDX } {
	catch { exec sh -c "$thedelay ${FSLDIR}/bin/feat $FSFROOT" & } junk
	MxPause "Running FEAT using setup file
${FSFROOT}.fsf"
    } else {
	puts "Running FEAT using setup file ${FSFROOT}.fsf ...."
	catch { exec sh -c "$thedelay ${FSLDIR}/bin/feat $FSFROOT" } junk
        puts $junk
	#feat5:proc $FSFROOT
	if { $fmri(medxload) } {
	    feat5:medxload_proc $feat_files(1) 1
	}
    }

    set fmri(feat_filename) [ exec ${FSLDIR}/bin/tmpnam /tmp/feat ].fsf

    update idletasks
}

#}}}
#{{{ feat5:updates

#{{{ feat5:updatestats

proc feat5:updatestats { w process dummy } {
    global fmri

    if { $process } {
	if { [ info exists fmri(w_model) ] && [ winfo exists $fmri(w_model) ] } {
	    destroy $fmri(w_model)
	}
	feat5:setup_model_vars_simple $w
	if { [ feat5:setup_model_preview $w ] } {
	    return 1
	}
    }

    pack forget $w.prewhiten $w.wizard $w.model $w.mixed

    if { $fmri(level) == 1 } {
	pack $w.prewhiten $w.wizard $w.model -in $fmri(statsf) -anchor w -side top -padx 5 -pady 3
    } else {
	pack $w.mixed $w.model               -in $fmri(statsf) -anchor w -side top -padx 5 -pady 3
    }
}

#}}}
#{{{ feat5:updateanalysis and updatelevel

proc feat5:updatelevel { w dummy } {
    global fmri

    if { $fmri(level) == 1 } {
	$w.mode.analysis enable 0
	$w.mode.analysis enable 7
	$w.mode.analysis enable 1
	$w.mode.analysis enable 3
	$w.mode.analysis enable 2
	$w.mode.analysis enable 4
	set fmri(analysis) 7
	set fmri(reg_yn) 1
	set fmri(wizard_type) 1
	set fmri(a_count) 30
	set fmri(b_count) 30
    } else {
	set fmri(analysis) 6
	set fmri(reg_yn) 0
	$w.mode.analysis disable 0
	$w.mode.analysis disable 7
	$w.mode.analysis disable 1
	$w.mode.analysis disable 3
	$w.mode.analysis disable 2
	$w.mode.analysis disable 4
    }

    feat5:setup_model_vars_simple $w
    set fmri(paradigm_hp) 100
    set fmri(filmsetup) 0

    feat5:updateanalysis $w 0
}

proc feat5:updateanalysis { w dummy } {
    global fmri

    set fmri(filtering_yn) [ expr   $fmri(analysis)       % 2 == 1 ]
    set fmri(stats_yn)     [ expr ( $fmri(analysis) / 2 ) % 2 == 1 ]
    set fmri(poststats_yn) [ expr ( $fmri(analysis) / 4 ) % 2 == 1 ]

    #{{{ update notebook view

if { !$fmri(filtering_yn) } {
    $w.nb pageconfigure filtering -state disabled
}  else {
    $w.nb pageconfigure filtering -state normal
}

if { !$fmri(stats_yn) } {
    $w.nb pageconfigure stats -state disabled
}  else {
    $w.nb pageconfigure stats -state normal
}

if { !$fmri(poststats_yn) } {
    $w.nb pageconfigure poststats -state disabled
} else {
    $w.nb pageconfigure poststats -state normal
    if { !$fmri(stats_yn) } {
	set fmri(ndelete) 0
    }
}

if { $fmri(level)==1 } {
    $w.nb pageconfigure reg -state normal
}  else {
    $w.nb pageconfigure reg -state disabled
}

#}}}
    #{{{ update misc and data

pack forget $w.newdir_yn
if { $fmri(analysis) == 0 || $fmri(analysis) == 4 } {
    pack $w.newdir_yn -in $fmri(miscf) -anchor w -side top -padx 3 -pady 1
}

pack forget $w.inputtype $w.datamain $w.outputdir
if { $fmri(analysis) != 0 && $fmri(analysis) != 4 } {
    pack $w.outputdir -in $fmri(dataf) -anchor w -side top -pady 1
    if { $fmri(level) == 1  } {
	pack $w.datamain  -in $fmri(dataf) -anchor w -side top -pady 3
    } else {
	pack $w.inputtype -in $fmri(dataf) -before $w.multiple -anchor w -side top -pady 3
    }
}

#}}}

    feat5:updatestats $w 0 -1
    feat5:updatepoststats $w 0
    feat5:updatereg $w

    $w.nb raise data
}

#}}}
#{{{ feat5:updateselect

proc feat5:updateselect { w dummy } {
    global fmri INMEDX feat_files

    if { $fmri(level) == 1 } {
	if { $INMEDX } {
	    $w.multiple.number configure -min 0
	} else {
	    $w.multiple.number configure -min 1
	}
    } else {
	if { $fmri(analysis) != 4 } {
	    $w.multiple.number configure -min 2
	    if { $fmri(multiple) < 2 } {
		set fmri(multiple) 2
	    }
	} else {
	    $w.multiple.number configure -min 1
	}
    }

    if { $fmri(level) == 1 && $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
	if { $fmri(multiple) < 1 } {
	    $w.multiple.setup configure -text "Select group"
	    if { [ info exists fmri(groupname) ] } {	    
		MxGetCurrentFolder Folder
		MxGetPageByName $Folder $fmri(groupname) fmri(grouppage)
		MxGetPagesFromGroup $fmri(grouppage) GPages
		if { [ info exists GPages ] } {
		    $w.groupname configure -text $fmri(groupname)
		    pack $w.groupname -in $fmri(dataf) -after $w.multiple -anchor e -padx 10
		    set fmri(npts) [ llength $GPages ]
		    set feat_files(1) $fmri(grouppage)
		    feat5:updateimageinfo $w 1
		} else {
		    set feat_files(1) ""
		}
	    }
	} else {
	    pack forget $w.groupname
	    $w.multiple.setup configure -text "Select 4D data"
	}
    } else {
	if { $fmri(level) == 1 || $fmri(inputtype) == 1 } {
	    if { $fmri(multiple) == 1 } {
		$w.multiple.setup configure -text "Select FEAT directory"
	    } else {
		$w.multiple.setup configure -text "Select FEAT directories"
	    }
	} else {
		$w.multiple.setup configure -text "Select cope images"
	}
    }

    pack forget $fmri(unwarpf).unwarpmultiple $fmri(unwarpf).unwarpsingle \
	    $fmri(initial_highresf).initial_highresmultiple $fmri(initial_highresf).initial_highressingle \
	    $fmri(highresf).highresmultiple $fmri(highresf).highressingle

    if { ! $fmri(relative_yn) } {
	if { $fmri(multiple) < 2 } {
	    pack $fmri(unwarpf).unwarpsingle -in $fmri(unwarpf) -before $fmri(unwarpf).opts -anchor w -side top -pady 2 -padx 3
	    pack $fmri(initial_highresf).initial_highressingle -in $fmri(initial_highresf) -before $fmri(initial_highresf).opts -anchor w -side top -pady 2 -padx 3
	    pack $fmri(highresf).highressingle -in $fmri(highresf) -before $fmri(highresf).opts -anchor w -side top -pady 2 -padx 3
	} else {
	    pack $fmri(unwarpf).unwarpmultiple -in $fmri(unwarpf) -before $fmri(unwarpf).opts -anchor w -side top -pady 2 -padx 3
	    pack $fmri(initial_highresf).initial_highresmultiple -in $fmri(initial_highresf) -before $fmri(initial_highresf).opts -anchor w -side top -pady 2 -padx 3
	    pack $fmri(highresf).highresmultiple -in $fmri(highresf) -before $fmri(highresf).opts -anchor w -side top -pady 2 -padx 3
	}
    }
}

#}}}
#{{{ feat5:updatehelp

proc feat5:updatehelp { w } {
    global fmri

    if { $fmri(help_yn) == 1 } {
	$w.bhelp configure -state balloon
    } else {
	$w.bhelp configure -state none
    }
}

#}}}
#{{{ feat5:updatemc

proc feat5:updatemc { w dummy } {
    global fmri

#    if { $fmri(mc) != 4 } {
#	pack forget $w.sh
#    } else {
#	pack $w.sh -in $fmri(mcf) -side left
#    }
}

#}}}
#{{{ feat5:updatest

proc feat5:updatest { w dummy } {
    global fmri

    if { $fmri(st) < 3 } {
	pack forget $w.st_file
    } else {
	pack $w.st_file -in $fmri(stf) -side left -padx 5
    }
}

#}}}
#{{{ feat5:updatepoststats

proc feat5:updatepoststats { w dummy } {
    global fmri

    pack forget $w.modelcon $w.uncorrected $w.voxel $w.z_thresh $w.prob_thresh $w.conmask $w.render $w.bgimage $w.zmin $w.zmax

    if { ! $fmri(stats_yn) } {
	pack $w.modelcon -in $fmri(poststatsf) -before $fmri(poststatsf).threshmask -side top -anchor w -padx 5 -pady 5
    }

    if { $fmri(thresh) } {
	if { $fmri(thresh) == 1 } {
	    pack $w.uncorrected -in $fmri(lfthresh) -side left
	} elseif { $fmri(thresh) == 2 } {
	    pack $w.voxel -in $fmri(lfthresh) -side left
	} else {
	    pack $w.z_thresh $w.prob_thresh -in $fmri(lfthresh) -side left -anchor w -pady 2
	}

	pack $w.conmask -in $fmri(poststatsf) -after $w.thresh -side top -anchor w -pady 5 -padx 5
	pack $w.render  -in $fmri(poststatsf) -after $w.conmask -side top -anchor w -pady 5

	pack $fmri(lfrenderingtop) -in $fmri(lfrendering) -anchor w -padx 3 -pady 3

	if { $fmri(level) > 1 } {
	    pack $w.bgimage -in $fmri(lfrendering) -before $fmri(lfrenderingtop) -anchor w -padx 3 -pady 3
	}
	
	if { $fmri(zdisplay) } {
	    pack $w.zmin $w.zmax -in $fmri(lfrenderingtop) -after $w.zmaxmenu -side left -anchor w -padx 3 -pady 5
	}
    }

}

#}}}
#{{{ feat5:updatereg

proc feat5:updatereg_hr_init { w } {
    global fmri

    if { $fmri(reginitial_highres_yn) == 1 } {
	set fmri(reghighres_yn) 1
    }

    feat5:updatereg $w
}

proc feat5:updatereg_hr { w } {
    global fmri

    if { $fmri(reghighres_yn) == 0 } {
	set fmri(reginitial_highres_yn) 0
    }

    feat5:updatereg $w
}

proc feat5:updatereg { w } {
    global fmri

    if { $fmri(regreduce_dof) > 0 } {
	set thedof 7
	if { $fmri(regreduce_dof) == 2 } {
	    set thedof 3
	}
	if { $fmri(reginitial_highres_yn) } {
	    set fmri(reginitial_highres_dof) $thedof
	} elseif { $fmri(reghighres_yn) } {
	    set fmri(reghighres_dof) $thedof
	} elseif { $fmri(regstandard_yn) } {
	    set fmri(regstandard_dof) $thedof
	}
    }

    if { $fmri(regunwarp_yn) } {
	pack forget $fmri(regf).unwarp.label 
	pack $fmri(regf).unwarp.lf -in $fmri(regf).unwarp -side left
    } else {
	pack forget $fmri(regf).unwarp.lf 
	pack $fmri(regf).unwarp.label -in $fmri(regf).unwarp -side left
    }

    if { $fmri(reginitial_highres_yn) } {
	pack forget $fmri(regf).initial_highres.label 
	pack $fmri(regf).initial_highres.lf -in $fmri(regf).initial_highres -side left
    } else {
	pack forget $fmri(regf).initial_highres.lf 
	pack $fmri(regf).initial_highres.label -in $fmri(regf).initial_highres -side left
    }

    if { $fmri(reghighres_yn) } {
	pack forget $fmri(regf).highres.label 
	pack $fmri(regf).highres.lf -in $fmri(regf).highres -side left
    } else {
	pack forget $fmri(regf).highres.lf 
	pack $fmri(regf).highres.label -in $fmri(regf).highres -side left
    }

    if { $fmri(regstandard_yn) } {
	pack forget $fmri(regf).standard.label 
	pack $fmri(regf).standard.lf -in $fmri(regf).standard -side left
    } else {
	pack forget $fmri(regf).standard.lf 
	pack $fmri(regf).standard.label -in $fmri(regf).standard -side left
    }

    feat5:updateselect $w 0
}

#}}}
#{{{ feat5:updateimageinfo

proc feat5:updateimageinfo { w i } {
    global FSLDIR feat_files fmri

    set thefile [ file rootname $feat_files($i) ]

    if { [ file exists ${thefile}.hdr ] } {
	set npts [ exec ${FSLDIR}/bin/avwnvols $thefile 2> /dev/null ]

	if { $npts > 0 } {
	    set fmri(npts) $npts
	}

	# set BET and FLIRT DOF according to FOV
	set xfov [ expr abs([ exec $FSLDIR/bin/avwval $thefile pixdim1 ] * [ exec $FSLDIR/bin/avwval $thefile dim1 ]) ]
	set yfov [ expr abs([ exec $FSLDIR/bin/avwval $thefile pixdim2 ] * [ exec $FSLDIR/bin/avwval $thefile dim2 ]) ]
	set zfov [ expr abs([ exec $FSLDIR/bin/avwval $thefile pixdim3 ] * [ exec $FSLDIR/bin/avwval $thefile dim3 ]) ]

	set BETMINFOV 30
	if { $xfov < $BETMINFOV || $yfov < $BETMINFOV || $zfov < $BETMINFOV } {
	    set fmri(bet_yn) 0
	}

	set REGMINFOV 120
	if { $xfov < $REGMINFOV || $yfov < $REGMINFOV || $zfov < $REGMINFOV } {
	    set fmri(regreduce_dof) 1
	    feat5:updatereg $w
	}

	set REGTRANSMINFOV 60
	if { $xfov < $REGMINFOV || $yfov < $REGMINFOV || $zfov < $REGMINFOV } {
	    set fmri(regreduce_dof) 2
	    feat5:updatereg $w
	}
    }

    if { $fmri(level) > 1 && $fmri(analysis) != 4 } {
	set fmri(npts) $fmri(multiple)
    }
}

#}}}

#}}}
#{{{ feat5:write

proc feat5:write { w feat_model write_image_filenames filename } {
    global fmri FSLDIR feat_files unwarp_files initial_highres_files highres_files

    if { $fmri(level) == 1 && $fmri(analysis) > 0 && $fmri(con_mode) == "orig" && [ feat5:setup_model_update_contrasts_mode $w 0 ] == -1 } {
	return -1
    }

    #{{{ test f-tests

for { set f 1 } { $f <= $fmri(nftests_real) } { incr f 1 } {
    for { set c 1 } { $c <= $fmri(ncon_real) } { incr c 1 } {
	if { $fmri(ftest_real${f}.${c}) == 1 } {

	    for { set cc 1 } { $cc < $c } { incr cc 1 } {
		if { $fmri(ftest_real${f}.${cc}) == 1 } {

		    set sum 0
		    for { set e 1 } { $e <= $fmri(evs_real) } { incr e 1 } {
			set sum [ expr $sum + $fmri(con_real${c}.${e}) * $fmri(con_real${cc}.${e}) ]
		    }

		    if { $sum != 0 } {
			MxPause "Warning - F-test $f contains contrasts ($cc and $c) which are correlated (non-orthogonal); this may mean that it is hard to make a sensible interpretation of the F-test results."
			return -1
		    }
		}
	    }
	}
    }
}

#}}}

    set filename [ file rootname $filename ]

    if { $fmri(level) > 1 } {
	set fmri(npts) $fmri(multiple)
	set fmri(ndelete) 0
    }

    set channel [ open ${filename}.fsf "w" ]

    #{{{ basic variables

    puts $channel "
# FEAT version number
set fmri(version) $fmri(version)

# Analysis level
set fmri(level) $fmri(level)

# Which stages to run
# 0 : No first-level analysis (registration and/or group stats only)
# 7 : Full first-level analysis
# 1 : Pre-Stats
# 3 : Pre-Stats + Stats
# 2 :             Stats
# 6 :             Stats + Contrasts, Thresholding, Rendering
# 4 :                     Contrasts, Thresholding, Rendering
set fmri(analysis) $fmri(analysis)

# Delay before starting (hours)
set fmri(delay) $fmri(delay)

# Use relative filenames
set fmri(relative_yn) $fmri(relative_yn)

# Balloon help
set fmri(help_yn) $fmri(help_yn)

# Output directory
set fmri(outputdir) \"$fmri(outputdir)\"

# TR(s)
set fmri(tr) $fmri(tr)

# Total volumes
set fmri(npts) $fmri(npts)

# Delete volumes
set fmri(ndelete) $fmri(ndelete)

# Number of first-level analyses
set fmri(multiple) $fmri(multiple)

# Higher-level input type
# 1 : Inputs are lower-level FEAT directories
# 2 : Inputs are cope images
set fmri(inputtype) $fmri(inputtype)

# Carry out pre-stats processing?
set fmri(filtering_yn) $fmri(filtering_yn)

# Brain/background threshold, %
set fmri(brain_thresh) $fmri(brain_thresh)

# Post-stats-only directory copying
# 0 : Overwrite original post-stats results
# 1 : Copy original FEAT directory for new Contrasts, Thresholding, Rendering
set fmri(newdir_yn) $fmri(newdir_yn)

# Slice timing correction
# 0 : None
# 1 : Regular up (0, 1, 2, 3, ...)
# 2 : Regular down
# 3 : Use slice order file
# 4 : Use slice timings file
set fmri(st) $fmri(st)

# Slice timings file
set fmri(st_file) \"$fmri(st_file)\"

# Motion correction
# 0 : None
# 1 : MCFLIRT
set fmri(mc) $fmri(mc)

# Spin-history (currently obsolete)
set fmri(sh_yn) $fmri(sh_yn)

# BET brain extraction
set fmri(bet_yn) $fmri(bet_yn)

# Spatial smoothing FWHM (mm)
set fmri(smooth) $fmri(smooth)

# Intensity normalization
set fmri(norm_yn) $fmri(norm_yn)

# Highpass temporal filtering
set fmri(temphp_yn) $fmri(temphp_yn)

# Lowpass temporal filtering
set fmri(templp_yn) $fmri(templp_yn)

# Carry out main stats?
set fmri(stats_yn) $fmri(stats_yn)

# Carry out prewhitening?
set fmri(prewhiten_yn) $fmri(prewhiten_yn)

# Higher-level mixed effects choice
# 0 : Simple mixed effects
# 1 : FLAME Mixed effects
set fmri(mixed_yn) $fmri(mixed_yn)

# Number of EVs
set fmri(evs_orig) $fmri(evs_orig)
set fmri(evs_real) $fmri(evs_real)

# Number of contrasts
set fmri(ncon_orig) $fmri(ncon_orig)
set fmri(ncon_real) $fmri(ncon_real)

# Number of F-tests
set fmri(nftests_orig) $fmri(nftests_orig)
set fmri(nftests_real) $fmri(nftests_real)

# Add constant column to design matrix? (obsolete)
set fmri(constcol) $fmri(constcol)

# Carry out post-stats steps?
set fmri(poststats_yn) $fmri(poststats_yn)

# Pre-threshold masking?
set fmri(threshmask) \"$fmri(threshmask)\"

# Thresholding
# 0 : None
# 1 : Uncorrected
# 2 : Voxel
# 3 : Cluster
set fmri(thresh) $fmri(thresh)

# P threshold
set fmri(prob_thresh) $fmri(prob_thresh)

# Z threshold
set fmri(z_thresh) $fmri(z_thresh)

# Z min/max for colour rendering
# 0 : Use actual Z min/max
# 1 : Use preset Z min/max
set fmri(zdisplay) $fmri(zdisplay)

# Z min in colour rendering
set fmri(zmin) $fmri(zmin)

# Z max in colour rendering
set fmri(zmax) $fmri(zmax)

# Colour rendering type
# 0 : Solid blobs
# 1 : Transparent blobs
set fmri(rendertype) $fmri(rendertype)

# Background image for higher-level stats overlays
# 1 : Mean highres
# 2 : First highres
# 3 : Mean functional
# 4 : First functional
# 5 : Standard space template
set fmri(bgimage) $fmri(bgimage)

# Registration?
set fmri(reg_yn) $fmri(reg_yn)

# B0 fieldmap unwarping?
set fmri(regunwarp_yn) $fmri(regunwarp_yn)

# Dwell/Asymmetry ratio 
set fmri(dwellasym) $fmri(dwellasym)

# Registration to initial structural
set fmri(reginitial_highres_yn) $fmri(reginitial_highres_yn)

# Search space for registration to initial structural
# 0   : No search
# 90  : Normal search
# 180 : Full search
set fmri(reginitial_highres_search) $fmri(reginitial_highres_search)

# Degrees of Freedom for registration to initial structural
set fmri(reginitial_highres_dof) $fmri(reginitial_highres_dof)

# Do nonlinear registration to initial structural?
set fmri(reginitial_highres_nonlinear_yn) $fmri(reginitial_highres_nonlinear_yn)

# Registration to main structural
set fmri(reghighres_yn) $fmri(reghighres_yn)

# Search space for registration to main structural
# 0   : No search
# 90  : Normal search
# 180 : Full search
set fmri(reghighres_search) $fmri(reghighres_search)

# Degrees of Freedom for registration to main structural
set fmri(reghighres_dof) $fmri(reghighres_dof)

# Do nonlinear registration to main structural?
set fmri(reghighres_nonlinear_yn) $fmri(reghighres_nonlinear_yn)

# Registration to standard image?
set fmri(regstandard_yn) $fmri(regstandard_yn)

# Standard image
set fmri(regstandard) \"$fmri(regstandard)\"

# Search space for registration to standard space
# 0   : No search
# 90  : Normal search
# 180 : Full search
set fmri(regstandard_search) $fmri(regstandard_search)

# Degrees of Freedom for registration to standard space
set fmri(regstandard_dof) $fmri(regstandard_dof)

# Do nonlinear registration to standard space?
set fmri(regstandard_nonlinear_yn) $fmri(regstandard_nonlinear_yn)

# High pass filter cutoff
set fmri(paradigm_hp) $fmri(paradigm_hp)"

#}}}
    #{{{ input and highres filenames

if { $write_image_filenames } {

if { $fmri(multiple) > 0 } {
    if { $w != -1 } {
	if { [ feat5:multiple_check $w 0 0 0 0 ] } {
	    return 1
	}
    }
    for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {
	puts $channel "
# 4D AVW data or FEAT directory ($i)
set feat_files($i) \"$feat_files($i)\""
    }
}


if { $fmri(reg_yn) && ( $fmri(regunwarp_yn) || $fmri(reginitial_highres_yn) || $fmri(reghighres_yn) ) } {

    if { $fmri(multiple) < 2 } {
	set nhighres 1
    } else {
	set nhighres $fmri(multiple)
    }

    if { $fmri(relative_yn) } {
	for { set i 1 } { $i <= $nhighres } { incr i 1 } {
	    set unwarp_files($i) [ file dirname $feat_files($i) ]/blahstruct_brain.hdr
	    set initial_highres_files($i) [ file dirname $feat_files($i) ]/struct_brain.hdr
	    set highres_files($i) [ file dirname $feat_files($i) ]/blah2struct_brain.hdr
	}
    }

    if { $w != -1 } {
	if { $fmri(regunwarp_yn) } {
	    if { [ feat5:multiple_check $w 1 0 0 0 ] } {
		return 1
	    }
	    for { set i 1 } { $i <= $nhighres } { incr i 1 } {
		puts $channel "
# B0 unwarp input image for analysis $i
set unwarp_files($i) \"$unwarp_files($i)\""
            }
	}
	if { $fmri(reginitial_highres_yn) } {
	    if { [ feat5:multiple_check $w 2 0 0 0 ] } {
		return 1
	    }
	    for { set i 1 } { $i <= $nhighres } { incr i 1 } {
		puts $channel "
# Session's structural image for analysis $i
set initial_highres_files($i) \"$initial_highres_files($i)\""
            }
	}
	if { $fmri(reghighres_yn) } {
	    if { [ feat5:multiple_check $w 3 0 0 0 ] } {
		return 1
	    }
	    for { set i 1 } { $i <= $nhighres } { incr i 1 } {
		puts $channel "
# Subject's structural image for analysis $i
set highres_files($i) \"$highres_files($i)\""
            }
	}
    }
}

} else { 
    if { $fmri(npts) < 1 } {
	MxPause "Please either select input data or set the number of total volumes before attempting to view the design."
	return -1
    }
}

#}}}
    #{{{ EVs

	for { set i 1 } { $i <= $fmri(evs_orig) } { incr i 1 } {

	    if { [ info exists fmri(shape$i) ] } {

		puts $channel "
# Basic waveform shape (EV $i)
# 0 : Square
# 1 : Sinusoid
# 2 : Custom (1 entry per volume)
# 3 : Custom (3 column format)
# 4 : Interaction
set fmri(shape$i) $fmri(shape$i)

# Convolution (EV $i)
# 0 : None
# 1 : Gaussian
# 2 : Gamma
# 3 : Double-Gamma HRF
# 4 : Gamma basis functions
# 5 : Sine basis functions
# 6 : FIR basis functions
set fmri(convolve$i) $fmri(convolve$i)

# Convolve phase (EV $i)
set fmri(convolve_phase$i) $fmri(convolve_phase$i)

# Apply temporal filtering (EV $i)
set fmri(tempfilt_yn$i) $fmri(tempfilt_yn$i)

# Add temporal derivative (EV $i)
set fmri(deriv_yn$i) $fmri(deriv_yn$i)"

		switch $fmri(shape$i) {
		    0 { 
			puts $channel "
# Skip (EV $i)
set fmri(skip$i) $fmri(skip$i)

# Off (EV $i)
set fmri(off$i) $fmri(off$i)

# On (EV $i)
set fmri(on$i) $fmri(on$i)

# Phase (EV $i)
set fmri(phase$i) $fmri(phase$i)

# Stop (EV $i)
set fmri(stop$i) $fmri(stop$i)"
		    }
		    1 { 
			puts $channel "
# Skip (EV $i)
set fmri(skip$i) $fmri(skip$i)

# Period (EV $i)
set fmri(period$i) $fmri(period$i)

# Phase (EV $i)
set fmri(phase$i) $fmri(phase$i)

# Sinusoid harmonics (EV $i)
set fmri(nharmonics$i) $fmri(nharmonics$i) 

# Stop (EV $i)
set fmri(stop$i) $fmri(stop$i)"
		    }
		    2 { puts $channel "
# Custom EV file (EV $i)
set fmri(custom$i) \"$fmri(custom$i)\"" }
		    3 { puts $channel "
# Custom EV file (EV $i)
set fmri(custom$i) \"$fmri(custom$i)\"" }
		    4 {
			for { set j 1 } { $j < $i } { incr j 1 } {
			    puts $channel "
# Interactions (EV $i with EV $j)
set fmri(interactions${i}.$j) $fmri(interactions${i}.$j)"
			}
		    }
		}
		
		switch $fmri(convolve$i) {
		    0 { }
		    1 { 
			puts $channel "
# Gauss sigma (EV $i)
set fmri(gausssigma$i) $fmri(gausssigma$i)

# Gauss delay (EV $i)
set fmri(gaussdelay$i) $fmri(gaussdelay$i)"
		    }
		    2 {
			puts $channel "
# Gamma sigma (EV $i)
set fmri(gammasigma$i) $fmri(gammasigma$i)

# Gamma delay (EV $i)
set fmri(gammadelay$i) $fmri(gammadelay$i)"
		    }
		    3 { }
		    4 {
			puts $channel "
# Gamma basis functions number (EV $i)
set fmri(basisfnum$i) $fmri(basisfnum$i)

# Gamma basis functions window(s) (EV $i)
set fmri(basisfwidth$i) $fmri(basisfwidth$i)"
                    }
		    5 {
			puts $channel "
# Sine basis functions number (EV $i)
set fmri(basisfnum$i) $fmri(basisfnum$i)

# Sine basis functions window(s) (EV $i)
set fmri(basisfwidth$i) $fmri(basisfwidth$i)"
                    }
		    6 {
			puts $channel "
# FIR basis functions number (EV $i)
set fmri(basisfnum$i) $fmri(basisfnum$i)

# FIR basis functions window(s) (EV $i)
set fmri(basisfwidth$i) $fmri(basisfwidth$i)"
                    }
		}
		
		puts $channel "
# Orthogonalise EV $i
set fmri(ortho${i}.0) $fmri(ortho${i}.0)"
                if { $fmri(ortho${i}.0) == 1 } {
		    for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
			puts $channel "
# Orthogonalise EV $i wrt EV $j
set fmri(ortho${i}.$j) $fmri(ortho${i}.$j)"
                    }
		}

		if { $fmri(level) > 1 } {
		    for { set j 1 } { $j <= $fmri(npts) } { incr j 1 } {
			puts $channel "
# Higher-level EV value for EV $i and input $j
set fmri(evg${j}.$i) $fmri(evg${j}.$i)"
		    }
		}
		
	    }
	}

	if { $fmri(level) > 1 } {
	    puts $channel "
# Number of higher-level groups
set fmri(ngroups) $fmri(ngroups)"

            for { set j 1 } { $j <= $fmri(multiple) } { incr j 1 } {
		puts $channel "
# Group membership for input $j
set fmri(groupmem.$j) $fmri(groupmem.$j)"
            }
	}

#}}}
    #{{{ contrasts & F-tests

puts $channel "
# Contrast & F-tests mode
# real : control real EVs
# orig : control original EVs
set fmri(con_mode_old) $fmri(con_mode)
set fmri(con_mode) $fmri(con_mode)"

if { $fmri(level) == 1 } {
    set modes "real orig"
} else {
    set modes real
}

foreach cm $modes {

    for { set i 1 } { $i <= $fmri(ncon_${cm}) } { incr i 1 } {

	puts $channel "
# Display images for contrast_${cm} $i
set fmri(conpic_${cm}.$i) $fmri(conpic_${cm}.$i)

# Title for contrast_${cm} $i
set fmri(conname_${cm}.$i) \"$fmri(conname_${cm}.$i)\""

        for { set j 1 } { $j <= $fmri(evs_${cm}) } { incr j 1 } {
	    puts $channel "
# Real contrast_${cm} vector $i element $j
set fmri(con_${cm}${i}.${j}) $fmri(con_${cm}${i}.${j})"
        }

	for { set j 1 } { $j <= $fmri(nftests_${cm}) } { incr j 1 } {
	    puts $channel "
# F-test $j element $i
set fmri(ftest_${cm}${j}.${i}) $fmri(ftest_${cm}${j}.${i})"
        }

    }

}

puts $channel "
# Contrast masking - use >0 instead of thresholding?
set fmri(conmask_zerothresh_yn) $fmri(conmask_zerothresh_yn)"

set total [ expr $fmri(ncon_real) + $fmri(nftests_real) ]
set fmri(conmask1_1) 0
for { set c 1 } { $c <= $total } { incr c } {
    for { set C 1 } { $C <= $total } { incr C } {
	if { $C != $c } {
	    if { ! [ info exists fmri(conmask${c}_${C}) ] } {
		set fmri(conmask${c}_${C}) 0
	    }
	    puts $channel "
# Mask real contrast/F-test $c with real contrast/F-test $C?
set fmri(conmask${c}_${C}) $fmri(conmask${c}_${C})"

	    if { $fmri(conmask${c}_${C}) } {
		set fmri(conmask1_1) 1
	    }
	}
    }
}

puts $channel "
# Do contrast masking at all?
set fmri(conmask1_1) $fmri(conmask1_1)"

#}}}

    close $channel
    
    if { $w != -1 } {
	set result 0
	if { $feat_model } {

	    set result [ catch { exec ${FSLDIR}/bin/feat_model $filename } ErrMsg ]
	    if {$result != 0 || [ string length $ErrMsg ] > 0 } {
		    MxPause "Problem with processing the model: $ErrMsg"
	    }
	}

	return $result
    }
}

#}}}
#{{{ feat5:load

proc feat5:load { w full filename } {
    global fmri feat_files unwarp_files initial_highres_files highres_files VERSION

    set version [ exec grep "fmri(version)" $filename | awk {{ print $3 }} - ]

    if { $version > 3.99 } {

	if { $w != -1 } {
	    set level    $fmri(level)
	    set analysis $fmri(analysis)
	    set multiple $fmri(multiple)
	    set reg_yn   $fmri(reg_yn)
	    for { set i 1 } { $i <= $multiple } { incr i 1 } {
		if { [ info exists feat_files($i) ] } {
		    set lfeat_files($i) $feat_files($i)
		}
	    }
	}

	source ${filename}

        #{{{ process pre-version-5 design file

if { $fmri(version) < 4.999 } {

    set fmri(con_mode_old) orig
    set fmri(con_mode)     orig
    set fmri(evs_orig)     $fmri(evs)
    set fmri(ncon_orig)    $fmri(ncon)
    set fmri(nftests_orig) $fmri(nftests)

    for { set i 1 } { $i <= $fmri(ncon_orig) } { incr i 1 } {
        set fmri(conname_orig.$i) C$i
        set fmri(conpic_orig.$i)  $fmri(conpic.$i)
        for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
            set fmri(con_orig$i.$j) $fmri(con$i.$j)
        }
    }

    for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {

        set fmri(convolve_phase$j) 0

        set fmri(ortho${j}.0) 0
        for { set i 1 } { $i < $j } { incr i 1 } {
            if { $fmri(ortho${j}.$i) == 1 } {
                set fmri(ortho${j}.0) 1
            }
        }

	for { set i 1 } { $i <= $fmri(nftests_orig) } { incr i 1 } {
	    set fmri(ftest_orig$i.$j) $fmri(ftest$i.$j)
	}
    }

}

#}}}

	if { $w != -1 && $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
	    feat5:updateimageinfo $w 1
	}

	if { ! $full } {
	    if { $level != $fmri(level) } {
		feat5:updatelevel $w 0
		set fmri(level) $level
	    }

	    set fmri(npts) [ expr $fmri(npts) - $fmri(ndelete) ]
	    set fmri(ndelete) 0
	    set fmri(reg_yn)   $reg_yn
	    set fmri(analysis) $analysis
	    set fmri(multiple) $multiple
	    for { set i 1 } { $i <= $multiple } { incr i 1 } {
		set feat_files($i) $lfeat_files($i)
	    }
	}

	if { $w != -1 } {
	    feat5:updateanalysis $w 0
	    feat5:updatehelp $w
	    feat5:updatemc $w 0
	}

    } else {
	MxPause "FEAT setup file is too old to load - sorry!"
    }
    
    set fmri(version) $VERSION
}

#}}}
#{{{ feat5:film_wizard

proc feat5:film_wizard { w } {
    global FSLDIR fmri

    #{{{ setup window

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0

    wm title $w0 "FILM simple model setup"
    wm iconname $w0 "FILM"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f
    pack $w0.f

#}}}
    #{{{ choose paradigm type

frame $w0.f.paradigm_type

set fmri(wizard_type) 1	

radiobutton $w0.f.paradigm_type.jab -text "ABAB..." -value 1 -variable fmri(wizard_type) -command "feat5:update_film_wizard $w0"
radiobutton $w0.f.paradigm_type.jabac -text "ABACABAC..." -value 2 -variable fmri(wizard_type) -command "feat5:update_film_wizard $w0"

$w.bhelp bind $w0.f.paradigm_type -msg "Choose whether to setup ABAB... or ABACABAC... designs (block or
single-event). The A blocks will normally be rest (or control)
conditions."

pack $w0.f.paradigm_type.jab $w0.f.paradigm_type.jabac -in $w0.f.paradigm_type -side top -padx 5 -side left

#}}}
    #{{{ frame ab

frame $w0.f.ab

set fmri(a_count) 30
tixControl $w0.f.ab.a_count -label "A (rest) period (s)" \
	-variable fmri(a_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

set fmri(b_count) 30
tixControl $w0.f.ab.b_count -label "B period (s)" \
	-variable fmri(b_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

set fmri(c_count) 30
tixControl $w0.f.ab.c_count -label "C period (s)" \
	-variable fmri(c_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

pack $w0.f.ab.a_count $w0.f.ab.b_count -in $w0.f.ab -side left -padx 5 -pady 3 -anchor n

$w.bhelp bind $w0.f.ab.a_count -msg "The A period (seconds) is normally the rest period, i.e., no
stimulation was applied during this period."

$w.bhelp bind $w0.f.ab.b_count -msg "The B period (seconds) is normally the activation period, i.e.,
stimulation was applied during this period."

$w.bhelp bind $w0.f.ab.c_count -msg "The C period (seconds) is normally the period associated with a second
type of activation, i.e., stimulation type 2 was applied during this
period."

#}}}
    #{{{ setup button

button $w0.f.cancel -command "feat5:updatestats $w 1 -1 ; destroy $w0" -text "Process"

#}}}
 
    pack $w0.f.paradigm_type $w0.f.ab $w0.f.cancel -in $w0.f -pady 5
}

#{{{ feat5:update_film_wizard

proc feat5:update_film_wizard { w } {
    global fmri

    if { $fmri(wizard_type) == 1 } {
	set fmri(a_count) 30
	set fmri(b_count) 30
	pack forget $w.f.ab.c_count
    } else {
	set fmri(a_count) 30
	set fmri(b_count) 30
	set fmri(c_count) 30
	pack $w.f.ab.c_count -in $w.f.ab -after $w.f.ab.b_count -side left -padx 5 -pady 3 -anchor n
    }
}

#}}}

#}}}
#{{{ feat5:relative (DELETE)

proc feat5:relative { input_1 input_2 output_1 } {

    # find beginning of different substring
    for { set start 1 } { $start < [ string length $input_1 ] && $start < [ string length $output_1 ] } { incr start 1 } {
	if { [ string range $input_1  $start $start ] != [ string range $output_1 $start $start ] } {
	    break
	}
    }

    # find end of different substring
    for { set end 1 } { $end < [ string length $input_1 ] && $end < [ string length $output_1 ] } { incr end 1 } {
	if { [ string range $input_1 [ expr [ string length $input_1 ] - $end - 1 ] [ expr [ string length $input_1 ] - $end - 1 ] ] != \
		[ string range $output_1 [ expr [ string length $output_1 ] - $end - 1 ] [ expr [ string length $output_1 ] - $end - 1 ] ] } {
	    break
	}
    }

    # set substrings_1a  [ string range $input_1  0 [ expr $start - 1 ] ]
    set substrings_1bi [ string range $input_1  $start [ expr [ string length $input_1  ] - $end - 1 ] ]
    set substrings_1bo [ string range $output_1 $start [ expr [ string length $output_1 ] - $end - 1 ] ]
    # set substrings_1c  [ string range $input_1  [ expr [ string length $input_1 ] - $end ] [ expr [ string length $input_1 ] - 1 ] ]
    
    regsub $substrings_1bi $input_2 $substrings_1bo output_2

    return $output_2
}

proc feat5:relativetest { } {

feat5:relative /data/heidi/ac/ac_left.hdr /data/heidi/ac/ac_right.hdr /data/heidi/ac/ac_struct_brain.hdr
feat5:relative /data/heidi/ac/ac_left.hdr /data/heidi/at/at_left.hdr /data/heidi/ac/ac_struct_brain.hdr
feat5:relative /data/heidi/ac/ac_left.hdr /data/heidi/at/at_right.hdr /data/heidi/ac/ac_struct_brain.hdr
feat5:relative /data/heidi/ac/fmri.hdr /data/heidi/at/fmri.hdr /data/heidi/ac/struct_brain.hdr
feat5:relative /data/heidi/fmri/ac.hdr /data/heidi/fmri/at.hdr /data/heidi/struct/ac.hdr
feat5:relative /grot/s1_fmri.hdr /grot/s2_fmri.hdr /grot/s1_struct.hdr

}

#}}}
#{{{ feat5:setup_model

proc feat5:setup_model { w } {
    global FSLDIR fmri VARS PWD

    #{{{ setup window

    if { [ info exists fmri(w_model) ] } {
	if { [ winfo exists $fmri(w_model) ] } {
	    return 0
	}
    }

    set count 0
    set w0 ".wdialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".wdialog[incr count]"
    }

    set fmri(w_model) $w0

    toplevel $w0

    wm title $w0 "General Linear Model"
    wm iconname $w0 "GLM"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f
    pack $w0.f -expand yes -fill both -in $w0 -side top

    canvas $w0.f.viewport -yscrollcommand "$w0.f.ysbar set" -xscrollcommand "$w0.xsbar set"
    scrollbar $w0.xsbar -command "$w0.f.viewport xview" -orient horizontal
    scrollbar $w0.f.ysbar -command "$w0.f.viewport yview" -orient vertical
    frame $w0.f.viewport.f
    $w0.f.viewport create window 0 0 -anchor nw -window $w0.f.viewport.f
    bind $w0.f.viewport.f <Configure> "feat5:scrollform_resize $w0 $w0.f.viewport"
    pack $w0.f.viewport -side left -fill both -expand true -in $w0.f

    set NB $w0.f.viewport.f.nb
    tixNoteBook $NB
    if { $fmri(analysis) != 4 } {
	$NB add evs -label "EVs"
    }
    $NB add contrasts -label "Contrasts & F-tests"
    pack $NB -in $w0.f.viewport.f

#}}}
    #{{{ setup contrasts & F-tests

set fmri(contrastsf) [ $NB subwidget contrasts ]

if { $fmri(level) == 1 } {
    tixOptionMenu $fmri(contrastsf).con_mode -label "Setup contrasts & F-tests for " -command "feat5:setup_model_update_contrasts_mode $w 1 ; feat5:setup_model_update_contrasts $w"
    $fmri(contrastsf).con_mode add command orig -label "Original EVs"
    $fmri(contrastsf).con_mode add command real -label "Real EVs"
    $fmri(contrastsf).con_mode configure -variable fmri(con_mode)
    pack $fmri(contrastsf).con_mode -in $fmri(contrastsf) -side top -anchor w -padx 5 -pady 5
    $w.bhelp bind $fmri(contrastsf).con_mode -msg "For first-level analyses, it is common for the final design matrix to
have a greater number of \"real EVs\" than the \"original\" number; for
example, when using basis functions, each \"original EV\" gives rise
to several \"real EVs\".

Therefore it is possible it many cases for you to setup contrasts and
F-tests with respect to the \"original EVs\", and FEAT will work out
for you what these will be for the final design matrix. For example, a
single \[1\] contrast on an original EV for which basis function HRF
convolutions have been chosen will result in a single \[1\] contrast
for each resulting real EV, and then an F-test across these.

In general you can switch between setting up contrasts and F-tests
with respect to \"Original EVs\" and \"Real EVs\"; though of course if
you fine-tune the contrasts for real EVs and then revert to original
EV setup some settings may be lost.

When you \"View\" the design matrix or press \"Done\" at the end of
setting up the model, an \"Original EVs\" setup will get converted to
the appropriate \"Real EVs\" settings."
}

frame $fmri(contrastsf).num
tixControl $fmri(contrastsf).num.con -label "Contrasts " \
        -variable fmri(ncon_$fmri(con_mode)) -step 1 -min 1 -command "feat5:setup_model_update_contrasts $w"
tixControl $fmri(contrastsf).num.ftests -label "   F-tests " \
        -variable fmri(nftests_$fmri(con_mode)) -step 1 -min 0 -command "feat5:setup_model_update_contrasts $w"
pack $fmri(contrastsf).num.con $fmri(contrastsf).num.ftests -in $fmri(contrastsf).num -side left -anchor n -padx 5 -pady 0
pack $fmri(contrastsf).num -in $fmri(contrastsf) -side top -anchor w -padx 0 -pady 5

feat5:setup_model_update_contrasts $w 0

$w.bhelp bind $fmri(contrastsf).num.con -msg "Each EV (explanatory variable, i.e., waveform) in the design matrix
results in a PE (parameter estimate) image.  This estimate tells you
how strongly that waveform fits the data at each voxel - the higher it
is, the better the fit. For an unblurred square wave input (which will
be scaled in the model from -0.5 to 0.5), the PE image is equivalent to
the \"mean difference image\". To convert from a PE to a t
statistic image, the PE is divided by it's standard error, which is
derived from the residual noise after the complete model has been
fit. The t image is then transformed into a Z statistic via standard
statistical transformation. As well as Z images arising from single
EVs, it is possible to combine different EVs (waveforms) - for
example, to see where one has a bigger effect than another. To do
this, one PE is subtracted from another, a combined standard error is
calculated, and a new Z image is created.

All of the above is controlled by you, by setting up contrasts. Each
output Z statistic image is generated by setting up a contrast vector;
thus set the number of outputs that you want, using \"Number of
contrasts\". To convert a single EV into a Z statistic image, set it's
contrast value to 1 and all others to 0. Thus the simplest design,
with one EV only, has just one contrast vector, and only one entry in
this contrast vector; 1. To add more contrast vectors, increase the
\"Number of contrasts\". To compare two EVs, for example, to subtract
one stimulus type (EV1) from another type (EV2), set EV1's contrast
value to -1 and EV2's to 1. A Z statistic image will be generated
according to this request."

$w.bhelp bind $fmri(contrastsf).num.ftests -msg "F-tests enable you to investigate several contrasts at the same time,
for example to see whether any of them (or any combination of them) is
significantly non-zero. Also, the F-test allows you to compare the
contribution of each contrast to the model and decide on significant
and non-significant ones.

One example of F-test usage is if a particular stimulation is to be
represented by several EVs, each with the same input function
(e.g. square wave or custom timing) but all with different HRF
convolutions - i.e. several \"basis functions\". Putting all relevant
resulting parameter estimates together into an F-test allows the
complete fit to be tested against zero without having to specify the
relative weights of the basis functions (as one would need to do with
a single contrast). So - if you had three basis functions (EVs 1,2 and
3) the wrong way of combining them is a single (T-test) contrast of
\[1 1 1\]. The right way is to make three contrasts \[1 0 0\] \[0 1 0\] and
\[0 0 1\] and enter all three contrasts into an F-test. As
described above, FEAT will automatically do this for you if you set up
contrasts for \"original EVs\" instead of \"real EVs\".

You can carry out as many F-tests as you like. Each test includes the
particular contrasts that you specify by clicking on the appropriate
buttons."

#}}}
    #{{{ setup EVs

if { $fmri(analysis) != 4 } {

    set fmri(evsf) [ $NB subwidget evs ]

    set ev_description ""
    if { $fmri(level) == 1 } {
	set ev_description " original"
    }
    tixControl $fmri(evsf).evs -label " Number of$ev_description EVs " -variable fmri(evs_orig) -step 1 -min 1 -command "feat5:setup_model_update_evs $w $fmri(evsf) 1"
    $w.bhelp bind $fmri(evsf).evs -msg "The basic number of explanatory variables in the design matrix; this
means the number of different effects that you wish to model - one for
each modelled stimulation type, and one for each modelled confound.

For first-level analyses, it is common for the final design matrix to
have a greater number of \"real EVs\" than this \"original\" number; for
example, when using basis functions, each \"original EV\" gives rise
to several \"real EVs\"."
    pack $fmri(evsf).evs -in $fmri(evsf) -padx 2 -pady 2 -side top -anchor w

    if { $fmri(level) == 1 } {
	tixNoteBook $fmri(evsf).evsnb
	pack $fmri(evsf).evsnb -in $fmri(evsf) -padx 2 -pady 2 -side top -anchor w
    } else {
	tixControl $fmri(evsf).ngroups -label " Number of groups " -variable fmri(ngroups) -step 1 -min 1 -command "feat5:setup_model_update_evs $w $fmri(evsf) 1"
	$w.bhelp bind $fmri(evsf).ngroups -msg "How many groups of subjects (or sessions, etc.) are there in this higher-level analysis?"
        pack $fmri(evsf).ngroups -in $fmri(evsf) -padx 2 -pady 2 -side top -anchor w
    }

    feat5:setup_model_update_evs $w $fmri(evsf) 0 1

} else {
    feat5:setup_model_update_evs $w 0 0 0
}

#}}}
    #{{{ setup buttons

frame $w0.btns
pack $w0.btns -padx 2 -pady 2 -in $w0 -side bottom

frame $w0.btns.b -relief raised -borderwidth 2
pack $w0.btns.b -in $w0.btns -side bottom -fill x -padx 2 -pady 2

button $w0.btns.b.view -command "feat5:setup_model_preview $w" -text "View design"
$w.bhelp bind $w0.btns.b.view -msg $fmri(design_help)

set fmri(cov_help) "This is a graphical representation of the covariance matrix of the
design matrix.

The first matrix shows the absolute value of the normalised
correlation of each EV with each EV. If a design is well-conditioned
(i.e. not approaching rank deficiency) then the diagonal elements
should be white and all others darker.

So - if there are any very bright elements off the diagonal, you can
immediately tell which EVs are too similar to each other - for
example, if element \[1,3\] (and \[3,1\]) is bright then columns 1 and
3 in the design matrix are possibly too similar.

Note that this includes all real EVs, including any added temporal
derivatives, basis functions, etc.

The second matrix shows a similar thing after the design matrix has
been run through SVD (singular value decomposition). All non-diagonal
elements will be zero and the diagonal elements are given by the
eigenvalues of the SVD, so that a poorly-conditioned design is obvious
if any of the diagonal elements are black."

button $w0.btns.b.acview -command "feat5:setup_model_acpreview $w" -text "Covariance"
$w.bhelp bind $w0.btns.b.acview -msg $fmri(cov_help)

button $w0.btns.b.cancel -command "feat5:setup_model_destroy $w $w0" -text "Done"
pack $w0.btns.b.view $w0.btns.b.acview $w0.btns.b.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
}

#{{{ feat5:setup_model_vars_simple

proc feat5:setup_model_vars_simple { w } {
    global fmri

    set fmri(filmsetup) 1

    if { $fmri(level) == 1 } {

	set fmri(con_mode) orig
	set fmri(con_mode_old) orig

	if { $fmri(wizard_type) == 1 } {
	    #{{{ setup wizard type 1

	set fmri(evs_orig) 1
	set fmri(evs_real) 2

	set fmri(shape1) 0
	set fmri(skip1) 0
	set fmri(off1) $fmri(a_count)
	set fmri(on1) $fmri(b_count)
	set fmri(phase1) 0
	set fmri(stop1) -1

	set fmri(convolve1) 2
	set fmri(convolve_phase1) 0
	set fmri(gammasigma1) 3
	set fmri(gammadelay1) 6
	set fmri(tempfilt_yn1) 1
	set fmri(deriv_yn1) 1

	set fmri(ortho1.0) 0

	set fmri(ncon_orig) 1
	set fmri(con_orig1.1) 1
	set fmri(conpic_orig.1) 1
	set fmri(conname_orig.1) ""

	set fmri(nftests_orig) 0

	set fmri(paradigm_hp) [ expr 1.5 * ( $fmri(a_count) + $fmri(b_count) ) ]

#}}}
	} else {
	    #{{{ setup wizard type 2

	set fmri(evs_orig) 2
	set fmri(evs_real) 4

	set fmri(shape1) 0
	set fmri(shape2) 0
	set fmri(skip1) 0
	set fmri(skip2) 0
	set fmri(off1) [ expr $fmri(a_count) * 2 + $fmri(c_count) ]
	set fmri(on1) $fmri(b_count)
	set fmri(off2) [ expr $fmri(a_count) * 2 + $fmri(b_count) ]
	set fmri(on2) $fmri(c_count)
	set fmri(phase1) [ expr $fmri(a_count) + $fmri(c_count) ]
	set fmri(phase2) 0
	set fmri(stop1) -1
	set fmri(stop2) -1
	set fmri(convolve1) 2
	set fmri(convolve2) 2
	set fmri(convolve_phase1) 0
	set fmri(convolve_phase2) 0
	set fmri(gammasigma1) 3
	set fmri(gammasigma2) 3
	set fmri(gammadelay1) 6
	set fmri(gammadelay2) 6
	set fmri(tempfilt_yn1) 1
	set fmri(tempfilt_yn2) 1
	set fmri(deriv_yn1) 1
	set fmri(deriv_yn2) 1

	set fmri(ortho1.0) 0
	set fmri(ortho2.0) 0
	
	set fmri(ncon_orig) 4
	set fmri(con_orig1.1) 1
	set fmri(con_orig1.2) 0
	set fmri(con_orig2.1) 0
	set fmri(con_orig2.2) 1
	set fmri(con_orig3.1) 1
	set fmri(con_orig3.2) -1
	set fmri(con_orig4.1) -1
	set fmri(con_orig4.2) 1

	set fmri(conpic_orig.1) 1
	set fmri(conpic_orig.2) 1
	set fmri(conpic_orig.3) 1
	set fmri(conpic_orig.4) 1

	set fmri(conname_orig.1) ""
	set fmri(conname_orig.2) ""
	set fmri(conname_orig.3) ""
	set fmri(conname_orig.4) ""

	set fmri(nftests_orig) 1
	set fmri(ftest_orig1.1) 1
	set fmri(ftest_orig1.2) 1
	set fmri(ftest_orig1.3) 0
	set fmri(ftest_orig1.4) 0

	set fmri(paradigm_hp) [ expr 1.5 * ( ( 2 * $fmri(a_count) ) + $fmri(b_count) + $fmri(c_count) ) ]

#}}}
	}

    } else {

	#{{{ setup higher-level starting values

	set fmri(tr) 3
	set fmri(evs_orig) 1
	set fmri(evs_real) 1
	set fmri(custom1) dummy
	set fmri(shape1) 2
	set fmri(convolve1) 0    
	set fmri(tempfilt_yn1) 0
	set fmri(deriv_yn1) 0
	set fmri(ortho1.0) 0
	set fmri(ncon_real) 1
	set fmri(con_mode) real
	set fmri(con_mode_old) real
	set fmri(con_real1.1) 1
	set fmri(conpic_real.1) 1
	set fmri(nftests_real) 0

#}}}

    }
}

#}}}
#{{{ feat5:setup_model_update_ev_i

proc feat5:setup_model_update_ev_i { w w0 i force dummy } {
    global fmri

    if { $fmri(level)==1 } {

	#{{{ basic shape/timings stuff

pack forget $w0.evsnb.skip$i $w0.evsnb.off$i $w0.evsnb.on$i $w0.evsnb.phase$i $w0.evsnb.stop$i $w0.evsnb.period$i $w0.evsnb.nharmonics$i $w0.evsnb.custom$i $w0.evsnb.interaction$i

if { $fmri(shape$i) == 0 } {

    pack $w0.evsnb.skip$i $w0.evsnb.off$i $w0.evsnb.on$i $w0.evsnb.phase$i $w0.evsnb.stop$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w

} elseif { $fmri(shape$i) == 1 } {

    pack $w0.evsnb.skip$i $w0.evsnb.period$i $w0.evsnb.phase$i $w0.evsnb.nharmonics$i $w0.evsnb.stop$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w

} elseif { $fmri(shape$i) == 2 || $fmri(shape$i) == 3 } {

    pack $w0.evsnb.custom$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w

} elseif { $fmri(shape$i) == 4 } {

    pack $w0.evsnb.interaction$i -in $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w
    set selected 0
    for { set j 1 } { $j < $i } { incr j 1 } {
	set selected [ expr $selected + $fmri(interactions${i}.$j) ]
    }
    if { $selected < 2 } {
	if { !$fmri(interactions${i}.1) } {
	    set fmri(interactions${i}.1) 1
	} else {
	    set fmri(interactions${i}.2) 1
	}
    }

}

if { $force } {

    if { $fmri(shape$i) == 1 } {
	set fmri(phase$i) -6
    } else {
	set fmri(phase$i) 0
    }

    if { $fmri(shape$i) != 4 } {
	set fmri(convolve$i) 2
	set fmri(tempfilt_yn$i) 1
    }
}

#}}}

	if { $fmri(shape$i) != 1 && $fmri(shape$i) != 4 } {
	    pack $w0.evsnb.conv$i -in $fmri(modelf$i) -after $w0.evsnb.timings$i -padx 5 -pady 2 -side top -anchor w
	    pack forget $w0.evsnb.convolve_phase$i $w0.evsnb.gausssigma$i $w0.evsnb.gaussdelay$i \
		    $w0.evsnb.gammasigma$i $w0.evsnb.gammadelay$i $w0.evsnb.basisfnum$i $w0.evsnb.basisfwidth$i 
	    if { $fmri(convolve$i) > 0 } {
		pack $w0.evsnb.convolve_phase$i -in $w0.evsnb.conv$i -padx 5 -pady 2 -side top -anchor w
		if { $fmri(convolve$i) == 1 } {
		    pack $w0.evsnb.gausssigma$i $w0.evsnb.gaussdelay$i -in $w0.evsnb.conv$i -padx 5 -pady 2 -side top -anchor w
		} elseif { $fmri(convolve$i) == 2 } {
		    pack $w0.evsnb.gammasigma$i $w0.evsnb.gammadelay$i -in $w0.evsnb.conv$i -padx 5 -pady 2 -side top -anchor w
		} elseif { $fmri(convolve$i) > 3 } {
		    pack $w0.evsnb.basisfnum$i $w0.evsnb.basisfwidth$i -in $w0.evsnb.conv$i -padx 5 -pady 2 -side top -anchor w
		}
	    }
	} else {
	    pack forget $w0.evsnb.conv$i
	    set fmri(convolve$i) 0
	}

	# tempfilt goes next (always packed)

	#{{{ orthogonalise

if { $fmri(evs_orig) > 1 } {

    if { ! [ winfo exists $w0.evsnb.ortho$i ] } {

	frame $w0.evsnb.ortho$i

	checkbutton $w0.evsnb.ortho$i.0 -variable fmri(ortho${i}.0) -font $fmri(bold_font) \
		-command "feat5:setup_model_update_ev_i $w $w0 $i 0 0"

	pack $w0.evsnb.ortho$i.0 -in $w0.evsnb.ortho$i -side left

	$w.bhelp bind $w0.evsnb.ortho$i -msg "Orthogonalising an EV with respect to other EVs means that it is
completely independent of the other EVs, i.e. contains no component
related to them. Most sensible designs are already in this form - all
EVs are at least close to being orthogonal to all others. However,
this may not be the case; you can use this facility to force an EV to
be orthogonal to some or all other EVs. This is achieved by
subtracting from the EV that part which is related to the other EVs
selected here.

An example use would be if you had another EV which was a
constant height spike train, and the current EV is derived from this
other one, but with a linear increase in spike height imposed, to
model an increase in response during the experiment for any
reason. You would not want the current EV to contain any component of
the constant height EV, so you would orthogonalise the current EV wrt
the other."

        pack $w0.evsnb.ortho$i -in $fmri(modelf$i) -after $w0.evsnb.tempfilt$i -padx 5 -pady 2 -side top -anchor w
    }

    for { set j 1 } { $j > 0 } { incr j 1 } {
	if { [ winfo exists $w0.evsnb.ortho$i.$j ] } {
	    destroy $w0.evsnb.ortho$i.$j
	} elseif { $j != $i } {
	    set j -10
	}
    }

    if { $fmri(ortho${i}.0) == 1 } {
	$w0.evsnb.ortho$i.0 configure -text "Orthogonalise    wrt EVs "
	for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
	    if { $j != $i } {
		checkbutton $w0.evsnb.ortho$i.$j -text "$j " -font $fmri(bold_font) -variable fmri(ortho${i}.$j)
		pack $w0.evsnb.ortho$i.$j -in $w0.evsnb.ortho$i -side left -padx 0
		if { $fmri(evs_orig) < 3 } {
		    set fmri(ortho${i}.$j) 1
		}
	    }
	}
    } else {
	$w0.evsnb.ortho$i.0 configure -text "Orthogonalise"
    }

} else {

    if { [ winfo exists $w0.evsnb.ortho$i ] } {
	destroy $w0.evsnb.ortho$i $w0.evsnb.ortho$i.0
	set fmri(ortho${i}.0) 0
    }
}

#}}}

	#{{{ deriv

if { $fmri(shape$i) != 1 && $fmri(convolve$i) < 4 } {
    pack $w0.evsnb.deriv$i -in $fmri(modelf$i) -padx 5 -pady 2 -side bottom -anchor w
} else { 
    set fmri(deriv_yn$i) 0
    pack forget $w0.evsnb.deriv$i
}

#}}}
    }

    feat5:setup_model_update_contrasts $w 0
}

#}}}
#{{{ feat5:setup_model_update_evs

proc feat5:setup_model_update_evs { w w0 dummy update_gui } {
    global fmri PWD

    #{{{ initialise variables

    for { set i 1 } { $i <= $fmri(evs_orig) } { incr i 1 } {
	if { $fmri(level) == 1 } {
	    #{{{ initialise variables

if { ! [ info exists fmri(shape$i) ] } {
    set fmri(shape$i) 0
}

if { ! [ info exists fmri(skip$i) ] } {
    set fmri(skip$i) 0
}

if { ! [ info exists fmri(off$i) ] } {
    set fmri(off$i) 30
}

if { ! [ info exists fmri(on$i) ] } {
    set fmri(on$i) 30
}

if { ! [ info exists fmri(phase$i) ] } {
    set fmri(phase$i) 0
}

if { ! [ info exists fmri(stop$i) ] } {
    set fmri(stop$i) -1
}

if { ! [ info exists fmri(period$i) ] } {
    set fmri(period$i) 60
}

if { ! [ info exists fmri(nharmonics$i) ] } {
    set fmri(nharmonics$i) 0
}

if { ! [ info exists fmri(convolve$i) ] } {
    set fmri(convolve$i) 2
}

if { ! [ info exists fmri(convolve_phase$i) ] } {
    set fmri(convolve_phase$i) 0
}

if { ! [ info exists fmri(gausssigma$i) ] } {
    set fmri(gausssigma$i) 2.8
}

if { ! [ info exists fmri(gaussdelay$i) ] } {
    set fmri(gaussdelay$i) 5
}

if { ! [ info exists fmri(gammasigma$i) ] } {
    set fmri(gammasigma$i) 3
}

if { ! [ info exists fmri(gammadelay$i) ] } {
    set fmri(gammadelay$i) 6
}

if { ! [ info exists fmri(basisfnum$i) ] } {
    set fmri(basisfnum$i) 3
}

if { ! [ info exists fmri(basisfwidth$i) ] } {
    set fmri(basisfwidth$i) 15
}

if { ! [ info exists fmri(tempfilt_yn$i) ] } {
    set fmri(tempfilt_yn$i) 1
}

if { ! [ info exists fmri(deriv_yn$i) ] } {
    set fmri(deriv_yn$i) 1
}

if { ! [ info exists fmri(interactions${i}.1) ] && $i > 2 } {
    for { set j 1 } { $j < $i } { incr j 1 } {
	if { $j < 3 } {
	    set fmri(interactions${i}.$j) 1
	} else {
	    set fmri(interactions${i}.$j) 0
	}
    }
}

for { set j 0 } { $j <= $fmri(evs_orig) } { incr j 1 } {
    if { ! [ info exists fmri(ortho${i}.$j) ] } {
	set fmri(ortho${i}.$j) 0
    }
}

#}}}
	} else {
	    set fmri(custom$i) dummy
	    set fmri(shape$i) 2
	    set fmri(convolve$i) 0    
	    set fmri(convolve_phase$i) 0
	    set fmri(tempfilt_yn$i) 0
	    set fmri(deriv_yn$i) 0
	    set fmri(ortho${i}.0) 0
	    for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
		if { ! [ info exists fmri(evg$i.$j) ] } {
		    set fmri(evg$i.$j) 0
		}
	    }
	}
    }

#}}}

    if { $update_gui } {

	if { $fmri(level) == 1 } {
	    #{{{ setup EV notebook for level=1

	set LWIDTH 15

	for { set i 1 } { $i > 0 } { incr i 1 } {
	    if { $i <= $fmri(evs_orig) } {

		if { ! [ info exists fmri(modelf$i) ] || ! [ winfo exists $fmri(modelf$i) ] } {
		    #{{{ setup EV tab i

$w0.evsnb add ev$i -label "$i"
set fmri(modelf$i) [ $w0.evsnb subwidget ev$i ]

if { $fmri(level) == 1 } {

    #{{{ basic shape

    set grot $fmri(shape$i)

    tixOptionMenu $w0.evsnb.shape$i -label "Basic shape: " -variable fmri(shape$i)
    $w0.evsnb.shape$i add command 0 -label "Square"
    $w0.evsnb.shape$i add command 1 -label "Sinusoid"
    $w0.evsnb.shape$i add command 2 -label "Custom (1 entry per volume)"
    $w0.evsnb.shape$i add command 3 -label "Custom (3 column format)"
    if { $i > 2 } {
	$w0.evsnb.shape$i add command 4 -label "Interaction"
    }

    set fmri(shape$i) $grot

$w.bhelp bind $w0.evsnb.shape$i -msg "Choose the basic shape of the waveform that describes the stimulus or
confound that you wish to model. The basic waveform should be exactly
in time with the applied stimulation, i.e., not lagged at all. This is
because the measured (time-series) response will be delayed with
respect to the stimulation, and this delay is modelled in the design
matrix by convolution of the basic waveform with a suitable
haemodynamic response function (see appropriate bubble-help).

For an on/off (or a regularly-spaced single-event) experiment choose a
\"square\" wave. To model single-event experiments with this method,
the \"On\" periods will probably be small - e.g., 1s or even less.

For sinusoidal modelling choose the \"Sinusoid\" option and select the
number of \"Harmonics\" (or overtones) that you want to add to the
fundamental frequency.

For a single-event experiment with irregular timing for the
stimulations, a custom file can be used. With \"Custom (1 entry per
volume)\", you specify a single value for each timepoint. The custom
file should be a raw text file, and should be a list of numbers, separated by
spaces or newlines, with one number for each volume (after subtracting
the number of deleted images). These numbers can either all be 0s and
1s, or can take a range of values. The former case would be
appropriate if the same stimulus was applied at varying time points;
the latter would be appropriate, for example, if recorded subject
responses are to be inserted as an effect to be modelled. Note that it
may or may not be appropriate to convolve this particular waveform
with an HRF - in the case of single-event, it is.

For even finer control over the input waveform, choose \"Custom (3
column format)\". In this case the custom file consists of triplets of
numbers; you can have any number of triplets. Each triplet describes
a short period of time and the value of the model during that
time. The first number in each triplet is the onset (in seconds) of
the period, the second number is the duration (in seconds) of the
period, and the third number is the value of the input during that
period. The same comments as above apply, about whether these numbers
are 0s and 1s, or vary continuously. The start of the first
non-deleted volume correpsonds to t=0.

Note that whilst ALL columns are demeaned before model fitting,
neither custom format will get rescaled - it is up to you to make sure
that relative scaling between different EVs is sensible. If you double
the scaling of values in an EV you will halve the resulting parameter
estimate, which will change contrasts of this EV against others.

If you select \"Interaction\" then you can set which EVs this EV
models the interaction between. This EV is produced by multiplying
together selected EVs, and allows the modelling of the non-additive
interaction between the selected EVs. This requires that the EVs are
sometimes on at the same time, and sometimes on separately."

#}}}
    #{{{ timings / custom name

frame $w0.evsnb.timings$i

tixControl $w0.evsnb.skip$i -variable fmri(skip$i) -step 1 -min 0 -selectmode immediate -label "    Skip (s)"
$w0.evsnb.skip$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.skip$i -msg "The initial period (seconds) before the waveform commences."

tixControl $w0.evsnb.off$i -variable fmri(off$i) -step 1 -min 0 -selectmode immediate -label "    Off (s)"
$w0.evsnb.off$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.off$i -msg "The duration (seconds) of the \"Off\" periods in the square wave."

tixControl $w0.evsnb.on$i -variable fmri(on$i) -step 1 -min 0 -selectmode immediate -label "    On (s)"
$w0.evsnb.on$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.on$i -msg "The duration (seconds) of the \"On\" periods in the square wave."

tixControl $w0.evsnb.phase$i -variable fmri(phase$i) -step 1 -selectmode immediate -label "    Phase (s)"
$w0.evsnb.phase$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.phase$i -msg "The phase shift (seconds) of the waveform. By default, after the \"Skip\" period, the square wave\nstarts with a full \"Off\" period and the \"Sinusoid\" starts by falling from zero. However, the\nwave can be brought forward in time according to the phase shift."

tixControl $w0.evsnb.stop$i -variable fmri(stop$i) -step 1 -min -1 -selectmode immediate -label "    Stop after (s)"
$w0.evsnb.stop$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.stop$i -msg "The active duration (seconds) of the waveform, starting\nafter the \"Skip\" period. \"-1\" means do not stop."

tixControl $w0.evsnb.period$i -variable fmri(period$i) -step 1 -min 0 -selectmode immediate -label "    Period (s)"
$w0.evsnb.period$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.period$i -msg "The period (seconds) of the \"Sinusoid\" waveform."

tixControl $w0.evsnb.nharmonics$i -variable fmri(nharmonics$i) -step 1 -min 0 -selectmode immediate -label "    Harmonics" \
	-command "feat5:setup_model_update_contrasts $w"
$w0.evsnb.nharmonics$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.nharmonics$i -msg "How many harmonics (sine waves with periods of half the primary sine
wave, then quarter, etc) would you like?"

FSLFileEntry $w0.evsnb.custom$i \
	-variable fmri(custom$i) \
	-pattern "*" \
	-directory $PWD \
	-label "    Filename" \
	-title "Select an event file" \
	-width 30 \
	-filterhist VARS(history)

if { $i > 2 } {
    frame $w0.evsnb.interaction$i
    label $w0.evsnb.interaction$i.label -text "Between EVs "
    pack $w0.evsnb.interaction$i.label -in $w0.evsnb.interaction$i -side left -padx 3
    for { set j 1 } { $j < $i } { incr j 1 } {
	checkbutton $w0.evsnb.interaction$i.$j -variable fmri(interactions${i}.$j) -text "$j " -font $fmri(bold_font) -command "feat5:setup_model_update_ev_i $w $w0 $i 0 0"
	pack $w0.evsnb.interaction$i.$j -in $w0.evsnb.interaction$i -side left
    }
}

#}}}

    #{{{ convolution

frame $w0.evsnb.conv$i

set grot $fmri(convolve$i)

tixOptionMenu $w0.evsnb.convolve$i -label "Convolution: " -variable fmri(convolve$i)
$w0.evsnb.convolve$i add command 0 -label "None"
$w0.evsnb.convolve$i add command 1 -label "Gaussian"
$w0.evsnb.convolve$i add command 2 -label "Gamma"
$w0.evsnb.convolve$i add command 3 -label "Double-Gamma HRF"
$w0.evsnb.convolve$i add command 4 -label "Gamma basis functions"
$w0.evsnb.convolve$i add command 5 -label "Sine basis functions"
$w0.evsnb.convolve$i add command 6 -label "FIR basis functions"

set fmri(convolve$i) $grot

$w.bhelp bind $w0.evsnb.convolve$i -msg "The form of the HRF (haemodynamic response function) convolution that
will be applied to the basic waveform. This blurs and delays the
original waveform, in an attempt to match the difference between the
input function (original waveform, i.e., stimulus waveform) and the
output function (measured FMRI haemodynamic response). 

If the original waveform is already in an appropriate form, e.g., was
sampled from the data itself, \"None\" should be selected.

The next three options are all somewhat similar blurring and delaying
functions. \"Gaussian\" is simply a Gaussian kernel, whose width and
lag can be altered. \"Gamma\" is a Gamma variate (in fact a
normalisation of the probability density function of the Gamma
function); again, width and lag can be altered. \"Double-Gamma HRF\" is a
preset function which is a mixture of two Gamma functions - a standard
positive function at normal lag, and a small, delayed, inverted Gamma,
which attempts to model the late undershoot. 

The remaining convolution options setup different \"basis functions\". This means
that the original EV waveform will get convolved by a \"basis set\" of
related but different convolution kernels - a set of Gammas of
different widths and lags, a set of sinusoids of differing frequencies
or a set of finite-impulse-response (FIR) filters (with FIR the
convolution kernel is represented as a set of discrete fixed-width
\"impulses\"). Therefore the \"original EV\" will generate a set of
\"real EVs\", one for each basis function."

pack $w0.evsnb.convolve$i -in $w0.evsnb.conv$i -padx 0 -pady 2 -side top -anchor w

#}}}
    #{{{ convolution parameters

tixControl $w0.evsnb.convolve_phase$i -variable fmri(convolve_phase$i) -step 1 -selectmode immediate -label "    Phase (s)"
$w0.evsnb.convolve_phase$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.convolve_phase$i -msg "This sets the \"Phase\" of the convolution - i.e. phase shifts the
convolved time series. Positive values shift the final time series
earlier in time."

tixControl $w0.evsnb.gausssigma$i -variable fmri(gausssigma$i) -step 0.1 -min 0 -selectmode immediate -label "    Sigma (s)"
$w0.evsnb.gausssigma$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.gausssigma$i -msg "This sets the half-width of the Gaussian smoothing of the
input waveform."

tixControl $w0.evsnb.gaussdelay$i -variable fmri(gaussdelay$i) -step 0.1 -min 0 -selectmode immediate -label "    Peak lag (s)"
$w0.evsnb.gaussdelay$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.gaussdelay$i -msg "This sets the peak lag of the Gaussian smoothing of the
input waveform."

tixControl $w0.evsnb.gammasigma$i -variable fmri(gammasigma$i) -step 0.1 -min 0.01 -selectmode immediate -label "    Stddev (s)"
$w0.evsnb.gammasigma$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.gammasigma$i -msg "This sets the half-width of the Gamma smoothing of the
input waveform."

tixControl $w0.evsnb.gammadelay$i -variable fmri(gammadelay$i) -step 0.1 -min 0.01 -selectmode immediate -label "    Mean lag (s)"
$w0.evsnb.gammadelay$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.gammadelay$i -msg "This sets the mean lag of the Gamma smoothing of the
input waveform."

tixControl $w0.evsnb.basisfnum$i -variable fmri(basisfnum$i) -step 1 -min 1 -selectmode immediate \
	-command "feat5:setup_model_update_contrasts $w" -label "    Number"
$w0.evsnb.basisfnum$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.basisfnum$i -msg "This sets the number of basis functions."

tixControl $w0.evsnb.basisfwidth$i -variable fmri(basisfwidth$i) -step 1 -min 1 -selectmode immediate -label "    Window (s)"
$w0.evsnb.basisfwidth$i subwidget label config -width $LWIDTH
$w.bhelp bind $w0.evsnb.basisfwidth$i -msg "This sets the total period over which the basis functions are spread."

#}}}

    #{{{ temporal filtering

checkbutton $w0.evsnb.tempfilt$i -variable fmri(tempfilt_yn$i) -text "Apply temporal filtering" -font $fmri(bold_font)

$w.bhelp bind $w0.evsnb.tempfilt$i -msg "You should normally apply the same temporal filtering to the model as
you have applied to the data, as the model is designed to look like
the data before temporal filtering was applied. Thus long-time-scale
components in the model will be dealt with correctly."

#}}}
    #{{{ temporal derivative

checkbutton $w0.evsnb.deriv$i -variable fmri(deriv_yn$i) -text "Add temporal derivative" -font $fmri(bold_font) \
	-command "feat5:setup_model_update_contrasts $w 0"

$w.bhelp bind $w0.evsnb.deriv$i -msg "Adding a fraction of the temporal derivative of the blurred original
waveform is equivalent to shifting the waveform slightly in time, in
order to achieve a slightly better fit to the data. Thus adding in the
temporal derivative of a waveform into the design matrix allows a
better fit for the whole model, reducing unexplained noise, and
increasing resulting statistical significances. This option is not
available if you are using basis functions."

#}}}

    pack $w0.evsnb.shape$i $w0.evsnb.timings$i $w0.evsnb.tempfilt$i -in $fmri(modelf$i) -padx 5 -pady 2 -side top -anchor w

    $w0.evsnb.shape$i configure  -command " feat5:setup_model_update_ev_i $w $w0 $i 1"
    $w0.evsnb.convolve$i configure -command " feat5:setup_model_update_ev_i $w $w0 $i 0"

} else {

    for { set j 1 } { $j <= $fmri(multiple) } { incr j 1 } {
	tixControl $w0.evg${i}_${j} -variable fmri(evg${i}.${j})  -label "$j " \
		-step 1 -selectmode immediate -options { entry.width 3 }
	pack $w0.evg${i}_${j} -in $fmri(modelf$i) -side top -anchor w -padx 5 -pady 2
    }

}

#}}}
		} else {
		    $w0.evsnb pageconfigure ev$i -state normal
		}

		feat5:setup_model_update_ev_i $w $w0 $i 0 0

	    } else {
		#{{{ disable nb page

		if { [ info exists fmri(modelf$i) ] && [ winfo exists $fmri(modelf$i) ] } {
		    $w0.evsnb pageconfigure ev$i -state disabled
		} else {
		    set i -10
		}

#}}}
	    }
	}

#}}}
	} else {
	    #{{{ EV grid for level>1

set w1 $fmri(evsf).grid
if { [ winfo exists $w1 ] } {
    destroy $w1
}
frame $w1
pack $w1 -in $fmri(evsf) -side top -anchor w -padx 5 -pady 5

label $w1.grouplabel -text "  Group  "
grid $w1.grouplabel -in $w1 -row 0 -column 1

for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
    label $w1.evlabel${j} -text "EV$j"
    grid $w1.evlabel${j} -in $w1 -row 0 -column [ expr $j + 1 ]
}

for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {

    label $w1.label$i -text "Input $i "
    grid $w1.label$i -in $w1 -row $i -column 0
    $w.bhelp bind $w1.label$i -msg "Input (lower-level FEAT directory) ${i}."

    if { ! [ info exists fmri(groupmem.$i) ] } {
	set fmri(groupmem.$i) 1
    }
    tixControl $w1.groupmem$i -variable fmri(groupmem.$i) -step 1 -min 1 -max $fmri(ngroups) -selectmode immediate -options { entry.width 3 }
    grid $w1.groupmem$i -in $w1 -row $i -column 1
    $w.bhelp bind $w1.groupmem$i -msg "Which group of subjects (or sessions, etc.) is this input a part of?"

    for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
	tixControl $w1.evg${i}_${j} -variable fmri(evg${i}.${j}) -step 1 -selectmode immediate -options { entry.width 3 }
	grid $w1.evg${i}_${j} -in $w1 -row $i -column [ expr $j + 1 ]
	$w.bhelp bind $w1.evg${i}_${j} -msg "Design matrix value for Input (lower-level FEAT directory) $i and EV ${j}."
    }
}

#}}}
	}
    }

    feat5:setup_model_update_contrasts $w 0
}    

#}}}
#{{{ feat5:setup_model_update_contrasts_real_per_orig

proc feat5:setup_model_update_contrasts_real_per_orig { w } {
    global fmri

    # first - are there any basis functions OR sinusoidal harmonics? if not return 0
    set is_simple 1
    for { set i 1 } { $i <= $fmri(evs_orig) } { incr i 1 } {
	if { $fmri(convolve$i) > 3 || $fmri(shape$i) == 1 } {
	    set is_simple 0
	}
    }
    if { $is_simple } {
	return 0
    }

    # if there are make sure the number of BF or harmonics is the same for every original EV
    for { set i 2 } { $i <= $fmri(evs_orig) } { incr i 1 } {
	if { $fmri(shape$i) != $fmri(shape1) || $fmri(evs_real.$i) != $fmri(evs_real.1) } {
	    return -1
	}
    }

    # so we're ok - return the number of BF or harmonics
    return $fmri(evs_real.1)	
}

#}}}
#{{{ feat5:setup_model_update_contrasts_mode

proc feat5:setup_model_update_contrasts_mode { w update_gui } {
    global fmri

    # if called from feat5:write, will have been called with mode=C
    if { ! $update_gui } {
	set fmri(con_mode_old) c
    }

    if { $fmri(con_mode) != $fmri(con_mode_old) } {

	if { $update_gui } {
	    $fmri(contrastsf).num.con    configure -variable fmri(ncon_$fmri(con_mode))
	    $fmri(contrastsf).num.ftests configure -variable fmri(nftests_$fmri(con_mode))
	}

	if { $fmri(con_mode) == "real" || ! $update_gui } {

	    set real_per_orig [ feat5:setup_model_update_contrasts_real_per_orig $w ]
	    if { $real_per_orig == -1 } {
		MxPause "In order to setup contrasts in \"Original EVs\" mode whilst using basis functions or sinusoidal harmonics, all the original EVs must be the same basic shape and generate the same number of real EVs. Please change the EV setup."
		return -1
	    }

	    if { $real_per_orig > 0 } {
		#{{{ do the case of basis functions or sinusoidal harmonics

set fmri(ncon_real)    [ expr $real_per_orig * $fmri(ncon_orig) ]
set fmri(nftests_real) [ expr $fmri(nftests_orig) + $fmri(ncon_orig) ]

# zero all F-tests
for { set Con 1 } { $Con <= $fmri(ncon_real) } { incr Con 1 } {
    for { set F 1 } { $F <= $fmri(nftests_real) } { incr F 1 } {
	set fmri(ftest_real${F}.$Con) 0
    }
}

# set explicitly asked for F-tests
for { set F 1 } { $F <= $fmri(nftests_orig) } { incr F 1 } {
    for { set Con 1 } { $Con <= $fmri(ncon_orig) } { incr Con 1 } {
	if { $fmri(ftest_orig${F}.$Con) == 1 } {
	    for { set con_real_inc 1 } { $con_real_inc <= $real_per_orig } { incr con_real_inc } {
		set fmri(ftest_real${F}.[ expr ( ( $Con - 1 ) * $real_per_orig ) + $con_real_inc ]) 1
	    }
	}
    }
}

# set the rest
set con_real 0
for { set Con 1 } { $Con <= $fmri(ncon_orig) } { incr Con 1 } {

    set ev_real 0
    for { set ev_orig 1 } { $ev_orig <= $fmri(evs_orig) } { incr ev_orig 1 } {
	for { set con_real_inc 1 } { $con_real_inc <= $real_per_orig } { incr con_real_inc } {
	    set fmri(conpic_real.[ expr $con_real + $con_real_inc ]) $fmri(conpic_orig.$Con)
	    set fmri(conname_real.[ expr $con_real + $con_real_inc ]) "$fmri(conname_orig.$Con) ($con_real_inc)"
	    for { set ev_real_inc 1 } { $ev_real_inc <= $real_per_orig } { incr ev_real_inc } {
		if { $con_real_inc == $ev_real_inc } {
		    set fmri(con_real[ expr $con_real + $con_real_inc ].[ expr $ev_real + $ev_real_inc ]) $fmri(con_orig${Con}.$ev_orig)
		} else {
		    set fmri(con_real[ expr $con_real + $con_real_inc ].[ expr $ev_real + $ev_real_inc ]) 0
		}
	    }
	}
	incr ev_real $real_per_orig
    }

    for { set con_real_inc 1 } { $con_real_inc <= $real_per_orig } { incr con_real_inc } {
	set fmri(ftest_real[ expr $Con + $fmri(nftests_orig) ].[ expr $con_real + $con_real_inc ]) 1
    }

    incr con_real $real_per_orig
}

#}}}
	    } else {
		#{{{ do temporal derivates etc.

for { set Con 1 } { $Con <= $fmri(ncon_orig) } { incr Con 1 } {
    set ev_real 1
    for { set ev_orig 1 } { $ev_orig <= $fmri(evs_orig) } { incr ev_orig 1 } {
	set fmri(conpic_real.$Con) $fmri(conpic_orig.$Con)
	set fmri(conname_real.$Con) $fmri(conname_orig.$Con)
	set fmri(con_real${Con}.$ev_real) $fmri(con_orig${Con}.$ev_orig)
	incr ev_real 1
	if { $fmri(deriv_yn$ev_orig) } {
	    set fmri(con_real${Con}.$ev_real) 0
	    incr ev_real 1
	}
    }    

    for { set F 1 } { $F <= $fmri(nftests_orig) } { incr F 1 } {
	set fmri(ftest_real${F}.${Con}) $fmri(ftest_orig${F}.${Con})
    }
}

set fmri(ncon_real) $fmri(ncon_orig)
set fmri(nftests_real) $fmri(nftests_orig)

#}}}
	    }

	}

	set fmri(con_mode_old) $fmri(con_mode)
    }

    return 1
}

#}}}
#{{{ feat5:setup_model_update_contrasts

proc feat5:setup_model_update_contrasts { w dummy } {
    global fmri

    #{{{ setup evs_real etc/

    set fmri(evs_real) 0

    for { set j 1 } { $j <= $fmri(evs_orig) } { incr j 1 } {
	set fmri(evs_real.$j) 1

	incr fmri(evs_real.$j) $fmri(deriv_yn$j)

	if { $fmri(convolve$j) > 3 } {
	    incr fmri(evs_real.$j) [ expr $fmri(basisfnum$j) - 1 ]
	}

	if { $fmri(shape$j) == 1 } {
	    incr fmri(evs_real.$j) $fmri(nharmonics$j)
	}

	incr fmri(evs_real) $fmri(evs_real.$j)
    }

#}}}

    if { $fmri(level) > 1 } {
	set fmri(con_mode) real
    }

    for { set i 1 } { $i <= $fmri(ncon_$fmri(con_mode)) } { incr i 1 } {
	if { ! [ info exists fmri(conpic_$fmri(con_mode).$i) ] } {
	    set fmri(conpic_$fmri(con_mode).$i) 1
	    set fmri(conname_$fmri(con_mode).$i) ""
	}
    }

    #{{{ destroy and recreate grid

set w0 $fmri(contrastsf).congrid

if { [ winfo exists $w0 ] } {
    destroy $w0
}

frame $w0

pack $w0 -in $fmri(contrastsf) -side top -anchor w -padx 5 -pady 5

#}}}
    #{{{ first 3 columns

for { set i 1 } { $i <= $fmri(ncon_$fmri(con_mode)) } { incr i 1 } {

    if { $fmri(con_mode) == "orig" } {
	label $w0.label$i -text "OC$i "
    } else {
	label $w0.label$i -text "C$i "
    }

    grid $w0.label$i -in $w0 -row $i -column 0
    $w.bhelp bind $w0.label$i -msg "Contrast vector number $i - this will result in Z statistic image number $i"

    checkbutton $w0.conpic$i -variable fmri(conpic_$fmri(con_mode).$i)
    grid $w0.conpic$i -in $w0 -row $i -column 1
    $w.bhelp bind $w0.conpic$i -msg "Include contrast $i in web page report? (Turn off if
this contrast is only to be used within F-tests.)"

    entry $w0.conname$i -textvariable fmri(conname_$fmri(con_mode).$i) -width 10
    grid $w0.conname$i -in $w0 -row $i -column 2
}

#}}}
    #{{{ top row and main grid

label $w0.evlabel0 -text "Title"
grid $w0.evlabel0 -in $w0 -row 0 -column 2

set ev_count 1
set ev_counti 1

for { set j 1 } { $j <= $fmri(evs_$fmri(con_mode)) } { incr j 1 } {

    if { $fmri(con_mode) == "real" || $ev_counti == 1 } {

	if { $ev_counti == 1 } {
	    label $w0.evlabel${j} -text "EV$ev_count"
	} else {
	    label $w0.evlabel${j} -text ""
	}
	grid $w0.evlabel${j} -in $w0 -row 0 -column [ expr $j + 2 ]

	for { set i 1 } { $i <= $fmri(ncon_$fmri(con_mode)) } { incr i 1 } {
	    if { ! [ info exists fmri(con_$fmri(con_mode)${i}.${j}) ] } {
		set fmri(con_$fmri(con_mode)${i}.${j}) 0
		if { $i==1 && $j==1 } {
		    set fmri(con_$fmri(con_mode)${i}.${j}) 1
		}
	    }
	    tixControl $w0.con${i}_${j} -variable fmri(con_$fmri(con_mode)${i}.${j}) -step 1 -selectmode immediate -options { entry.width 3 }
	    grid $w0.con${i}_${j} -in $w0 -row $i -column [ expr $j + 2 ]
	    $w.bhelp bind $w0.con${i}_${j} -msg "The weight given to EV$j within contrast vector $i."
	}


    }

    if { $fmri(con_mode) == "real" } {
	incr ev_counti 1
	if { $ev_counti > $fmri(evs_real.$ev_count) } {
	    set ev_counti 1
	    incr ev_count 1
	}
    } else {
	incr ev_count 1
    }
}

#}}}
    #{{{ ftests

label $w0.blank -text "       "
grid $w0.blank -in $w0 -row 0 -column [ expr $fmri(evs_real) + 3 ]

for { set i 1 } { $i <= $fmri(nftests_$fmri(con_mode)) } { incr i 1 } {

    label $w0.flabel$i -text "F$i "
    grid $w0.flabel$i -in $w0 -row 0 -column [ expr $i + $fmri(evs_real) + 3 ]
    $w.bhelp bind $w0.flabel$i -msg "F-test number $i - this will result in F statistic image number $i"
	
    for { set j 1 } { $j <= $fmri(ncon_$fmri(con_mode)) } { incr j 1 } {
	
	if { ! [ info exists fmri(ftest_$fmri(con_mode)${i}.${j}) ] } {
	    set fmri(ftest_$fmri(con_mode)${i}.${j}) 0
	}

	checkbutton $w0.ftest${i}_${j} -variable fmri(ftest_$fmri(con_mode)${i}.$j)
	grid $w0.ftest${i}_${j} -in $w0 -row $j -column [ expr $i + $fmri(evs_real) + 3 ]
	$w.bhelp bind $w0.ftest${i}_${j} -msg "Include contrast vector $j in F-test $i?"
    }
}

#}}}
}

#}}}
#{{{ feat5:setup_model_preview

proc feat5:setup_model_preview { w } {
    global fmri

    set fmri(filmsetup) 1

    set problem [ feat5:write $w 1 0 $fmri(feat_filename) ]
    if { $problem } {
	return 1
    }

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0 -visual truecolor
    wm title $w0 "Model"
    wm iconname $w0 "Model"

    frame $w0.f
    pack $w0.f -expand yes -fill both -in $w0 -side top
    canvas $w0.f.viewport -yscrollcommand "$w0.f.ysbar set" -xscrollcommand "$w0.xsbar set"
    scrollbar $w0.xsbar -command "$w0.f.viewport xview" -orient horizontal
    scrollbar $w0.f.ysbar -command "$w0.f.viewport yview" -orient vertical
    frame $w0.f.viewport.f
    $w0.f.viewport create window 0 0 -anchor nw -window $w0.f.viewport.f
    bind $w0.f.viewport.f <Configure> "feat5:scrollform_resize $w0 $w0.f.viewport"
    pack $w0.f.viewport -side left -fill both -expand true -in $w0.f

    set graphpic [ image create photo -file [ file rootname $fmri(feat_filename) ].ppm ]
    button $w0.f.viewport.f.btn -command "destroy $w0" -image $graphpic -borderwidth 0
    pack $w0.f.viewport.f.btn -in $w0.f.viewport.f

    $w.bhelp bind $w0.f.viewport.f.btn -msg $fmri(design_help)

    return 0
}

#}}}
#{{{ feat5:setup_model_acpreview

proc feat5:setup_model_acpreview { w } {
    global fmri

    if { $fmri(evs_orig) < 2 && $fmri(deriv_yn1) == 0 } {
	return 0
    }

    set problem [ feat5:write $w 1 0 $fmri(feat_filename) ]
    if { $problem } {
	return 1
    }

    set count 0
    set w1 ".dialog[incr count]"
    while { [ winfo exists $w1 ] } {
        set w1 ".dialog[incr count]"
    }

    toplevel $w1 -visual truecolor
    wm title $w1 "Covariance matrix"
    wm iconname $w1 "Covariance"

    set graphpic [ image create photo -file [ file rootname $fmri(feat_filename) ]_cov.ppm ]

    button $w1.btn -command "destroy $w1" -image $graphpic -borderwidth 0
    pack $w1.btn -in $w1

    $w.bhelp bind $w1.btn -msg $fmri(cov_help)

    return 0
}

#}}}
#{{{ feat5:setup_model_destroy

proc feat5:setup_model_destroy { w w0 } {
    global fmri

    if { ! [ feat5:setup_model_preview $w ] } {
	destroy $w0
    }
}

#}}}
#{{{ feat5:setup_conmask

proc feat5:setup_conmask { w } {
    global fmri FSLDIR

    #{{{ setup window

    if { [ info exists fmri(c_model) ] && [ winfo exists $fmri(c_model) ] } {
	return 0
    }

    set count 0
    set w0 ".cdialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".cdialog[incr count]"
    }

    set fmri(c_model) $w0

    toplevel $w0

    wm title $w0 "Setup Contrast Masking"
    wm iconname $w0 "Contrast Masking"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f
    pack $w0.f -expand yes -fill both -in $w0 -side top

    canvas $w0.f.viewport -yscrollcommand "$w0.f.ysbar set" -xscrollcommand "$w0.xsbar set"
    scrollbar $w0.xsbar -command "$w0.f.viewport xview" -orient horizontal
    scrollbar $w0.f.ysbar -command "$w0.f.viewport yview" -orient vertical
    frame $w0.f.viewport.f
    $w0.f.viewport create window 0 0 -anchor nw -window $w0.f.viewport.f
    bind $w0.f.viewport.f <Configure> "feat5:scrollform_resize $w0 $w0.f.viewport"
    pack $w0.f.viewport -side left -fill both -expand true -in $w0.f

    set v $w0.f.viewport.f

#}}}
    #{{{ setup grid

set total [ expr $fmri(ncon_real) + $fmri(nftests_real) ]

for { set C 1 } { $C <= $total } { incr C 1 } {
    if { $C <= $fmri(ncon_real) } {
	label $v.tl$C -text "C$C"
    } else {
	label $v.tl$C -text "F[ expr $C - $fmri(ncon_real)]"
    }
    $w.bhelp bind $v.tl$C -msg $fmri(conmask_help) 
    grid $v.tl$C -in $v -column $C -row 0
}

for { set c 1 } { $c <= $total } { incr c 1 } {

    if { $c <= $fmri(ncon_real) } {
	label $v.l$c -text "Mask real Contrast $c with:    "
    } else {
	label $v.l$c -text "Mask real F-test [ expr $c - $fmri(ncon_real)] with:    "
    }
    $w.bhelp bind $v.l$c -msg $fmri(conmask_help) 
    grid $v.l$c -in $v -column 0 -row $c

    for { set C 1 } { $C <= $total } { incr C 1 } {

	if { $C != $c } {
	    checkbutton $v.cb${c}_$C -variable fmri(conmask${c}_${C})
	    $w.bhelp bind $v.cb${c}_$C -msg $fmri(conmask_help) 
	    grid $v.cb${c}_$C -in $v -column $C -row $c
	}
    }
}

#}}}
    #{{{ setup buttons

checkbutton $w0.zeros -variable fmri(conmask_zerothresh_yn) -text "Mask using (Z>0) instead of (Z stats pass thresholding)" -font $fmri(bold_font)
$w.bhelp bind $w0.zeros -msg $fmri(conmask_help) 

button $w0.ok -command "destroy $w0" -text "OK"

pack $w0.ok $w0.zeros -in $w0 -side bottom -padx 3 -pady 5

#}}}
}

#}}}

#}}}


##### MEDx-specific stuff #####
#{{{ feat5:medxload

proc feat5:medxload { featdir } {
    global PXHOME FSLDIR VARS argc argv PWD entries loadtimeseries

    #{{{ setup main window

set count 0
set w ".dialog[incr count]"
while { [ winfo exists $w ] } {
    set w ".dialog[incr count]"
}

toplevel $w

wm title      $w "Load FEAT results into MEDx"
wm iconname   $w "FEAT Load"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

frame $w.f
pack $w.f -in $w -side top

if { $featdir != 0 } {
    set entries($w,1) $featdir
}

FSLFileEntry $w.featdir \
	-variable entries($w,1) \
	-pattern "*.feat" \
	-directory "~" \
	-label "Select a FEAT directory" \
	-title "Select a FEAT directory" \
	-width 20 \
	-dirasfile "design.fsf" \
	-filterhist VARS(history)

frame $w.timeseries
label $w.timeseries.label -text "Load timeseries data"
set loadtimeseries 1
checkbutton $w.timeseries.yn -variable loadtimeseries
pack $w.timeseries.label $w.timeseries.yn -in $w.timeseries -side left -anchor w

pack $w.featdir $w.timeseries -in $w.f -padx 5 -pady 5 -anchor w

#}}}
    #{{{ Button Frame

frame $w.btns
frame $w.btns.b -relief raised -borderwidth 1
    
button $w.apply -command "feat5:run_medxload_proc $w" -text "Go"

button $w.cancel -command "destroy $w" -text "Exit"

pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
pack $w.apply $w.cancel -in $w.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

pack $w.btns -in $w -side bottom -fill x

#}}}
}

proc feat5:run_medxload_proc { w } {
    global PXHOME FSLDIR entries loadtimeseries
    feat5:medxload_proc $entries($w,1) $loadtimeseries
}

proc feat5:medxload_proc { FD loadtimeseries } {
    #{{{ setup

    global PXHOME FSLDIR

    puts "Loading $FD"

    MxNewFolder folder window

    MxSetFolderProperties $folder "{Name $FD}"

if { ! [ file exists ${FD}/filtered_func_data.hdr ] } {
    set loadtimeseries 0
}

#}}}
    #{{{ load lowres files

catch { exec sh -c "echo ${FD}/example_func.hdr ${FD}/mask.hdr ${FD}/stats/z*stat*hdr ${FD}/stats/fixed*hdr ${FD}/stats/random*hdr ${FD}/cluster_mask_*hdr ${FD}/thresh_*hdr ${FD}/?endered_thresh_*.hdr" } lowreslist

set lowrespagelist ""
foreach lowres $lowreslist {
    if { [ file exists $lowres ] } {
	MxOpenImage $folder $lowres lowrespage
	set lowrespagelist "$lowrespage $lowrespagelist"
    }
}

#}}}
    if { $loadtimeseries } {
	#{{{ load func data

    MxOpenImage $folder ${FD}/filtered_func_data.hdr Group

    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]

    set AList ""
    lappend AList [ lindex $GPages 0 ]
    lappend AList [ lindex $GPages 1 ]
    MxGroupVolumes $AList "A" A

    set BList ""
    set i 2
    while { $i < $total_volumes } {
	lappend BList [ lindex $GPages $i ]
	incr i 1
    }
    MxGroupVolumes $BList "B" B

    MxBetweenGrpStatistics $A $B U blah blah2 blah3 FinalZscore ResultGroup

    MxGetPagesFromGroup $ResultGroup RP
    set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
    MxDeletePage { $RPList }
    MxDeleteGroupPages { $A $B $ResultGroup } No

    MxSetTimeSeriesProperties $Group 1 red

#}}}
    }
    #{{{ process lowres files

foreach lowrespage $lowrespagelist {
    MxSetCurrentPage $lowrespage
    MxGetImageProperties $lowrespage props
    if { $loadtimeseries } {
	MxDupPage $FinalZscore zscore
	MxExpr $zscore = " $lowrespage "
	MxDeletePage $lowrespage
    } else {
	set zscore $lowrespage
    }
    MxComputeDisplayRange $zscore V Min Max
    set thefile [ file rootname [ keylget props File ] ]
    set thename [ keylget props Name ]
    if { $loadtimeseries } {
	name_page $zscore $thename
    }

    if { [ string first endered $thename ] == 1 } {
	set cal_min  [ exec $FSLDIR/bin/avwval $thefile cal_min ]
	set cal_max  [ exec $FSLDIR/bin/avwval $thefile cal_max ]
	set aux_file [ string trimright [ exec $FSLDIR/bin/avwval $thefile aux_file ] " " ]
	MxSetDisplayRange $zscore V $cal_min $cal_max
	MxLoadMaterialMap $zscore ${FSLDIR}/etc/luts/${aux_file}.map
	MxLoadLUT $zscore ${FSLDIR}/etc/luts/${aux_file}.lut
    }
}

if { $loadtimeseries } {
    MxDeletePage $FinalZscore
}

#}}}
}

#}}}


##### FEAT analysis procedures #####
#{{{ feat5:link

proc feat5:link { linkfrom linkto } {

    if { [ file exists ${linkfrom}.hdr ] } {
	fsl:exec "/bin/rm -f ${linkto}.hdr ${linkto}.img"
	fsl:exec "ln -s ${linkfrom}.hdr ${linkto}.hdr"
	fsl:exec "ln -s ${linkfrom}.img ${linkto}.img"
    }

}

#}}}
#{{{ feat5:getconname

proc feat5:getconname { featdir contrastnumber } {
    source ${featdir}/design.fsf
    if { [ info exists fmri(conname_real.$contrastnumber) ] } {
	return $fmri(conname_real.$contrastnumber)
    } else {
	return ""
    }
}

#}}}
#{{{ feat5:connectivity

proc feat5:connectivity { image } {

    global FSLDIR

    set CONNECTIVITY "26"

#    maybe actually USE this test? but needs careful thought.......
#
#    set CONNECTIVITY "6"
#
#    if { [ expr abs([ exec $FSLDIR/bin/avwval $image pixdim3 ]) ] < 3.1 } {
#	set CONNECTIVITY "26"
#    }

    return $CONNECTIVITY
}

#}}}
#{{{ feat5:poststats

proc feat5:poststats { rerunning talspace } {

    global FSLDIR FD report ps rs logout comout fmri

    if { $fmri(thresh) == 0 } {
	return 0
    }

    set maskcomments "<p>"
    
    #{{{ setup raw stats list

cd ${FD}/stats
set rawstatslist "[ lsort -dictionary [ glob -nocomplain zstat*.hdr ] ] [ lsort -dictionary [ glob -nocomplain zfstat*.hdr ] ]"
cd ..

regsub -all ".hdr" $rawstatslist "" rawstatslist

#}}}
    #{{{ setup Talairach/non-Tal thingies

set TALOPT   ""
set TALEXT   ""
set SLICER   "-A"
set VOXorMM  ""

if { $talspace != 0 } {
    set TALOPT   "-tal"
    set TALEXT   "_tal"
    set SLICER   "-S 2"
    set VOXorMM  "--mm"
}

#}}}
    #{{{ brain-mask Z stats and find smoothness

foreach rawstats $rawstatslist {

    if { ! $rerunning } {
	fsl:exec "$FSLDIR/bin/avwmaths stats/$rawstats -mas mask thresh_$rawstats"

	if { $fmri(threshmask) != "" && [ file exists $fmri(threshmask) ] } {
	    fsl:exec "$FSLDIR/bin/avwmaths thresh_$rawstats -mas $fmri(threshmask) thresh_$rawstats"
	    set maskcomments "<p>Z-stat images were masked with $fmri(threshmask) before thresholding.<br>"
	}
    }

    if { [ file exists stats/smoothness ] } {
	set fmri(DLH$rawstats)     [ exec grep DLH    stats/smoothness | awk {{ print $2 }} - ]
	set fmri(RESELS$rawstats)  [ exec grep RESELS stats/smoothness | awk {{ print $2 }} - ]
    } else {
	fsl:exec "$FSLDIR/bin/smoothest -m mask -z stats/$rawstats > stats/${rawstats}.smoothness"
	set fmri(DLH$rawstats)     [ exec grep DLH    stats/${rawstats}.smoothness | awk {{ print $2 }} - ]
	set fmri(RESELS$rawstats)  [ exec grep RESELS stats/${rawstats}.smoothness | awk {{ print $2 }} - ]
    }

    if { ! $rerunning } {
	set fmri(VOLUME$rawstats) [ exec ${FSLDIR}/bin/avwstats thresh_$rawstats -V | awk {{ print $1 }} - ]
	fsl:exec "echo $fmri(VOLUME$rawstats) > thresh_${rawstats}.vol"
    } else {
	if { ! [ info exists fmri(VOLUME$rawstats) ] } {
	    if { [ file exists thresh_${rawstats}.vol ] } {
		set fmri(VOLUME$rawstats) [ exec cat thresh_${rawstats}.vol ]
	    } else {
		set fmri(VOLUME$rawstats) [ exec ${FSLDIR}/bin/avwstats mask -V | awk {{ print $1 }} - ]
	    }
	}
    }

    puts $logout "$rawstats: DLH=$fmri(DLH$rawstats) VOLUME=$fmri(VOLUME$rawstats) RESELS=$fmri(RESELS$rawstats)
"

}

#}}}
    #{{{ thresholding and contrast masking

if { ! $rerunning } {

    set firsttime 1
    foreach rawstats $rawstatslist {
	set i [ string trimleft $rawstats "abcdefghijklmnopqrstuvwxyz_" ]

	if { $firsttime == 1 } {
	    set ps "$ps Z (Gaussianised T/F) statistic images were thresholded"
	}

	if { $fmri(thresh) < 3 } {
	    #{{{ voxel-based thresholding

if { $firsttime == 1 } {
    if { $fmri(thresh) == 1 } {
	set ps "$ps at P=$fmri(prob_thresh) (uncorrected)."
	set zthresh [ fsl:exec "${FSLDIR}/bin/ptoz $fmri(prob_thresh)" ]
    } else {
	set ps "$ps using GRF-theory-based maximum height thresholding with a (corrected) significance threshold of P=$fmri(prob_thresh) \[Forman 1995\]."
	set rs "$rs\[Forman 1995\] S.D Forman, J.D. Cohen, M. Fitzgerald, W. Eddy, M.A. Mintun and D.C. Noll.
	Improved assessment of significant activation in functional magnetic resonance imaging (fMRI): Use of a cluster-size threshold. 
	Magnetic Resonance in Medicine 33(636-647) 1995.<br> 
	"
    }
}

if { $fmri(thresh) == 2 } {
    set zthresh [ fsl:exec "${FSLDIR}/bin/ptoz $fmri(prob_thresh) -g [ expr int ( $fmri(VOLUME$rawstats) / $fmri(RESELS$rawstats) ) ]" ]
}

fsl:exec "$FSLDIR/bin/avwmaths thresh_$rawstats -thr $zthresh thresh_$rawstats"

#}}}
	} else {
	    #{{{ cluster thresholding

if { $firsttime == 1 } {
    set ps "$ps using clusters determined by Z>$fmri(z_thresh) and a (corrected) cluster significance threshold of P=$fmri(prob_thresh) \[Worsley 1992, Friston 1994, Forman 1995\]."
    set rs "$rs\[Friston 1994\] K.J. Friston, K.J. Worsley, R.S.J. Frakowiak, J.C. Mazziotta and A.C. Evans. Assessing the significance of
    focal activations using their spatial extent. Human Brain Mapping 1(214-220) 1994.<br>
    \[Worsley 1992\] K.J. Worsley, A.C. Evans, S. Marrett and P. Neelin. A three-dimensional statistical analysis
    for CBF activation studies in human brain. Journal of Cerebral Blood Flow and Metabolism 12(900-918) 1992.<br> 
    \[Forman 1995\] S.D Forman, J.D. Cohen, M. Fitzgerald, W.F. Eddy, M.A. Mintun and D.C. Noll.
    Improved assessment of significant activation in functional magnetic resonance imaging (fMRI): Use of a cluster-size threshold. 
    Magnetic Resonance in Medicine 33(636-647) 1995.<br> 
    "
}

set COPE ""
if { [ string first "zfstat" $rawstats ] < 0 && [ file exists stats/cope${i}.hdr ] } {
    set COPE "-c stats/cope$i"
}

fsl:exec "$FSLDIR/bin/cluster -i thresh_$rawstats $COPE -t $fmri(z_thresh) -p $fmri(prob_thresh) -d $fmri(DLH$rawstats) --volume=$fmri(VOLUME$rawstats) --othresh=thresh_$rawstats -o cluster_mask_$rawstats --connectivity=[ feat5:connectivity thresh_$rawstats ] $VOXorMM --olmax=lmax_${rawstats}${TALEXT}.txt > cluster_${rawstats}${TALEXT}.txt"

fsl:exec "$FSLDIR/bin/cluster2html . cluster_$rawstats $TALOPT"

#}}}
	}

	set firsttime 0
    }

    #{{{ contrast masking

if { $fmri(conmask1_1) } {

    fsl:exec "mkdir conmask"

    foreach rawstats $rawstatslist {

	set i [ string trimleft $rawstats "abcdefghijklmnopqrstuvwxyz_" ]

	# check for being F-test
	set I $i
	if { [ string first "zstat" $rawstats ] == -1 } {
	    incr I $fmri(ncon_real)
	}

	set theinput thresh_$rawstats

	for { set C 1 } { $C <= [ expr $fmri(ncon_real) + $fmri(nftests_real) ] } { incr C } {
	    if { $C != $I } {

		set F ""
		set c $C
		if { $C > $fmri(ncon_real) } {
		    set F f
		    set c [ expr $C - $fmri(ncon_real) ]
		}

		if { $fmri(conmask${I}_$C) } {
		    
		    set themask thresh_z${F}stat$c
		    if { $fmri(conmask_zerothresh_yn) } {
			set themask stats/z${F}stat$c
		    }

		    puts $logout "Masking $theinput with $themask"

		    set maskcomments "$maskcomments
After all thresholding, $rawstats was masked with $themask.<br>"

		    if { [ file exists ${themask}.hdr ] } {
			fsl:exec "${FSLDIR}/bin/avwmaths $theinput -mas $themask conmask/thresh_$rawstats"
		    } else {
			fsl:exec "${FSLDIR}/bin/avwmaths $theinput -mul 0 conmask/thresh_$rawstats"
		    }

		    set theinput conmask/thresh_$rawstats
		}
		
	    }
	}
    }

    fsl:exec "/bin/mv -f conmask/* . ; rmdir conmask"

    #{{{ redo clustering

if { $fmri(thresh) == 3 } {
    foreach rawstats $rawstatslist {
	set i [ string trimleft $rawstats "abcdefghijklmnopqrstuvwxyz_" ]
	set COPE ""
	if { [ string first "zfstat" $rawstats ] < 0 && [ file exists stats/cope${i}.hdr ] } {
	    set COPE "-c stats/cope$i"
	}

	# we're not going to re-test cluster size so pthresh is set to 1000

	fsl:exec "$FSLDIR/bin/cluster -i thresh_$rawstats $COPE -t $fmri(z_thresh) -d $fmri(DLH$rawstats) --volume=$fmri(VOLUME$rawstats) --othresh=thresh_$rawstats -o cluster_mask_$rawstats --connectivity=[ feat5:connectivity thresh_$rawstats ] $VOXorMM --olmax=lmax_${rawstats}${TALEXT}.txt > cluster_${rawstats}${TALEXT}.txt"

	fsl:exec "$FSLDIR/bin/cluster2html . cluster_$rawstats $TALOPT"
    }
}

#}}}
}

#}}}
}

#}}}
    #{{{ re-run cluster for TalSpace

if { $rerunning && [ file exists reg/example_func2standard.mat ] && $fmri(thresh) == 3 } {

    set z_thresh    $fmri(z_thresh)
    set prob_thresh $fmri(prob_thresh)

    if { $fmri(analysis) == 0 } {
	set z_thresh    [ exec grep "set fmri(z_thresh)"    design.fsf | awk {{ print $3 }} - ]
	set prob_thresh [ exec grep "set fmri(prob_thresh)" design.fsf | awk {{ print $3 }} - ]
    }

    foreach rawstats $rawstatslist {

	set i [ string trimleft $rawstats "abcdefghijklmnopqrstuvwxyz_" ]

	set COPE ""
	if { [ string first "zfstat" $rawstats ] < 0 && [ file exists stats/cope${i}.hdr ] } {
	    set COPE "-c stats/cope$i"
	}

	fsl:exec "$FSLDIR/bin/cluster -i thresh_$rawstats ${COPE} -t $z_thresh -d $fmri(DLH$rawstats) --volume=$fmri(VOLUME$rawstats) -x reg/example_func2standard.mat --talvol=reg/standard --mm --connectivity=[ feat5:connectivity thresh_$rawstats ] --olmax=lmax_${rawstats}_tal.txt > cluster_${rawstats}_tal.txt"
	fsl:exec "$FSLDIR/bin/cluster2html . cluster_${rawstats} -tal"
    }
}

#}}}
    #{{{ rendering

if { ! $rerunning } {

    #{{{ Find group Z min and max

if { $fmri(zdisplay) == 0 } {

    set fmri(zmin) 100000
    set fmri(zmax) -100000

    foreach rawstats $rawstatslist {
    
	set zminmax [ fsl:exec "${FSLDIR}/bin/avwstats thresh_$rawstats -l 0.0001 -R" ]
	set zmin [ lindex $zminmax 0 ]
	set zmax [ lindex $zminmax 1 ]

	# test for non-empty thresh_zstats image
	if { $zmax > 0.0001 } {
	    if { $fmri(zmin) > $zmin } {
		set fmri(zmin) $zmin
	    }
	    if { $fmri(zmax) < $zmax } {
		set fmri(zmax) $zmax
	    }
	}
    }

    if { $fmri(zmin) > 99999 } {
	set fmri(zmin) 2.3
	set fmri(zmax) 8
    }
}

puts $logout "Rendering using zmin=$fmri(zmin) zmax=$fmri(zmax)"

#}}}

    set underlying example_func
    if { ! [ file exists ${underlying}.hdr ] } {
	set underlying mean_highres
    }

    set firsttime 1
    foreach rawstats $rawstatslist {
	
	set i [ string trimleft $rawstats "abcdefghijklmnopqrstuvwxyz_" ]
	
	if { [ string first "zstat" $rawstats ] < 0 || $fmri(conpic_real.$i) == 1 } {
	    #{{{ Rendering

set conname "$rawstats"
if { [ string first "zfstat" $rawstats ] < 0 } {
    set conname "$conname &nbsp;&nbsp;-&nbsp;&nbsp; C${i}"
    if { $fmri(conname_real.$i) != "" } {
	set conname "$conname ($fmri(conname_real.$i))"
    }
} else {
    set conname "$conname &nbsp;&nbsp;-&nbsp;&nbsp; F${i}"

    set start 1
    for { set c 1 } { $c <= $fmri(ncon_real) } { incr c 1 } {
	if { $fmri(ftest_real${i}.${c}) == 1 } {
	    if { $start == 1 } {
		set conname "$conname ("
		set start 0
	    } else {
		set conname "$conname & "
	    }
	    set conname "${conname}C$c"
	}
    }
    set conname "$conname)"
}

fsl:exec "$FSLDIR/bin/overlay $fmri(rendertype) 0 $underlying -a thresh_$rawstats $fmri(zmin) $fmri(zmax) rendered_thresh_$rawstats"
fsl:exec "${FSLDIR}/bin/slicer rendered_thresh_$rawstats $SLICER 750 rendered_thresh_${rawstats}.png"

if { $firsttime == 1 } {
    puts $report "<hr><font size=+2><b>Thresholded activation images</b></font>
&nbsp; &nbsp; &nbsp; &nbsp; 
[ expr int($fmri(zmin)*10)/10.0 ]
<IMG BORDER=0 SRC=\"${FSLDIR}/etc/luts/ramp.gif\">
[ expr int($fmri(zmax)*10)/10.0 ]
$maskcomments
"
    set firsttime 0
}

if { $fmri(thresh) == 3 } {
    puts $report "<p>$conname<br>
    <a href=\"cluster_${rawstats}${TALEXT}.html\"><IMG BORDER=0 SRC=\"rendered_thresh_${rawstats}.png\"></a>
    "
} else {
    puts $report "<p>$conname<br>
    <IMG BORDER=0 SRC=\"rendered_thresh_${rawstats}.png\">
    "
}

#}}}
	}
    }
}

#}}}

}

#}}}
#{{{ feat5:flirt

proc feat5:flirt { in ref dof search interp existing_mats report } {

    global FSLDIR logout comout CLUSTERRSH

    set out ${in}2$ref

    if { $existing_mats } {

	fsl:exec "${FSLDIR}/bin/flirt -ref $ref -in $in -out $out -applyxfm -init ${out}.mat -interp $interp"

    } else {

	if { $dof == 3 } {
	    set dof "6 -schedule ${FSLDIR}/etc/flirtsch/xyztrans.sch"
	}

	fsl:exec "${FSLDIR}/bin/flirt -ref $ref -in $in -out $out -omedx ${out}.xfm -omat ${out}.mat -cost corratio -dof $dof -searchrx -$search $search -searchry -$search $search -searchrz -$search $search -interp $interp" C

    }

    fsl:exec "${FSLDIR}/bin/convert_xfm -matonly -inverse -omat ${ref}2${in}.mat ${out}.mat"

    fsl:exec "${FSLDIR}/bin/slicer $out $ref -s 1 -x 0.35 sla -x 0.45 slb -x 0.55 slc -x 0.65 sld -y 0.35 sle -y 0.45 slf -y 0.55 slg -y 0.65 slh -z 0.35 sli -z 0.45 slj -z 0.55 slk -z 0.65 sll ; ${FSLDIR}/bin/convert -colors 100 +append sla slb slc sld sle slf slg slh sli slj slk sll ${out}.gif ; /bin/rm -f sla slb slc sld sle slf slg slh sli slj slk sll"

    fsl:exec "/bin/rm -f ${out}.hdr ${out}.img"

    puts $report "<hr><p>Registration of $in (underlying image) to $ref (red lines)<br><br>
<a href=\"${out}.mat\"><IMG BORDER=0 SRC=\"${out}.gif\"></a><br>"

    if { [ string compare $ref standard ] == 0 } {
	TalairachMedxTransform ${out}.xfm
    }
}

#}}}
#{{{ feat5:proc

proc feat5:proc { fsfroot } {

    #{{{ setup and load fsf

global FSLDIR PWD HOME HOSTNAME VERSION logout fmri feat_files unwarp_files initial_highres_files highres_files FD report ps rs logout comout
 
puts "
Started FEAT at [ exec date ]
"

if { [ string range $fsfroot 0 0 ] != "/" } {
    set fsfroot ${PWD}/$fsfroot
}

feat5:load -1 1 ${fsfroot}.fsf

#}}}

    #{{{ prepare higher-level data

if { $fmri(level) > 1 } {

    if { $fmri(analysis) != 4 } {

	#{{{ if inputtype=copes, fix feat_files

if { $fmri(inputtype) == 2 } {
    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	set copes($session) $feat_files($session)
	set feat_files($session) [ file dirname [ file dirname $feat_files($session) ] ]
    }
}	

#}}}
	#{{{ setup gfeat directory

if { $fmri(outputdir) != "" } {
    set featname [ file rootname $fmri(outputdir) ].gfeat
} else {
    set featname [ file rootname $feat_files(1) ].gfeat
}
set FD [ new_filename $featname ]
if { ! [ file writable [ file dirname $FD ] ] } {
    set FD [ new_filename ${HOME}/[ file tail $featname ] ]
}

fsl:exec "mkdir $FD"
fsl:exec "/bin/cp ${fsfroot}.fsf ${FD}/design.fsf"
cd $FD
fsl:exec "${FSLDIR}/bin/feat_model design"
set FD [ pwd ]
set gFD $FD

set fsfroot ${FD}/design

#}}}
	#{{{ write starts of log and report files

set thedate [ exec date ]

set logout [ open report.log "w" ]
set comout [ open report.com "w" ]
set greport [ open report.html "w" ]
fconfigure $logout -buffering none
fconfigure $comout -buffering none
fconfigure $greport -buffering none

set tailpid [ exec tail -f report.log & ]
set killpid [ exec sh -c "sleep 100000 ; kill -9 $tailpid; rm -f core" >& /dev/null & ]

puts $logout "Started higher-level FEAT at $thedate on $HOSTNAME"

puts $greport "<HTML>

<TITLE>Higher-Level FEAT</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg.jpg\">

<hr><CENTER>
<H1>Higher-Level FEAT Index</H1>
${FD}/report.html<br>
$thedate
</CENTER>
"

#}}}
	#{{{ start web page

puts $greport "<hr><H2>Input (previous-level) reports</H2>"

for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {
    puts $greport "${i} <A HREF=\"$feat_files($i)/report.html\">$feat_files($i)/report.html</A><br>
    "
}

puts $greport "<hr><H2>Higher-level reports</H2>"

#}}}
	#{{{ run featregapply on each input feat directory

    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {

	# fix old (<v5) FEAT directories
	if { [ file exists $feat_files($session)/example_func2highres.mat ] || [ file exists $feat_files($session)/example_func2standard.mat ] } {
	    fsl:exec "cd $feat_files($session) ; /bin/rm -f reg ; ln -s . reg"
	}

	fsl:exec "${FSLDIR}/bin/featregapply $feat_files($session)"
    }

#}}}
	#{{{ setup mask image

set mask_list ""

for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
    set mask_list "$mask_list $feat_files($session)/reg_standard/mask.hdr"
}

fsl:exec "${FSLDIR}/bin/avwmerge -t mask $mask_list"
fsl:exec "${FSLDIR}/bin/avwmaths mask -Tmin mask"

#}}}
	#{{{ setup background image

if { [ file exists $feat_files(1)/design.lev ] } {
    set fmri(bgimage) 4
}

set bg_list ""

switch $fmri(bgimage) {
    1 {
	for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	    if { [ file exists $feat_files($session)/reg_standard/reg/highres.hdr ] } {
		set bg_list "$bg_list $feat_files($session)/reg_standard/reg/highres.hdr"
	    }
	}
    }
    2 {
	for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	    if { [ file exists $feat_files($session)/reg_standard/reg/highres.hdr ] } {
		set bg_list "$bg_list $feat_files($session)/reg_standard/reg/highres.hdr"
		set session $fmri(multiple)
	    }
	}
    }
    3 {
	for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	    if { [ file exists $feat_files($session)/reg_standard/example_func.hdr ] } {
		set bg_list "$bg_list $feat_files($session)/reg_standard/example_func.hdr"
	    }
	}
    }
    4 {
	for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	    if { [ file exists $feat_files($session)/reg_standard/example_func.hdr ] } {
		set bg_list "$bg_list $feat_files($session)/reg_standard/example_func.hdr"
		set session $fmri(multiple)
	    }
	}
    }
}

if { [ string length $bg_list ] == 0 } {
    fsl:exec "/bin/cp $feat_files(1)/reg/standard.hdr bg_image.hdr"
    fsl:exec "/bin/cp $feat_files(1)/reg/standard.img bg_image.img"
} else {
    fsl:exec "${FSLDIR}/bin/avwmerge -t bg_image $bg_list"
    if { [ llength $bg_list ] > 1 } {
	fsl:exec "${FSLDIR}/bin/ip_16SI bg_image bg_image 10 -i 1000"
	fsl:exec "${FSLDIR}/bin/avwmaths bg_image -Tmean bg_image"
    }
}

#}}}
	if { $fmri(inputtype) == 1 } {
	    #{{{ create 4D input files from previous copes and extract contrastname

foreach cope [ lsort -dictionary [ glob -nocomplain $feat_files(1)/reg_standard/stats/cope*.hdr ] ] {

    set cope [ file tail $cope ]
    set cope_list ""
    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	if { [ file exists $feat_files($session)/reg_standard/stats/$cope ] } {
	    set cope_list "$cope_list $feat_files($session)/reg_standard/stats/$cope"
	} else {
	    puts "Error - not all input FEAT directories have the same set of contrasts!
(currently looking for $feat_files($session)/reg_standard/stats/$cope)"
	        exit 1
        }
    }
    fsl:exec "${FSLDIR}/bin/avwmerge -t $cope $cope_list"
    fsl:exec "${FSLDIR}/bin/avwmaths $cope -mas mask $cope"

    set contrastnumber [ string trimleft [ file rootname $cope ] cope ]
    set contrastname($contrastnumber) "[ feat5:getconname $feat_files(1) $contrastnumber ]"
}

#}}}
	    #{{{ create 4D varcope files from previous varcopes

foreach varcope [ lsort -dictionary [ glob -nocomplain $feat_files(1)/reg_standard/stats/varcope*.hdr ] ] {

    set varcope [ file tail $varcope ]
    set varcope_list ""
    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	set varcope_list "$varcope_list $feat_files($session)/reg_standard/stats/$varcope"
    }
    fsl:exec "${FSLDIR}/bin/avwmerge -t $varcope $varcope_list"
    fsl:exec "${FSLDIR}/bin/avwmaths $varcope -mas mask $varcope"
}

#}}}
	    #{{{ create 4D tdof files from previous tdofs

foreach tdof [ lsort -dictionary [ glob -nocomplain $feat_files(1)/reg_standard/stats/tdof*.hdr ] ] {

    set tdof [ file tail $tdof ]
    set tdof_list ""
    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
	set tdof_list "$tdof_list $feat_files($session)/reg_standard/stats/$tdof"
    }
    fsl:exec "${FSLDIR}/bin/avwmerge -t $tdof $tdof_list"
    fsl:exec "${FSLDIR}/bin/avwmaths $tdof -mas mask $tdof"
}

#}}}
	    #{{{ prepare feat_files()

set fmri(multiple) 0
foreach cope [ lsort -dictionary [ glob -nocomplain cope*.hdr ] ] {
    incr fmri(multiple) 1
    set feat_files($fmri(multiple)) ${FD}/$cope
    puts "$fmri(multiple) $feat_files($fmri(multiple))"
}

#}}}
	} else {
	    #{{{ setup

    set cope_list ""
    set varcope_list ""
    set dof_list ""

    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
        set copename [ file tail $copes($session) ]
        set copenumber [ file rootname [ string trimleft $copename "cope" ] ]

        set cope_list "$cope_list $feat_files($session)/reg_standard/stats/$copename"
        set varcope_list "$varcope_list $feat_files($session)/reg_standard/stats/var$copename"

	if { [ file exists $feat_files($session)/reg_standard/stats/tdof_t${copenumber}.hdr ] } {
	    set dof_list "$dof_list $feat_files($session)/reg_standard/stats/tdof_t${copenumber}"
	}
    }

    fsl:exec "${FSLDIR}/bin/avwmerge -t cope1 $cope_list"
    fsl:exec "${FSLDIR}/bin/avwmaths cope1 -mas mask cope1"

    fsl:exec "${FSLDIR}/bin/avwmerge -t varcope1 $varcope_list"
    fsl:exec "${FSLDIR}/bin/avwmaths varcope1 -mas mask varcope1"

    if { $dof_list != "" } {
	fsl:exec "${FSLDIR}/bin/avwmerge -t tdof_t1 $dof_list"
	fsl:exec "${FSLDIR}/bin/avwmaths tdof_t1 -mas mask tdof_t1"
    }

    set contrastname(1) ""

    set fmri(multiple) 1
    set feat_files($fmri(multiple)) ${FD}/cope1

#}}}
	}
	#{{{ kill logout reporting

after 2000

catch { exec kill -9 $tailpid $killpid } errmsg

#}}}

    } else {

	# NEVER CALLED!
	#{{{ setup poststats only for level 2

set new_multiple 0

for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
    foreach featsubdirs [ lsort -dictionary [ glob -nocomplain $feat_files($session)/*.feat ] ] {
	incr new_multiple 1
	set new_list($new_multiple) $featsubdirs
    }
}

set fmri(multiple) $new_multiple

puts "Re-running post-stats for FEAT sessions"
for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {
    set feat_files($session) $new_list($session)
    set contrastname($session) "[ exec sh -c "cat $feat_files($session)/design.lev" ]"
    puts "$feat_files($session) $contrastname($session)"
}

#}}}

    }

}

#}}}

    for { set session 1 } { $session <= $fmri(multiple) } { incr session 1 } {

	set FD $feat_files($session)
	set logout stdout

	#{{{ setup FEAT directory etc.

set thedate [ exec date ]

if { $fmri(analysis) == 0 || $fmri(analysis) == 4 } {

    if { $fmri(newdir_yn) } {
	set OldFD $feat_files($session)

	set FD [ new_filename $OldFD ]
    
	if { ! [ file writable [ file dirname $FD ] ] } {
	    set FD [ new_filename ${HOME}/[ file tail $OldFD ] ]
	}

	file copy $OldFD $FD
    }

    set BACKUPS [ new_filename ${FD}/old ]
    fsl:exec "/bin/mkdir $BACKUPS"

    fsl:exec "cd $FD ; /bin/mv design* $BACKUPS ; /bin/cp $BACKUPS/design.fsf ."

    if { $fmri(analysis) == 0 } {
	fsl:exec "cd $FD ; /bin/rm -f reg"
    }

    # fix old (<v5) FEAT directories
    if { [ file exists $FD/example_func2highres.mat ] || [ file exists $FD/example_func2standard.mat ] } {
	fsl:exec "cd $FD ; ln -s . reg"
    }

    if { $fmri(analysis) == 4 } {
	fsl:exec "cd $FD; /bin/mv report* ?endered_thresh_* $BACKUPS; /bin/rm -rf cluster_* lmax_* prob_mask_* thresh_* tsplot; cd stats; /bin/rm -f cope* neff* tstat* varcope* zstat* fstat* zfstat*"
    }

} else {

    if { $fmri(level) == 1 && $fmri(outputdir) != "" } {
	if { $fmri(multiple) == 1 } {
	    set featname [ file rootname $fmri(outputdir) ].feat
	} else {
	    set featname [ file rootname $feat_files($session) ]_[ file rootname [ file tail $fmri(outputdir) ] ].feat
	}
    } else {
	set featname [ file rootname $feat_files($session) ].feat
    }
    set FD [ new_filename $featname ]
    if { ! [ file writable [ file dirname $FD ] ] } {
	set FD [ new_filename ${HOME}/[ file tail $featname ] ]
    }

    fsl:exec "/bin/mkdir $FD"

    set funcdata [ file rootname $feat_files($session) ]
}

cd $FD
set FD [ pwd ]

fsl:exec "/bin/cp ${fsfroot}.fsf design.fsf"

#{{{ process relative filenames

if { $fmri(relative_yn) && $session > 1 } {

    set fix_design [ open design.fsf "a" ]

    set datarootdir [ file dirname $feat_files($session) ]

    for { set i 1 } { $i <= $fmri(evs_orig) } { incr i 1 } {
	if { $fmri(shape$i) > 2 } {
	    set fmri(custom$i) ${datarootdir}/[ file tail $fmri(custom$i)]
	    puts $fix_design "
# Custom EV file (EV $i)
set fmri(custom$i) \"$fmri(custom$i)\""
        }
    }

    close $fix_design
}

#}}}

set errmsg [ fsl:exec "${FSLDIR}/bin/feat_model design" ]
if { $errmsg != "" } {
    puts "Problem with processing the model: $errmsg"
}

if { $fmri(level) > 1 } {
    set levoutput [ open design.lev "w" ]
    puts $levoutput $contrastname($session)
    close $levoutput
}

set fsfroot ${FD}/design

#}}}

	if { $fmri(analysis) > 0 } {
	    #{{{ write starts of log and report files

set logout [ open report.log "w" ]
set comout [ open report.com "w" ]
set report [ open report.html "w" ]
fconfigure $logout -buffering none
fconfigure $comout -buffering none
fconfigure $report -buffering none

set tailpid [ exec tail -f report.log & ]
set killpid [ exec sh -c "sleep 100000 ; kill -9 $tailpid; rm -f core" >& /dev/null & ]

puts $logout "Started FEAT at $thedate on $HOSTNAME"

set conname ""
if { $fmri(level) > 1 } {
    set conname " for Lower-level Contrast $session"
    if { $contrastname($session) != "" } {
	set conname "$conname ($contrastname($session))"
    }
}

puts $report "<HTML>

<TITLE>FEAT Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg.jpg\">

<hr><CENTER>
<H1>FEAT Report$conname</H1>
${FD}/report.html<br>
$thedate
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Analysis was carried out using FEAT (FMRI Expert Analysis Tool) Version ${VERSION}, part of FSL (FMRIB's Software Library, www.fmrib.ox.ac.uk/fsl)."

set rs "<p><b>References</b><br>
"

#}}}

	    if { $fmri(level) == 1 } {
		#{{{ check npts, delete images, make example_func

if { $fmri(analysis) != 4 } {

    # check npts
    set total_volumes [ exec ${FSLDIR}/bin/avwnvols $funcdata 2> /dev/null ]
    puts $logout "Total original volumes = $total_volumes"
    if { $total_volumes != $fmri(npts) } {
	puts $logout "Error - $funcdata has a different number of time points to that in FEAT setup"
	return 1
    }

    # delete images
    if { $fmri(ndelete) > 0 } {
	puts $logout "Deleting $fmri(ndelete) volume(s) - BE WARNED for future analysis!"
	set total_volumes [ expr $total_volumes - $fmri(ndelete) ]
	fsl:exec "${FSLDIR}/bin/avwroi $funcdata prefiltered_func_data $fmri(ndelete) $total_volumes"
	set funcdata prefiltered_func_data
    }

    # choose halfway image and copy to example_func
    set target_vol_number [ expr $total_volumes / 2 ]
    fsl:exec "${FSLDIR}/bin/avwroi $funcdata example_func $target_vol_number 1"
}

#}}}
		#{{{ pre-stats

if { $fmri(analysis) != 4 } {
    if { $fmri(filtering_yn) } {

	set ps "$ps The following pre-statistics processing was applied"

	#{{{ slice timing correction

if { $fmri(st) > 0 } {

    set ps "$ps; slice-timing correction using Fourier-space time-series phase-shifting"

    set st_opts ""
    
    switch $fmri(st) {
	2 {
	    set st_opts "--down"
	}
	3 {
	    set st_opts "--ocustom=$fmri(st_file)"
	}
	4 {
	    set st_opts "--tcustom=$fmri(st_file)"
	}
    }

    fsl:exec "${FSLDIR}/bin/slicetimer -i $funcdata --out=prefiltered_func_data_st -r $fmri(tr) $st_opts"
    set funcdata prefiltered_func_data_st
}

#}}}
	#{{{ motion correction

#mc: 0=none 1=MCFLIRT

if { $fmri(mc) != 0 } {
    set ps "$ps; motion correction using MCFLIRT \[Jenkinson 2002\]"
    set rs "$rs\[Jenkinson 2002\] M. Jenkinson and P. Bannister and M. Brady and S. Smith. Improved optimisation for the robust and accurate linear registration and motion correction of brain images. NeuroImage (in press) 2002.<br>
    "

    fsl:exec "${FSLDIR}/bin/mcflirt -in $funcdata -out prefiltered_func_data_mcf -plots -refvol $target_vol_number -rmsrel -rmsabs" C
    set funcdata prefiltered_func_data_mcf

    fsl:exec "/bin/mkdir mc ; /bin/mv -f prefiltered_func_data_mcf.par prefiltered_func_data_mcf_abs.rms prefiltered_func_data_mcf_abs_mean.rms prefiltered_func_data_mcf_rel.rms prefiltered_func_data_mcf_rel_mean.rms mc"
    cd mc
    
    #{{{ make plots

fsl:exec "echo \"set term pbm color ; set size 1,0.3 ; set title 'MCFLIRT estimated rotations (radians)' ; plot 'prefiltered_func_data_mcf.par' using 1 title 'x' with lines , 'prefiltered_func_data_mcf.par' using 2 title 'y' with lines , 'prefiltered_func_data_mcf.par' using 3 title 'z' with lines\" | ${FSLDIR}/bin/gnuplot > rot.ppm"
fsl:exec "${FSLDIR}/bin/convert rot.ppm rot.gif"

fsl:exec "echo \"set term pbm color ; set size 1,0.3 ; set title 'MCFLIRT estimated translations (mm)' ; plot 'prefiltered_func_data_mcf.par' using 4 title 'x' with lines , 'prefiltered_func_data_mcf.par' using 5 title 'y' with lines , 'prefiltered_func_data_mcf.par' using 6 title 'z' with lines\" | ${FSLDIR}/bin/gnuplot > trans.ppm"
fsl:exec "${FSLDIR}/bin/convert trans.ppm trans.gif"

fsl:exec "echo \"set term pbm color ; set size 1,0.3 ; set title 'MCFLIRT estimated mean displacement (mm)' ; plot 'prefiltered_func_data_mcf_abs.rms' title 'absolute' with lines , 'prefiltered_func_data_mcf_rel.rms' title 'relative' with lines\" | ${FSLDIR}/bin/gnuplot > disp.ppm"
fsl:exec "${FSLDIR}/bin/convert disp.ppm disp.gif"

fsl:exec "/bin/rm -f rot.ppm trans.ppm disp.ppm"

#}}}
    #{{{ extract mean displacements

set mcchannel [ open prefiltered_func_data_mcf_abs_mean.rms "r" ]
gets $mcchannel line
scan $line "%f" absrms
set absrms [ expr int($absrms*100.0)/100.0 ]
close $mcchannel

set mcchannel [ open prefiltered_func_data_mcf_rel_mean.rms "r" ]
gets $mcchannel line
scan $line "%f" relrms
set relrms [ expr int($relrms*100.0)/100.0 ]
close $mcchannel

#}}}
    #{{{ web page report

set mcreport [ open index.html "w" ]

puts $mcreport "<HTML>

<TITLE>FEAT Motion Correction Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg.jpg\">

<hr><CENTER>
<H1>FEAT Motion Correction Report</H1>
for FEAT session &nbsp; &nbsp; <a href=\"../report.html\">${FD}</a><br>
[ exec date ]
<hr>

<p><IMG BORDER=0 SRC=\"rot.gif\">
<p><IMG BORDER=0 SRC=\"trans.gif\">
<p><IMG BORDER=0 SRC=\"disp.gif\">

<p>Mean (across voxels) voxel displacements:<br>
absolute (each time point with respect to the reference image) = ${absrms}mm;<br>
relative (each time point with respect to the previous timepoint) = ${relrms}mm.

</CENTER>
<HR><FONT SIZE=1>This page produced automatically by FEAT - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL</A>.</FONT>

</BODY></HTML>
"

close $mcreport

#}}}
    #{{{ main web report output

#set pixdimx [ expr abs([ exec $FSLDIR/bin/avwval example_func pixdim1 ]) ]
#set pixdimy [ expr abs([ exec $FSLDIR/bin/avwval example_func pixdim2 ]) ]
#set pixdimz [ expr abs([ exec $FSLDIR/bin/avwval example_func pixdim3 ]) ]
#set pixdim [ expr  pow($pixdimx * $pixdimy * $pixdimz,0.33333) ]
#
#set MC_TOLERANCE [ expr 0.5 * $pixdim / 5.0 ]

set MC_TOLERANCE 0.5

set mcwarning ""

if { $relrms > $MC_TOLERANCE } {
    set mcwarning " - warning - high levels of motion detected"
}

puts $report "<hr><a href=\"mc/index.html\">Motion correction report</a> (mean displacements: absolute=${absrms}mm, relative=${relrms}mm)$mcwarning"

#}}}

    cd ..
}

#}}}
	#{{{ BET

if { $fmri(bet_yn) } {
    set ps "$ps; non-brain removal using BET \[Smith 2002\]"
    set rs "$rs\[Smith 2002\] S. Smith. Fast Robust Automated Brain Extraction. Human Brain Mapping (in press) 2002.<br>
    "

    fsl:exec "${FSLDIR}/bin/betfunc $funcdata prefiltered_func_data_bet"
    set funcdata prefiltered_func_data_bet
}

#}}}
	#{{{ filtering

set DATATYPE ""
if { [ exec $FSLDIR/bin/avwval $funcdata datatype ] != 4 } {
    set DATATYPE "_32R"
}

set thecommand "${FSLDIR}/bin/ip$DATATYPE ${funcdata} filtered_func_data $fmri(brain_thresh)"

#{{{ Spatial filtering

if { $fmri(smooth) > 0.01 } {
    set thecommand "$thecommand -s [ expr $fmri(smooth) / 2.355 ]"
    set ps "$ps; spatial smoothing using a Gaussian kernel of FWHM $fmri(smooth)mm"
}

#}}}
#{{{ Masking and global intensity normalization

set thecommand "$thecommand -m mask"

set global_mean 10000

if { $fmri(brain_thresh) == 0 } {
    puts $logout "
Warning - using low global_mean (1000) because thresholding set to 0.
This must be also the case for all other subjects in a group analysis.
"
    set global_mean 1000
}

if { $fmri(norm_yn)} {
    set thecommand "$thecommand -i $global_mean"
    set ps "$ps; global (volumetric) multiplicative mean intensity renormalization"
} else {
    set thecommand "$thecommand -I $global_mean"
    set ps "$ps; mean-based intensity normalisation of all volumes by the same factor"
}

#}}}
#{{{ Temporal filtering

if { $fmri(temphp_yn) || $fmri(templp_yn) } {

    set hp_sigma_vol -1
    if { $fmri(temphp_yn) } {
	set hp_sigma_sec [ expr $fmri(paradigm_hp) / 2.0 ]
	set hp_sigma_vol [ expr $hp_sigma_sec / $fmri(tr) ]
	set ps "$ps; highpass temporal filtering (Gaussian-weighted LSF straight line fitting, with sigma=${hp_sigma_sec}s)"
    }

    set lp_sigma_vol -1
    if { $fmri(templp_yn) } {
	set lp_sigma_sec 2.8
	set lp_sigma_vol [ expr $lp_sigma_sec / $fmri(tr) ]
	set ps "$ps; Gaussian lowpass temporal filtering HWHM ${lp_sigma_sec}s"
    }

    set thecommand "$thecommand -t $hp_sigma_vol $lp_sigma_vol"

}

#}}}
#{{{ Call ip

set ps "$ps."

set absbrainthresh [ fsl:exec "$thecommand | grep THRESHOLD | awk '{ print \$2 }' -" ]

#}}}

#}}}

    } else {
	exec cp ${funcdata}.hdr filtered_func_data.hdr
	exec cp ${funcdata}.img filtered_func_data.img
    }
}

#{{{ make binary mask if it hasn't already been made

if { ! [ file exists mask.hdr ] } {
    set absbrainthresh [ fsl:exec "${FSLDIR}/bin/ip ${funcdata} prefiltered_func_data $fmri(brain_thresh) -m mask | grep THRESHOLD | awk '{ print \$2 }' -" ]
}

#}}}
#{{{ set absbrainthresh if brain_thresh==0

if { $fmri(brain_thresh) == 0 } {
    set absbrainthresh [ fsl:exec "$FSLDIR/bin/avwstats ${funcdata} -R | awk '{ print \$1 }' -" ]
} 

#}}}

fsl:exec "rm -rf prefiltered_func_data*"

#}}}
	    } else {
		#{{{ make symlinks for higher-level files

if { $fmri(analysis) != 4 } {
    feat5:link ../bg_image example_func
    feat5:link ../mask mask
    feat5:link ../[ file tail ${funcdata} ] filtered_func_data
    feat5:link ../var[ file tail ${funcdata} ] var_filtered_func_data
    feat5:link ../tdof_t[ string trimleft [ file tail ${funcdata} ] "cope" ] tdof_filtered_func_data
}

set absbrainthresh -1e10

#}}}
	    }

	    #{{{ stats

if { $fmri(stats_yn) || $fmri(analysis) == 4 } {

    if { $fmri(analysis) != 4 } {

	if { $fmri(level) == 1 } {

	    set film_opts "-sa -ms 5 -sp $FSLDIR/bin/susan_smooth"
	    set film_text " with local autocorrelation correction"

	    if { ! $fmri(prewhiten_yn) } {
		set film_opts "-noest" 
		set film_text ""
	    }

	    fsl:exec "$FSLDIR/bin/film_gls -rn stats $film_opts filtered_func_data design.mat $absbrainthresh" C

	    set ps "$ps Time-series statistical analysis was carried out using FILM (FMRIB's Improved Linear Model)$film_text \[Woolrich 2001\]."
	    set rs "$rs\[Woolrich 2001\] M.W. Woolrich, B.D. Ripley, J.M. Brady and S.M. Smith. Temporal Autocorrelation in Univariate Linear Modelling of FMRI Data. NeuroImage 14:6(1370-1386) 2001.<br>
"
	} else {

	    set FTESTS ""
	    if { [ file exists design.fts ] } {
		set FTESTS "--fc=design.fts"
	    }

	    set zthresh 1.5
	    if { $fmri(thresh) == 3 } {
		set zthresh [ expr $fmri(z_thresh) * 0.85 ]
	    }
	    if { ! $fmri(mixed_yn) } {
		set zthresh 100000
	    }

	    set DOFS ""
	    if { [ file exists tdof_filtered_func_data.hdr ] } {
		set DOFS "--dvc=tdof_filtered_func_data"
	    }

	    fsl:exec "$FSLDIR/bin/flame --cope=filtered_func_data --vc=var_filtered_func_data $DOFS --mask=mask --ld=stats --dm=design.mat --cs=design.grp --tc=design.con --zt=$zthresh $FTESTS" C

	    fsl:exec "echo [ expr [ fsl:exec "grep NumPoints design.mat | awk '{ print \$2 }' -" ] - [ fsl:exec "grep NumWaves design.mat | awk '{ print \$2 }' -" ] ] > stats/dof"

	    set ps "$ps Higher-level analysis was carried out using FLAME (FMRIB's Local Analysis of Mixed Effects)."
	}

	if { [ file exists stats/res4d.hdr ] } {
	    fsl:exec "$FSLDIR/bin/smoothest -d [ exec cat stats/dof ] -m mask -r stats/res4d > stats/smoothness"
	    fsl:exec "rm -f stats/res4d*"
	}
    }

    if { $fmri(level) == 1 } {
	set FTESTS ""
	if { [ file exists design.fts ] } {
	    set FTESTS "-f design.fts"
	}
	fsl:exec "$FSLDIR/bin/contrast_mgr $FTESTS stats design.con"
    }

}

#}}}
	    #{{{ poststats
		
if { $fmri(poststats_yn) } {
    feat5:poststats 0 [ expr $fmri(level) - 1 ]
}

#}}}
	    #{{{ time series plots and report output for each contrast

if { $fmri(stats_yn) || $fmri(poststats_yn) } {

    set fmrifile filtered_func_data
    if { ! [ file exists ${fmrifile}.hdr ] } {
	set fmrifile [ string trimright [ file root ${FD} ] + ]
    }

    fsl:exec "${FSLDIR}/bin/tsplot . -f $fmrifile"

    set tsplot_list "[ lsort -dictionary [ glob -nocomplain tsplot_zstat*.txt ] ] [ lsort -dictionary [ glob -nocomplain tsplot_zfstat*.txt ] ]"

    if { [ string length $tsplot_list ] > 0 } {

	puts $report "<hr><H2>Time series plots</H2><p>"

	fsl:exec "mkdir grot; mv *tsplot* grot; mv grot tsplot"
	cd tsplot

	foreach tsplot $tsplot_list {

	    regsub tsplot_ $tsplot "" tmptmp
	    regsub .txt $tmptmp "" tsplot_rootname

	    puts $report "<a href=\"tsplot/${tsplot_rootname}.html\"><IMG BORDER=0 SRC=\"tsplot/tsplot_${tsplot_rootname}.gif\"></a><br>
	    "

	    #{{{ web page

set tsplot_report [ open ${tsplot_rootname}.html "w" ]

puts $tsplot_report "<HTML>

<TITLE>$tsplot_rootname</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg.jpg\">

<hr><CENTER>
<H1>FEAT Time Series Report $tsplot_rootname</H1>
for FEAT session &nbsp; &nbsp; <a href=\"../report.html\">${FD}</a><br>
[ exec date ]
</CENTER><hr><b>Full plots</b><p>

Full model fit - peak voxel<br>
<a href=\"tsplot_${tsplot_rootname}.txt\"><IMG BORDER=0 SRC=\"tsplot_${tsplot_rootname}.gif\"></a><br><br>
"

if { [ file exists tsplot_${tsplot_rootname}p.gif ] } {
    puts $tsplot_report "Partial model fit - peak voxel<br>
<a href=\"tsplot_${tsplot_rootname}.txt\"><IMG BORDER=0 SRC=\"tsplot_${tsplot_rootname}p.gif\"></a><br><br>
"
}

if { [ file exists tsplotc_${tsplot_rootname}.gif ] } {
    puts $tsplot_report "Full model fit - averaged over all active voxels, weighted by Z-stat values<br>
<a href=\"tsplotc_${tsplot_rootname}.txt\"><IMG BORDER=0 SRC=\"tsplotc_${tsplot_rootname}.gif\"></a><br><br>
"
}

if { [ file exists tsplotc_${tsplot_rootname}p.gif ] } {
    puts $tsplot_report "Partial model fit - averaged over all active voxels, weighted by Z-stat values<br>
<a href=\"tsplotc_${tsplot_rootname}.txt\"><IMG BORDER=0 SRC=\"tsplotc_${tsplot_rootname}p.gif\"></a><br><br>
"
}

set ps_list [ lsort -dictionary [ glob -nocomplain ps_tsplot_${tsplot_rootname}_ev*.gif ] ]

if { [ string length $ps_list ] > 0 } {

    puts $tsplot_report "<p><hr><b>Peri-stimulus plots</b><p><table>
"

    foreach ps_item $ps_list {
	if { [ string first p.gif $ps_item ] == -1 } {
	    regsub .gif $ps_item "" ps_root
	    puts $tsplot_report "<tr><td><a href=\"${ps_root}.txt\"><IMG BORDER=0 SRC=\"${ps_root}.gif\"></a>"
	    if { [ file exists ${ps_root}p.gif ] } {
		puts $tsplot_report "<td><a href=\"${ps_root}.txt\"><IMG BORDER=0 SRC=\"${ps_root}p.gif\"></a>"
	    }
	    puts $tsplot_report "</tr>"
	}
    }

    puts $tsplot_report "</table>"
}

puts $tsplot_report "

<HR><FONT SIZE=1>This page produced automatically by FEAT - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL</A>.</FONT>

</BODY></HTML>
"

close $tsplot_report

#}}}

	}

	cd ..
    }
}

#}}}
	    #{{{ final outputs

set feat_files($session) $FD

if { $fmri(level) == 1 } {
    puts $report "<hr><b>Registration</b> (if carried out) can be viewed <a href=\"reg/index.html\">here</a>.
"
}

puts $report "

$ps
<!--featregps-->

$rs
<!--featregrs-->

"

if { [ file exists design.mat ] } {
    puts $report "<p><p><b>Design matrix</b><br><a href=\"design.mat\"><IMG BORDER=0 SRC=\"design.gif\"></a>
    "

    if { [ file exists design_cov.gif ] } {
	puts $report "<p><p><b>Covariance matrix</b><br><IMG BORDER=0 SRC=\"design_cov.gif\">
	"
    }
}

puts $report "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL</A>.</FONT>

</BODY></HTML>
"

close $report

#}}}
	}

	if { $fmri(level) == 1 } {
	    #{{{ registration and re-thresholding

#set tmpchannel [ open ${fsfroot}.fsf "a" ]
#puts $tmpchannel "
## session number
#set fmri(session) $session"
#close $tmpchannel

if { $fmri(regunwarp_yn) || $fmri(reginitial_highres_yn) || $fmri(reghighres_yn) || $fmri(regstandard_yn) } {

    set existing_mats 0

    #{{{ setup

if { ! [ file exists reg ] } {
    fsl:exec "/bin/mkdir reg"
}

cd reg

feat5:link ../example_func example_func

set reg_report [ open index.html "w" ]
fconfigure $reg_report -buffering none

puts $reg_report "<HTML>

<TITLE>FEAT Registration Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg.jpg\">

<hr><CENTER>
<H1>FEAT Registration Report</H1>
for FEAT session &nbsp; &nbsp; <a href=\"../report.html\">${FD}</a><br>
[ exec date ]
</CENTER>
"

set reg_ps "Registration to high resolution and/or standard images was carried out using FLIRT \[Jenkinson 2001, 2002\]."
set reg_rs "\[Jenkinson 2001\] M. Jenkinson and S.M. Smith. A Global Optimisation Method for Robust Affine Registration of Brain Images. Medical Image Analysis 2001:5(2)(143-156).<br>
\[Jenkinson 2002\] M. Jenkinson, P. Bannister, M. Brady and S. Smith. Improved Optimisation for the Robust and Accurate Linear Registration and Motion Correction of Brain Images. Neuroimage 2002: in publication.<br>
"

#}}}
    #{{{ B0 unwarping

set ef example_func

if { $fmri(regunwarp_yn) } {
    if { [ info exists unwarp_files($session) ] } {
	feat5:link [ file rootname $unwarp_files($session) ] phasemap
    } else { 
	if { ! [ file exists phasemap.hdr ] } {
	    puts $logout "Warning - registration to unwarp turned on but
no unwarp phasemap image specified in setup file or in
FEAT directory! Will not unwarp."
	    set fmri(regunwarp_yn) 0
	}
    }
}

if { $fmri(regunwarp_yn) } {

    # Generating raw phase map 
    fsl:exec "${FSLDIR}/bin/prelude -c phasemap -u phasemap_unwrapped"

    # Generating regularised shift map
    fsl:exec "${FSLDIR}/bin/fugue -p phasemap_unwrapped -d $fmri(dwellasym) --smooth2=1.0 --saveshift=b0_shift"

    # Generate warped skull-stripped "structural"
    fsl:exec "${FSLDIR}/bin/avwcomplex -realabs phasemap phase_struct 0 0"
    fsl:exec "${FSLDIR}/bin/bet phase_struct phase_struct_brain"
    fsl:exec "${FSLDIR}/bin/fugue -i phase_struct_brain -w phase_struct_brain_warped --loadshift=b0_shift --nofill"

    # Register EPI to this
    fsl:exec "${FSLDIR}/bin/flirt -dof 6 -in example_func -ref phase_struct_brain_warped -omat example_func2phase_struct_brain_warped.mat"

    # Apply unwarping to registered EPI (the following applies the affine and warp with a single interpolation)
    fsl:exec "${FSLDIR}/bin/convertwarp -o example_func2example_func_unwarped --premat=example_func2phase_struct_brain_warped.mat --shiftmap=b0_shift"
    fsl:exec "${FSLDIR}/bin/applywarp -i example_func -o example_func_unwarped -w example_func2example_func_unwarped"

    set ef example_func_unwarped
}

#}}}
    #{{{ setup flirt files

if { $fmri(reginitial_highres_yn) } {
    if { [ info exists initial_highres_files($session) ] } {
	feat5:link [ file rootname $initial_highres_files($session) ] initial_highres
    } else { 
	if { ! [ file exists initial_highres.hdr ] } {
	    puts $logout "Warning - registration to initial_highres turned on but
no initial_highres image specified in setup file or in
FEAT directory! Will not register to initial_highres."
	    set fmri(reginitial_highres_yn) 0
	}
    }
}

if { $fmri(reghighres_yn) } {
    if { [ info exists highres_files($session) ] } {
	feat5:link [ file rootname $highres_files($session) ] highres
    } else { 
	if { ! [ file exists highres.hdr ] } {
	    puts $logout "Warning - registration to highres turned on but
no highres image specified in setup file or in
FEAT directory! Will not register to highres."
	    set fmri(reghighres_yn) 0
	}
    }
}

if { $fmri(regstandard_yn) } {
    feat5:link [ file rootname $fmri(regstandard) ] standard
}

#}}}
    #{{{ -> highres

if { $fmri(reghighres_yn) } {

    if { $fmri(reginitial_highres_yn) } {

	feat5:flirt $ef initial_highres $fmri(reginitial_highres_dof) $fmri(reginitial_highres_search) trilinear $existing_mats $reg_report

	feat5:flirt initial_highres highres $fmri(reghighres_dof) $fmri(reghighres_search) trilinear $existing_mats $reg_report

	fsl:exec "${FSLDIR}/bin/convert_xfm -ref highres -in $ef -omedx ${ef}2highres.xfm -omat ${ef}2highres.mat -concat initial_highres2highres.mat -middlevol initial_highres ${ef}2initial_highres.mat"

        feat5:flirt $ef highres 0 0 trilinear 1 $reg_report

    } else {

	feat5:flirt $ef highres $fmri(reghighres_dof) $fmri(reghighres_search) trilinear $existing_mats $reg_report

    }
}

#}}}
    #{{{ -> standard

if { $fmri(regstandard_yn) } {

    if { $fmri(reghighres_yn) } {

	feat5:flirt highres standard $fmri(regstandard_dof) $fmri(regstandard_search) trilinear $existing_mats $reg_report

	fsl:exec "${FSLDIR}/bin/convert_xfm -ref standard -in $ef -omedx ${ef}2standard.xfm -omat ${ef}2standard.mat -concat highres2standard.mat -middlevol highres ${ef}2highres.mat"
	TalairachMedxTransform ${ef}2standard.xfm

        feat5:flirt $ef standard 0 0 trilinear 1 $reg_report

    } else {

	feat5:flirt $ef standard $fmri(regstandard_dof) $fmri(regstandard_search) trilinear $existing_mats $reg_report

    }
}

#}}}
    #{{{ put biblio stuff into original report

cd ..

set iptr [ open report.html r ]
set optr [ open tmpreport.html w ]

while { ( [ gets $iptr line ] >= 0 ) } {

    if { [ regexp "<!--featregpsstart-->" $line ] } {
	while { ( [ gets $iptr line ] >= 0 ) && ! [ regexp "<!--featregps-->" $line ] } {
	}
    }

    if { [ regexp "<!--featregps-->" $line ] } {
        puts $optr "<!--featregpsstart-->
$reg_ps
"
    }

    if { [ regexp "<!--featregrsstart-->" $line ] } {
	while { ( [ gets $iptr line ] >= 0 ) && ! [ regexp "<!--featregrs-->" $line ] } {
	}
    }

    if { [ regexp "<!--featregrs-->" $line ] } {
        puts $optr "<!--featregrsstart-->
$reg_rs
"
    }

    puts $optr $line
}

close $iptr
close $optr

exec mv -f tmpreport.html report.html

#}}}
    #{{{ timing and final outputs

puts $reg_report "<HR><FONT SIZE=1>This page produced automatically by FEAT - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL</A>.</FONT>

</BODY></HTML>
"

puts $logout "Finished registration at [ exec date ]"

close $reg_report

#}}}
}

if { [ file exists ${FD}/reg/example_func2standard.mat ] } {
    feat5:poststats 1 0
}

#}}}
	}

	#{{{ final logout report and finish up

if { $fmri(level) > 1 && $fmri(analysis) != 4 } {
    set conname ""
    if { $contrastname($session) != "" } {
	set conname "($contrastname($session))"
    }

    puts $greport "<a href=\"[ file tail $FD ]/report.html\">Lower-level contrast $session $conname</a><br>
"
}

puts $logout "Finished FEAT at [ exec date ]
To view the FEAT report point your web browser at
${FD}/report.html
"

after 2000

catch { exec kill -9 $tailpid $killpid } errmsg

#}}}
    }

    #{{{ final level 2 outputs

if { $fmri(level) > 1 && $fmri(analysis) != 4 } {

    if { [ file exists design.mat ] } {
	puts $greport "<hr><p><p><b>Design matrix</b><br><a href=\"design.mat\"><IMG BORDER=0 SRC=\"design.gif\"></a>
"

        if { [ file exists design_cov.gif ] } {
	    puts $greport "<p><p><b>Covariance matrix</b><br><IMG BORDER=0 SRC=\"design_cov.gif\">
"
        }
    }

    puts $greport "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL</A>.</FONT>

</BODY></HTML>
"

    close $greport

    puts "Finished higher-level FEAT at [ exec date ]
To view the FEAT report point your web browser at
${gFD}/report.html
"

}

#}}}
}

#}}}


#{{{ call GUI 

if { ! [ info exists INGUI ] } {
    wm withdraw .
    feat5 .r
    tkwait window .r
}

#}}}

